'use strict';

var _Constants = require('./Constants');

var _CastSubsonic = require('./CastSubsonic');

var _CastSubsonic2 = _interopRequireDefault(_CastSubsonic);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Callback when media is loaded in local player
 */
_CastSubsonic2.default.prototype.onMediaLoadedLocally = function (evt) {
  var localPlayer = document.getElementById('media_element');
  console.log('onMediaLoadedLocally', evt.target, localPlayer);
  this.playerHandler.loaded();
};

/**
 * Set the PlayerHandler target to use the video-element player
 */
_CastSubsonic2.default.prototype.setupLocalPlayer = function () {
  var playerTarget = this.playerTarget = {};
  var csThis = this;

  var localPlayer = document.getElementById('media_element');

  if (!this.initializedLocalPlayer) {
    localPlayer.addEventListener('loadeddata', this.onMediaLoadedLocally.bind(this));
    // localPlayer.addEventListener("canplay", this.onMediaLoadedLocally.bind(this));

    localPlayer.addEventListener('pause', function (evt) {
      console.log('event paused', evt);
      this.playerHandler.pause(this.autoplay);
    }.bind(this));

    localPlayer.addEventListener('playing', function () {
      console.log('event play');
      this.playerHandler.play();
    }.bind(this));

    localPlayer.addEventListener('ended', function () {
      console.log('ended');
      this.endPlayback();
    }.bind(this));
  }

  this.hackCheck = function () {
    if (csThis.playerState === _Constants.PLAYER_STATE.PLAYING) {
      if (csThis.currentMediaTime === 0 && csThis.timeSinceRestart > 1000) {
        console.warn('had to hack', csThis.currentMediaTime, csThis.currentMediaDuration, csThis.timeSinceRestart);
        csThis.playerHandler.seekTo(csThis.timeSinceRestart / 1000);
      }
      csThis.timeSinceRestart += _Constants.TIMER_STEP;
      csThis.currentMediaTime += _Constants.TIMER_STEP / 1000;
    }
  };

  playerTarget.play = function () {
    console.log('local play');
    var that = this;
    var playResult = localPlayer.play();
    if (playResult && playResult.then) {
      playResult.then(function (res) {
        console.log('playing, really!');
        that.attemptedPause = false;
      }).catch(function (err) {
        console.error(err);
        // HACK: next?
        if (that.attemptedPause) {
          that.attemptedPause = false;
          return;
        }
        if (window.castSubsonic) {
          window.castSubsonic.endPlayback();
        }
      });
    } else {
      console.log('ie/edge playing, really!');
      that.attemptedPause = false;
    }
  };

  playerTarget.pause = function () {
    console.log('local pause');
    localPlayer.pause();
    this.attemptedPause = true;
  };

  playerTarget.stop = function () {
    localPlayer.stop();
  };

  playerTarget.load = function (media) {
    console.log('localPlayer load');
    localPlayer.src = media.src;
    localPlayer.load();
  };

  playerTarget.getCurrentMediaTime = function () {
    return localPlayer.currentTime;
  };

  playerTarget.getMediaDuration = function () {
    return localPlayer.duration;
  };

  playerTarget.mute = function () {
    localPlayer.muted = true;
  };

  playerTarget.unMute = function () {
    localPlayer.muted = false;
  };

  playerTarget.isMuted = function () {
    return localPlayer.muted;
  };

  playerTarget.seekTo = function (time) {
    localPlayer.currentTime = time;
  };

  this.playerHandler.setTarget(playerTarget);

  // this.playerHandler.setVolume(DEFAULT_VOLUME * FULL_VOLUME_HEIGHT);
  //
  // this.showFullscreenButton();

  if (this.currentMediaTime > 0) {
    console.log('hack skip');
    this.playerHandler.play();
    // should this be seek to?
  }

  this.initializedLocalPlayer = true;
};