'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SubsonicCache = exports.SubsonicFetch = exports.md5 = exports.URI = undefined;

var _URI = require('../../utils/URI');

var _URI2 = _interopRequireDefault(_URI);

var _md = require('../../utils/md5');

var _md2 = _interopRequireDefault(_md);

var _utils = require('../../utils/utils');

var _SubsonicCache = require('./SubsonicCache');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// HACK make global for now...

// export default {versionCompare: versionCompare, getRandomIntInclusive: getRandomIntInclusive, arrayShuffle: arrayShuffle, toHHMMSS: toHHMMSS, arrayRemove: arrayRemove, arrayFlatten: arrayFlatten };
/*
(The MIT License)

Copyright (c) 2014-2015 iReal

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the 'Software'), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
// jshint -W116, -W016, -W098
/* eslint no-unused-vars: 0 */
window.toHHMMSS = _utils.toHHMMSS;

var SubsonicFetch = {
  serverAPIVersion: '1.13.0',
  _u: '',
  _s: '',
  _p: '',
  _b: '0',
  _c: 'SubFireMobile',
  coverArtCache: {},

  _buildURI: function _buildURI(method, params) {
    var uri = this._s + '/rest/' + method + '.view',
        that = this;
    uri = (0, _URI2.default)(uri);
    uri.addQuery(params || {});
    uri.addQuery({
      v: that.serverAPIVersion,
      f: 'json',
      c: that._c,
      u: that._u
    });
    if (that._p && (0, _utils.versionCompare)(that.serverAPIVersion, '1.13.0') >= 0) {
      var s = Math.random().toString(36).replace(/[^a-z]+/g, '');
      var t = (0, _md2.default)(that._p + s);
      uri.addQuery({
        s: s,
        t: t
      });
    } else {
      // currently unsupported
      that._encP = 'enc:' + that._p.hexEncode();
      uri.addQuery({
        p: that._encP
      });
    }
    // console.log(params);
    // uri = uri + $.param(params, traditional);
    return uri.toString();
  },

  // I hate that i'm having to add this last
  _execute: function _execute(method, params) {
    var that = this;
    return new Promise(function (rs, rj) {
      if (that.connected === false) {
        rj({
          'subsonic-response': {
            status: 'failed',
            error: {
              code: '40',
              message: 'INTERNAL Wrong username or password'
            }
          }
        });
      }

      var uri = that._buildURI(method, params);
      // console.log(uri);
      fetch(uri.toString()).then(function (res) {
        if (!res.ok) {
          rj(res);
          return;
        }
        return res.text();
      }).then(function (res) {
        // console.warn(res);
        res = JSON.parse(res);
        var reply = res['subsonic-response'];
        // ampache returns "ok" but with error...
        if (reply.status === 'failed' || reply.error) {
          rj(reply);
        } else {
          rs(reply);
        }
      }).catch(function (err) {
        console.error(err);
        rj(err);
      });
    });
  },

  open: function open(server, username, password, bitrate, ping) {
    this.connected = null;
    this.coverArtCache = {};

    // reset capabilities
    this.serverAPIVersion = '1.13.0';
    this._u = username;
    this._s = server;
    this._b = bitrate || '0';
    this._p = password || '';
    this._c = window.subsonicClientName;
    if (!ping) return this;

    var that = this;
    return new Promise(function (rs, rj) {
      that.ping().then(function (res) {
        that.serverAPIVersion = res.version;
        that.connected = true;
        rs(res);
      }).catch(function (err) {
        console.warn('catch api check', err);
        if (err && err.error && err.error.code === 41) {
          that.serverAPIVersion = err.version;
          // console.warn('ping again', that._u, that._p);
          return that.ping();
        }
        that.connected = false;
        console.warn('reject');
        rj(err);
        return null;
      }).then(function (res2) {
        // console.warn('backup try', res2);
        if (res2) {
          that.connected = true;
          rs(res2);
        }
      }).catch(function (err) {
        // second ping failed, we really are invalid even on ampache.
        rj(err);
      });
    });
  },

  ping: function ping() {
    var that = this;
    // console.log('ping', that._u, that._p);
    return this._execute('ping');
  },
  getNowPlaying: function getNowPlaying() {
    return this._execute('getNowPlaying');
  },
  getPlaylists: function getPlaylists() {
    return this._execute('getPlaylists');
  },
  getPlaylist: function getPlaylist(id) {
    return this._execute('getPlaylist', {
      id: id
    });
  },
  createPlaylist: function createPlaylist(name, id) {
    var that = this;
    return new Promise(function (rs, rj) {
      var params = { name: name };
      if (id) params.playlistId = id;
      return that._execute('createPlaylist', params).then(function (data) {
        console.log('new playlist', data);
        if (data.playlist) {
          rs(data.playlist);
          return 'got one...must be 6.0b2+';
        }
        // 6.0b2: data will have the playlist so no need to do the rest of this, just rs()
        return that.getPlaylists();
      }).then(function (res) {
        console.log(res);
        if (res === 'got one...must be 6.0b2+') return;
        var p = res.playlists.playlist,
            i,
            l = p.length,
            m = id ? null : p[0];
        for (i = 1; i < p.length; ++i) {
          if (id) {
            m = p[i].id === id ? p[i] : m;
          } else {
            if (p[i].name === name && p[i].id * 1 > m.id * 1 && p[i].owner === that._u) {
              m = p[i];
            }
          }
        }
        rs(m);
      }).catch(function (err) {
        rj(err);
      });
    });
  },
  updatePlaylist: function updatePlaylist(id, comment, share, name, songsToAdd, indexesToRemove) {
    var params = {
      playlistId: id
    };
    if (comment) params.comment = comment;
    if (share === true || share === false) params.public = share;
    if (name) params.name = name;
    if (songsToAdd) params.songIdToAdd = songsToAdd;
    if (indexesToRemove) params.songIndexToRemove = songsToAdd;
    return this._execute('updatePlaylist', params);
  },
  deletePlaylist: function deletePlaylist(id) {
    return this._execute('deletePlaylist', { id: id });
  },
  getMusicFolders: function getMusicFolders() {
    return this._execute('getMusicFolders');
  },
  getIndexes: function getIndexes(id) {
    var params = id === null || id === undefined ? {} : {
      musicFolderId: id
    };
    return this._execute('getIndexes', params);
  },
  getMusicDirectory: function getMusicDirectory(id) {
    return this._execute('getMusicDirectory', {
      id: id
    });
  },
  getGenres: function getGenres() {
    return this._execute('getGenres');
  },
  getStarred: function getStarred() {
    return this._execute('getStarred');
  },
  getStarred2: function getStarred2() {
    return this._execute('getStarred2');
  },
  getArtists: function getArtists(musicFolderId) {
    var params = musicFolderId && musicFolderId !== '-1' ? { musicFolderId: musicFolderId } : {};
    return this._execute('getArtists', params);
  },
  getArtist: function getArtist(id) {
    return this._execute('getArtist', {
      id: id
    });
  },
  getTopSongs: function getTopSongs(name, count) {
    return this._execute('getTopSongs', {
      artist: name,
      count: count || 50
    });
  },
  getAlbum: function getAlbum(id) {
    return this._execute('getAlbum', {
      id: id
    });
  },
  getSong: function getSong(id) {
    return this._execute('getSong', {
      id: id
    });
  },
  getVideos: function getVideos() {
    return this._execute('getVideos');
  },
  getAlbumList: function getAlbumList(type, params) {
    params = params || {};
    params.type = type;
    return this._execute('getAlbumList', params);
  },
  getAlbumList2: function getAlbumList2(type, params) {
    params = params || {};
    params.type = type;
    return this._execute('getAlbumList2', params);
  },
  getRandomSongs: function getRandomSongs(genre, fromYear, toYear, id, size) {
    var params = {};
    if (id) params.musicFolderId = id;
    if (genre) params.genre = genre;
    if (fromYear) params.fromYear = fromYear;
    if (toYear) params.toYear = toYear;
    params.size = size || 50;
    return this._execute('getRandomSongs', params);
  },
  getAvatarURL: function getAvatarURL(username) {
    username = username || this._u;
    var key = 'av-' + username,
        url;
    var firstUrl = this.coverArtCache[key];
    if (!firstUrl) {
      url = this._buildURI('getAvatar', { username: username });
      this.coverArtCache[key] = url;
    } else {
      // console.log("match", "using", firstUrl);
      url = firstUrl;
    }
    return url;
  },
  getCoverArtURL: function getCoverArtURL(id, size) {
    var params = {
      id: id || -1
    };
    if (size) params.size = size;
    var key = id + '--' + size; // trimImageUrlEssentials(url);
    var firstUrl = this.coverArtCache[key];
    if (!firstUrl) {
      var url = this._buildURI('getCoverArt', params);
      this.coverArtCache[key] = url;
    } else {
      // console.log("match", "using", firstUrl);
      url = firstUrl;
    }
    return url;
  },
  getHLSURL: function getHLSURL(id, params) {
    params = Object.assign({
      id: id,
      maxBitRate: this._b
    }, params || {});
    return this._buildURI('hls', params);
  },
  getStreamingURL: function getStreamingURL(id, params) {
    params = Object.assign({
      id: id,
      maxBitRate: this._b
    }, params || {});
    return this._buildURI('stream', params);
  },
  getDownloadURL: function getDownloadURL(id) {
    return this._buildURI('download', {
      id: id
    });
  },
  getArtistInfo2: function getArtistInfo2(id, similarArtistCount) {
    return this.getArtistInfo(id, true, similarArtistCount);
  },
  getArtistInfo: function getArtistInfo(id, useID3) {
    var similarArtistCount = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 20;

    var method = useID3 ? 'getArtistInfo2' : 'getArtistInfo';
    var params = {
      id: id,
      count: similarArtistCount
    };
    return this._execute(method, params);
  },
  getSimilarSongs2: function getSimilarSongs2(id, count) {
    return this.getSimilarSongs(id, true, count);
  },
  getSimilarSongs: function getSimilarSongs(id, useID3, count) {
    var method = useID3 ? 'getSimilarSongs2' : 'getSimilarSongs';
    var params = {
      id: id,
      count: count
    };
    return this._execute(method, params);
  },
  search: function search(_search, useID3, params) {
    var method = useID3 ? 'search3' : 'search2';
    params.query = _search;
    return this._execute(method, params);
  },
  savePlayQueue: function savePlayQueue(ids, current, currentMediaTime) {
    var method = 'savePlayQueue';
    var params = {
      id: ids,
      current: current,
      position: currentMediaTime || 0
    };
    return this._execute(method, params, true);
  },
  getPlayQueue: function getPlayQueue(cb) {
    return this._execute('getPlayQueue', null);
  },
  getChatMessages: function getChatMessages(since) {
    return this._execute('getChatMessages', since ? {
      since: since
    } : null);
  },
  addChatMessage: function addChatMessage(message) {
    return this._execute('addChatMessage', {
      message: message
    });
  },
  createBookmark: function createBookmark(id, position, comment) {
    var params = {
      id: id,
      position: position
    };
    if (comment) params.comment = comment;
    return this._execute('createBookmark', params);
  },
  deleteBookmark: function deleteBookmark(id) {
    var params = {
      id: id
    };
    return this._execute('deleteBookmark', params);
  },
  getBookmarks: function getBookmarks() {
    return this._execute('getBookmarks', null);
  },
  getInternetRadioStations: function getInternetRadioStations() {
    return this._execute('getInternetRadioStations', null);
  },
  createInternetRadioStation: function createInternetRadioStation(name, streamUrl, homepageUrl) {
    return this._execute('createInternetRadioStation', {
      name: name,
      streamUrl: streamUrl,
      homepageUrl: homepageUrl
    });
  },
  updateInternetRadioStation: function updateInternetRadioStation(id, name, streamUrl, homepageUrl) {
    return this._execute('updateInternetRadioStation', {
      id: id,
      name: name,
      streamUrl: streamUrl,
      homepageUrl: homepageUrl
    });
  },
  deleteInternetRadioStation: function deleteInternetRadioStation(id) {
    return this._execute('deleteInternetRadioStation', {
      id: id
    });
  },

  getPodcasts: function getPodcasts(includeEpisodes, id) {
    var params = {};
    if (includeEpisodes) params.includeEpisodes = true;
    if (id) params.id = id;
    return this._execute('getPodcasts', params);
  },

  getNewestPodcasts: function getNewestPodcasts(count) {
    var params = {};
    if (count) params.count = count;
    return this._execute('getNewestPodcasts', params);
  },

  getCompleteAlbum: function getCompleteAlbum(id) {
    var that = this;
    return new Promise(function (rs, rj) {
      that._execute('getAlbum', {
        id: id
      }).then(function (data) {
        data.album.song = [data.album.song];
        rs(data);
      }).catch(function (error) {
        rj(error);
      });
    });
  },

  getSimilarArtists: function getSimilarArtists(id, count, depth) {
    var that = this,
        c = this.empty(count) ? '10' : count,
        d = this.empty(depth) ? '2' : depth;
    c = c * 1;
    d = d * 1;
    var similarArtists = ['' + id];
    return new Promise(function (rs, rj) {
      var pinit = _SubsonicCache.SubsonicCache.ArtistInfoById[id] ? Promise.resolve({
        artistInfo2: _SubsonicCache.SubsonicCache.ArtistInfoById[id]
      }) : that.getArtistInfo2(id, count);
      pinit.then(function (data) {
        var nextloaders = [];
        var info = _SubsonicCache.SubsonicCache.ArtistInfoById[id] = data.artistInfo2;
        if (info.similarArtist) {
          info.similarArtist.forEach(function (a) {
            // cache artist name, because it may not be cached in station mode
            if (!_SubsonicCache.SubsonicCache.ArtistsById[a.id]) _SubsonicCache.SubsonicCache.ArtistsById[a.id] = a;
            if (d) nextloaders.push(that.getSimilarArtists(a.id, c, d - 1));
            similarArtists.push(a.id);
          });
        }
        console.log('similar to', id, similarArtists, nextloaders.length);
        if (!nextloaders.length) {
          rs(similarArtists.unique());
          return;
        }
        Promise.all(nextloaders).then(function (res) {
          res.forEach(function (sa) {
            console.log('sa', sa);
            sa.forEach(function (s) {
              console.log(similarArtists, s);
              similarArtists = similarArtists.concat(s);
            });
          });
          console.log('similar to', id, similarArtists);
          rs(similarArtists.unique());
        }).catch(function (err2) {
          console.error(err2);
          rj(err2);
        });
      }).catch(function (err) {
        rj(err);
      });
    });
  },

  empty: function empty(v) {
    return v === undefined || v === null || v === '';
  },

  getTopSimilarSongs: function getTopSimilarSongs(id, count, depth) {
    var that = this;
    count = this.empty(count) ? '10' : count;
    depth = this.empty(depth) ? '2' : depth;
    return new Promise(function (rs, rj) {
      that.getSimilarArtists(id, count, depth).then(function (data) {
        var nextloaders = [];
        data.forEach(function (i) {
          var a = _SubsonicCache.SubsonicCache.ArtistsById[i] ? _SubsonicCache.SubsonicCache.ArtistsById[i].name : null;
          if (a) nextloaders.push(that.getTopSongs(a));
        });
        return Promise.all(nextloaders);
      }).then(function (data) {
        data = data.map(function (d) {
          return d.topSongs.song;
        }).flatten().unique(true);
        console.log(data);
        rs(data);
      }).catch(function (err) {
        rj(err);
      });
    });
  },

  // the following are needed only for the radio-generator. they recursively (bredth first search) try to find all songs under a particular artist/index/Folder
  getMusicDirectorySongs: function getMusicDirectorySongs(id) {
    //console.log(id, "fetching...");
    var that = this;
    var songs = [];
    return new Promise(function (rs, rj) {
      that.getMusicDirectory(id).then(function (data) {
        //console.log(id, "returned", data);
        var children = data.directory.child || [],
            i;
        var dirloaders = [];
        for (i = 0; i < children.length; ++i) {
          if (children[i].isDir) {
            dirloaders.push(that.getMusicDirectorySongs(children[i].id));
          } else {
            //console.log("adding song", children[i]);
            songs.push(children[i]);
          }
        }
        if (dirloaders.length) {
          //console.log(id, "more", dirloaders.length);
          Promise.all(dirloaders).then(function (res) {
            res.forEach(function (md) {
              md.musicDirectory.song.forEach(function (s) {
                songs.push(s);
              });
            });
            rs({ musicDirectory: { song: songs } });
          }).catch(function (err2) {
            console.error(err2);
            rj(err2);
          });
        } else {
          console.log(id, 'done', children.length, songs.length);
          rs({ musicDirectory: { song: songs } });
        }
      }).catch(function (err) {
        console.error(err);
        rj(err);
      });
    });
  },
  getMusicDirectoryAlbums: function getMusicDirectoryAlbums(id) {
    //console.log(id, "fetching...");
    var that = this;
    return new Promise(function (rs, rj) {
      that.getMusicDirectory(id).then(function (data) {
        var albums = [];
        //console.log(id, "returned", data);
        var children = data.directory.child || [],
            i;
        var dirloaders = [];
        var songs = [];
        for (i = 0; i < children.length; ++i) {
          if (children[i].isDir) {
            //console.log("fetching folder", children[i]);
            dirloaders.push(that.getMusicDirectoryAlbums(children[i].id));
          } else {
            //console.log("adding song", id, children[i]);
            songs.push(children[i]);
          }
        }
        if (songs.length) albums.push(songs);
        if (dirloaders.length) {
          //console.log(id, "more", dirloaders.length);
          Promise.all(dirloaders).then(function (res) {
            res.forEach(function (md) {
              //console.log(md);
              md.musicDirectoryAlbums.album.forEach(function (s) {
                //console.log('appending album length', s.length, s);
                albums.push(s);
              });
            });
            rs({ musicDirectoryAlbums: { album: albums } });
          }).catch(function (err2) {
            console.error(err2);
            rj(err2);
          });
        } else {
          //console.log(id, "done", albums.length);
          rs({ musicDirectoryAlbums: { album: albums } });
        }
      }).catch(function (err) {
        console.error(err);
        rj(err);
      });
    });
  },
  getArtistSongs: function getArtistSongs(id) {
    var that = this;
    return new Promise(function (rs, rj) {
      that.getArtist(id).then(function (data) {
        console.log(data);
        var album = data && data.artist ? data.artist.album || [] : [];
        var albumloaders = album.map(function (a) {
          return that.getAlbum(a.id);
        });
        console.log(albumloaders);
        Promise.all(albumloaders).then(function (albums) {
          var rv = {
            albumCollection: {
              song: []
            }
          };
          albums.forEach(function (al) {
            al.album.song.forEach(function (s) {
              rv.albumCollection.song.push(s);
            });
          });
          console.log('rv', rv);
          rs(rv);
        }).catch(function (err) {
          rj(err);
        });
      }).catch(function (err) {
        console.error(err);
        rj(err);
      });
    });
  },
  getArtistAlbums: function getArtistAlbums(id) {
    var that = this;
    return new Promise(function (rs, rj) {
      that.getArtist(id).then(function (data) {
        console.log(data);
        var albumloaders = data.artist.album.map(function (a) {
          return that.getAlbum(a.id);
        });
        console.log(albumloaders);
        Promise.all(albumloaders).then(function (albums) {
          var rv = {
            albumCollection: {
              albums: []
            }
          };
          albums.forEach(function (al) {
            var alSongs = [];
            al.album.song.forEach(function (s) {
              alSongs.push(s);
            });
            rv.albumCollection.albums.push(alSongs);
          });
          console.log('rv', rv);
          rs(rv);
        }).catch(function (err) {
          rj(err);
        });
      }).catch(function (err) {
        console.error(err);
        rj(err);
      });
    });
  },
  getPlaylistAsAlbumList: function getPlaylistAsAlbumList(id, id3) {
    var that = this;
    return new Promise(function (rs, rj) {
      that.getPlaylist(id).then(function (res) {
        rs(that.convertPlaylistToAlbumList(res.playlist, id3));
      });
    });
  },
  convertPlaylistToAlbumList: function convertPlaylistToAlbumList(pl, id3) {
    var init = {
      done: {},
      result: []
    };
    pl = pl || { entry: [] };
    pl.entry = pl.entry || [];
    var res = pl.entry.reduce(function (v, p, idx, a) {
      var r;
      if (id3) {
        // <album id="1768" name="Duets" coverArt="al-1768" songCount="2" created="2002-11-09T15:44:40"
        //  duration="514" artist="Nik Kershaw" artistId="829"/>
        r = {
          coverArt: p.coverArt,
          name: p.album,
          artist: p.artist,
          id: p.albumId,
          artistId: p.artistId,
          created: p.created
        };
      } else {
        // <album id="11" parent="1" title="Arrival" artist="ABBA" isDir="true" coverArt="22"
        //  userRating="4" averageRating="4.5"/>
        r = {
          coverArt: p.coverArt,
          name: p.album,
          artist: p.artist,
          id: p.parent
        };
      }
      if (v.done[r.id]) return v;
      v.done[r.id] = r;
      v.result.push(r);
      return v;
    }, init);
    console.log(res);
    var rv = {};
    rv[id3 ? 'albumList2' : 'albumList'] = {
      album: res.result
    };
    return rv;
  }
};
// console.log({ URI: URI, md5: md5, SubsonicCache: SubsonicCache, SubsonicFetch: SubsonicFetch });
exports.URI = _URI2.default;
exports.md5 = _md2.default;
exports.SubsonicFetch = SubsonicFetch;
exports.SubsonicCache = _SubsonicCache.SubsonicCache;
exports.default = SubsonicFetch;