import React, { Component } from 'react';

import { HashRouter as Router, Route, Link } from 'react-router-dom';

import Button from '@material-ui/core/Button';
import Card from '@material-ui/core/Card';
import CssBaseline from '@material-ui/core/CssBaseline';
import { amber, indigo } from '@material-ui/core/colors';
import { withStyles } from '@material-ui/core/styles';

// eslint-disable-next-line
import 'subsonic/api/RadioMaker';

import LoginCredentials from 'subsonic/components/creds/LoginCredentials';
import ImportCredentials from 'subsonic/components/creds/ImportCredentials';
import AuthCodeClient from 'subfire/components/creds/AuthCodeClient';
import AuthCodeSource from 'subfire/components/creds/AuthCodeSource';

import CredentialsActions, {
  allocateDigits
} from 'subsonic/actions/CredentialsActions';
import CredentialsContainer from 'subsonic/components/containers/CredentialsContainer';

import RemotesActions from 'subfire/actions/RemotesActions';
import SubsonicContainer from 'subsonic/components/containers/SubsonicContainer';

import PlaylistsBrowser from 'subsonic/components/browsers/PlaylistsBrowser';
import AlbumQueryBrowser from 'subsonic/components/browsers/AlbumQueryBrowser';
import ServerSelect from 'subsonic/components/browsers/ServerSelect';
import ArtistsBrowser from 'subsonic/components/browsers/ArtistsBrowser';
import AlbumSongBrowser from 'subsonic/components/browsers/AlbumSongBrowser';
import PlaylistBrowser from 'subsonic/components/browsers/PlaylistBrowser';
import ArtistAlbumsBrowser from 'subsonic/components/browsers/ArtistAlbumsBrowser';
import DirectoryBrowser from 'subsonic/components/browsers/DirectoryBrowser';
import IndexBrowser from 'subsonic/components/browsers/IndexBrowser';
import TreeBrowser from 'subsonic/components/browsers/TreeBrowser';
import NowPlayingBrowser from 'subsonic/components/browsers/NowPlayingBrowser';
import BookmarksBrowser from 'subsonic/components/browsers/BookmarksBrowser';
import PodcastsBrowser from 'subsonic/components/browsers/PodcastsBrowser';
import VideoBrowser from 'subsonic/components/browsers/VideoBrowser';
import RadioBrowser from 'subfire/components/browsers/RadioBrowser';
import MuppetPlayer from 'subfire/components/external/Muppet';
import Search from 'subsonic/components/search/Search';
import Preferences from 'subfire/components/Preferences';
import LoadedSongsBrowser from 'subsonic/components/browsers/LoadedSongsBrowser';
import ColorContainer from 'subfire/components/containers/ColorContainer';
import ColorTheme from 'subfire/components/containers/ColorTheme';
import RemoteConfigScreen from 'subfire/components/remotes/RemoteConfigScreen';
import RemoteConfigInit from 'subfire/components/remotes/RemoteConfigInit';
import RemoteControl from 'subfire/components/remotes/RemoteControl';

import DemoPlayer from 'components/DemoPlayer';
import DemoMain from 'components/DemoMain';

import SubsonicLoader from 'subsonic/components/SubsonicLoader';
import SubsonicLoadingCard from 'components/SubsonicLoadingCard';

import CastSubsonicContext, {
  CastSubsonicProvider
} from 'subsonic/context/CastSubsonic';

// import { ChromecastButton } from "subsonic/components/controls/MediaButtons";

import RemoteControlKeyHandler from 'subfire/components/RemoteControlKeyHandler';

import Snacker from 'subsonic/components/controls/Snacker';

window.subsonicClientName = 'SubFireBase';

const DemoButtons = props => {
  return (
    <React.Fragment>
      <Button id="b1" component={Link} to="/login">
        Login
      </Button>
      <Button id="b2" component={Link} to="/linkClient">
        Link Client
      </Button>
      <Button id="b3" component={Link} to="/linkSource">
        Link Server
      </Button>
      <Button id="b4" onClick={window.openSmall}>
        Go Small
      </Button>
      <CredentialsContainer>
        <ServerSelect handleServerChange={props.handleServerChange} />
      </CredentialsContainer>
    </React.Fragment>
  );
};

// const nav = {
//   "ArrowLeft": {
//     "#select-server": "#b4",
//     "#b4": "#b3",
//     "#b3": "#b2",
//     "#b2": "#b1"
//   },
//   "ArrowRight": {
//     "#b1": "#b2",
//     "#b2": "#b3",
//     "#b3": "#b4",
//     "#b4": "#select-server"
//   },
//   "ArrowUp": {
//   },
//   "ArrowDown": {
//   }
// }

const styles = theme => ({
  tableCells: {
    fontSize: '1.5em'
  }
});

class App extends Component {
  constructor() {
    super();
    CredentialsActions.doLogin.defer();
    this.state = {
      ih: window.innerHeight
    };
  }

  componentDidMount() {
    RemotesActions.startCheckingPlaylists();
    window.addEventListener('resize', this.onResize);
  }

  // onResize = evt => {
  //   this.setState({ ih: window.innerHeight });
  // };

  handleCredentialsSubmit = (id, creds) => {
    console.log('logging in or updating', creds);
    CredentialsActions.saveCredentials(id, creds, true);
  };

  handleServerChange = evt => {
    console.log(evt.target.value);
    CredentialsActions.changeCurrent(evt.target.value, true);
  };

  handleCredentialsDelete = id => {
    console.log('delete', id);
    CredentialsActions.deleteCredentials(id);
  };

  loadPlaylist = (id, route) => {
    let link = '/loading/playlist/' + id + '/list';
    const { history } = route;
    history.push(link);
  };

  playPlaylist = (id, route, shuffle) => {
    let link = '/loading/playlist/' + id;
    if (shuffle) link += '/shuffle';
    const { history } = route;
    history.push(link);
  };

  playPlaylistLink = React.forwardRef((props, ref) => (
    <Link
      innerRef={ref}
      to={
        '/loading/playlist/' +
        props.id +
        (props.options && props.options.shuffle ? '/shuffle' : '')
      }
    >
      {props.children}
    </Link>
  ));

  loadAlbum = React.forwardRef((props, ref) => (
    <Link innerRef={ref} to={'/loading/album/' + props.id}>
      {props.children}
    </Link>
  ));

  playAlbum = React.forwardRef((props, ref) => (
    <Link innerRef={ref} to={'/loading/directoryAction/' + props.id + '/all'}>
      {props.children}
    </Link>
  ));

  loadAlbumID3 = React.forwardRef((props, ref) => (
    <Link innerRef={ref} to={'/loading/albumID3/' + props.id}>
      {props.children}
    </Link>
  ));
  playAlbumID3 = React.forwardRef((props, ref) => (
    <Link innerRef={ref} to={'/loading/albumAction/' + props.id + '/play'}>
      {props.children}
    </Link>
  ));

  render() {
    const { classes } = this.props;

    const loader = route => (
      <SubsonicContainer route={route}>
        <SubsonicLoader
          LoadingComponent={SubsonicLoadingCard}
          remoteRoute="/remoteControl"
        />
      </SubsonicContainer>
    );

    // dialogs
    const loginDialog = route => (
      <CredentialsContainer route={route}>
        <LoginCredentials
          native={window.isTV}
          handleSubmit={this.handleCredentialsSubmit}
          handleDelete={this.handleCredentialsDelete}
        />
      </CredentialsContainer>
    );

    // dialogs
    const importDialog = route => (
      <CredentialsContainer route={route}>
        <ImportCredentials />
      </CredentialsContainer>
    );

    const digits = allocateDigits();
    const linkClient = route => (
      <AuthCodeClient
        route={route}
        handleClose={route.history.goBack}
        digits={digits}
      />
    );

    const linkSource = route => (
      <AuthCodeSource
        route={route}
        handleClose={route.history.goBack}
        handleSubmit={this.handleACSubmit}
      />
    );

    const demoMain = route => (
      <div>
        <Card
          style={{
            padding: '20px'
          }}
        >
          <DemoButtons
            route={route}
            handleServerChange={this.handleServerChange}
          />
        </Card>
        <SubsonicContainer>
          <DemoMain {...route} />
        </SubsonicContainer>
      </div>
    );

    //       <Navigator navigation={nav} initialElementSelector="#b1"/>

    // var { ih } = this.state;
    const typeographyConfig = {
      // h2: {
      //   fontSize: ih > 900 ? "3.9rem" : ih > 700 ? "2.4rem" : "1.312rem",
      //   lineHeight: ih > 900 ? "4.1rem" : ih > 700 ? "2.6rem" : "1.16667rem"
      // },
      // h4: {
      //   fontSize: "1.125rem",
      //   lineHeight: "2.17rem",
      //   fontWeight: "400"
      // },
      // h5: {
      //   fontSize: ih > 900 ? "2.3rem" : ih > 700 ? "1.5rem" : "1rem",
      //   lineHeight: ih > 900 ? "2.5rem" : ih > 700 ? "1.8rem" : "1.5em",
      //   fontWeight: "300"
      // },
      // caption: {
      //   fontSize: ih > 900 ? "1.2rem" : ih > 700 ? "1.0rem" : "0.75rem",
      //   lineHeight: ih > 900 ? "1.2rem" : ih > 700 ? "1.1rem" : "1.375em"
      // }
    };

    const overrides = {
      // MuiInputBase: {
      //   input: {
      //     fontSize: '1.5rem'
      //   }
      // }
    };

    return (
      <CastSubsonicProvider>
        <Router basename="/">
          <ColorContainer>
            <ColorTheme
              defaultPrimary={amber}
              defaultSecondary={indigo}
              typography={typeographyConfig}
              overrides={overrides}
            >
              <CssBaseline />
              <Snacker />
              <SubsonicContainer>
                <RemoteConfigInit />
                <RemoteControlKeyHandler />
              </SubsonicContainer>
              <Route exact={true} path="/" component={demoMain} />

              <Route
                path="/load(ing)/:type/:id/:mode?"
                component={loader}
                exact={true}
                strict={true}
              />
              <Route
                path="/bookmark/:bookmarkId/:position/loading/:type/:id/:mode?"
                component={loader}
                exact={true}
                strict={true}
              />

              <Route
                path="/prefs"
                render={props => <Preferences {...props} />}
              />

              <Route
                path="/remote"
                render={props => (
                  <SubsonicContainer {...props}>
                    <RemoteConfigScreen {...props} />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/remoteControl"
                render={props => (
                  <SubsonicContainer {...props}>
                    <RemoteControl {...props} />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/player"
                render={props => (
                  <CastSubsonicContext.Consumer>
                    {csProps => (
                      <SubsonicContainer {...props} csProps={csProps}>
                        <DemoPlayer {...props} useFuzzyImage="useFuzzyImage" />
                      </SubsonicContainer>
                    )}
                  </CastSubsonicContext.Consumer>
                )}
              />
              <Route
                path="/playlists/:all"
                render={props => (
                  <SubsonicContainer {...props}>
                    <PlaylistsBrowser
                      onSelect={this.loadPlaylist}
                      linkComponent={this.playPlaylistLink}
                      renderAllPlaylists={props.match.params.all === 'all'}
                      renderMode={PlaylistsBrowser.renderTypes.GRID}
                      showShuffleOption
                    />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/playlist/:id"
                render={props => (
                  <SubsonicContainer {...props}>
                    <PlaylistBrowser />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/radio"
                render={props => (
                  <SubsonicContainer {...props}>
                    <RadioBrowser
                      renderMode={RadioBrowser.renderTypes.GRID}
                      loadPlaylistContents={false}
                    />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/albums"
                render={props => (
                  <SubsonicContainer {...props}>
                    <AlbumQueryBrowser
                      native={false}
                      linkComponent={this.playAlbum}
                      linkComponentID3={this.playAlbumID3}
                      renderMode={RadioBrowser.renderTypes.GRID}
                      fabProps={{
                        selector: '#album-query-wrapper'
                      }}
                    />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/album/:id"
                render={props => (
                  <SubsonicContainer {...props}>
                    <AlbumSongBrowser history={props.history} showBackButton />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/artists"
                render={props => (
                  <SubsonicContainer {...props}>
                    <ArtistsBrowser />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/artist/:id"
                render={props => (
                  <SubsonicContainer {...props}>
                    <ArtistAlbumsBrowser
                      linkComponent={this.playAlbumID3}
                      history={props.history}
                      showBackButton
                    />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/bookmarks"
                render={props => (
                  <SubsonicContainer {...props}>
                    <BookmarksBrowser cellClasses={classes.tableCells} />
                  </SubsonicContainer>
                )}
              />
              <Route
                path="/nowPlaying"
                render={props => (
                  <SubsonicContainer {...props}>
                    <NowPlayingBrowser />
                  </SubsonicContainer>
                )}
              />
              <Route
                path="/podcasts"
                render={props => (
                  <SubsonicContainer {...props}>
                    <PodcastsBrowser
                      podcastProps={{ variant: 'h6' }}
                      podcastItemProps={{ variant: 'subtitle1' }}
                    />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/indexes"
                render={props => (
                  <SubsonicContainer {...props}>
                    <IndexBrowser />
                  </SubsonicContainer>
                )}
              />
              <Route
                path="/directory/:id"
                render={props => (
                  <SubsonicContainer {...props}>
                    <DirectoryBrowser history={props.history} showBackButton />
                  </SubsonicContainer>
                )}
              />
              <Route
                path="/tree"
                render={props => (
                  <SubsonicContainer {...props}>
                    <TreeBrowser />
                  </SubsonicContainer>
                )}
              />
              <Route
                path="/search"
                render={props => (
                  <SubsonicContainer {...props}>
                    <Search />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/videos"
                render={props => (
                  <SubsonicContainer {...props}>
                    <VideoBrowser />
                  </SubsonicContainer>
                )}
              />

              <Route
                path="/songs"
                render={props => (
                  <SubsonicContainer {...props}>
                    <LoadedSongsBrowser />
                  </SubsonicContainer>
                )}
              />
              <Route
                path="/muppet"
                render={props => <MuppetPlayer playerType="mobile" />}
              />

              <Route path="/login" component={loginDialog} />
              <Route path="/import" component={importDialog} />
              <Route path="/linkClient" component={linkClient} />
              <Route path="/linkSource" component={linkSource} />
            </ColorTheme>
          </ColorContainer>
        </Router>
      </CastSubsonicProvider>
    );
  }
}

export default withStyles(styles)(App);
