import { TIMER_STEP, PLAYER_STATE } from './Constants';
import PlayerHandler from './PlayerHandler';
const UAParser = require('ua-parser-js');

// console.log(UAParser);

const parser = new UAParser();

/**
 * Castvar parser = require('ua-parser-js');
 player object
 * Main variables:
 *  - PlayerHandler object for handling media playback
 *  - Cast player variables for controlling Cast mode media playback
 *  - Current media variables for transition between Cast and local modes
 * @struct @constructor
 */
/* global chrome cast */
var CastSubsonic = function() {
  this.timeSinceRestart = 0;

  this.hackCheck = function() {
    console.warn('hackCheck not initialized');
  };

  /** @type {PlayerHandler} Delegation proxy for media playback */
  this.playerHandler = new PlayerHandler(this);

  /** @type {PLAYER_STATE} A state for media playback */
  this.playerState = PLAYER_STATE.IDLE;

  var os = parser.getOS();

  // security changes - don't autoplay if we start out on the player in mobile apps.
  this.autoplay = window.location.hash !== '#/player' && os.name !== 'iOS';

  /* Cast player variables */
  /** @type {cast.framework.RemotePlayer} */
  this.remotePlayer = null;
  /** @type {cast.framework.RemotePlayerController} */
  this.remotePlayerController = null;

  /* Current media variables */
  /** @type {number} A number for current media index */
  this.currentMediaIndex = 0;
  /** @type {number} A number for current media time */
  this.currentMediaTime = 0;
  /** @type {number} A number for current media duration */
  this.currentMediaDuration = -1;
  /** @type {?number} A timer for tracking progress of media */
  this.timer = null;

  /** @type {Object} media contents from JSON */
  this.mediaContents = null;

  /** @type {boolean} Fullscreen mode on/off */
  this.fullscreen = false;

  /** @type {function()} */
  this.updateStatusHandler = this.updateStatus.bind(this);

  this.setupLocalPlayer();
  // this.addVideoThumbs();
  // this.initializeUI();

  this.listeners = [];
};

CastSubsonic.prototype.initializeCastSubsonic = function() {
  var options = {};

  // Set the receiver application ID to your own (created in the
  // Google Cast Developer Console), or optionally
  // use the chrome.cast.media.DEFAULT_MEDIA_RECEIVER_APP_ID
  options.receiverApplicationId =
    chrome.cast.media.DEFAULT_MEDIA_RECEIVER_APP_ID;
  // options.receiverApplicationId = '4F8B3483';  demo video
  // options.receiverApplicationId = '4FBFE470';  subsonic supposedly
  // Auto join policy can be one of the following three:
  // ORIGIN_SCOPED - Auto connect from same appId and page origin
  // TAB_AND_ORIGIN_SCOPED - Auto connect from same appId, page origin, and tab
  // PAGE_SCOPED - No auto connect
  options.autoJoinPolicy = chrome.cast.AutoJoinPolicy.ORIGIN_SCOPED;
  try {
    cast.framework.CastContext.getInstance().setOptions(options);

    this.remotePlayer = new cast.framework.RemotePlayer();
    this.remotePlayerController = new cast.framework.RemotePlayerController(
      this.remotePlayer
    );
    this.remotePlayerController.addEventListener(
      cast.framework.RemotePlayerEventType.IS_CONNECTED_CHANGED,
      this.switchPlayer.bind(this)
    );
  } catch (e) {
    console.error(e);
    console.warn('no cast what up dude?');
  }
};

/* PlayerHandler and setup functions */

CastSubsonic.prototype.switchPlayer = function() {
  this.switchTimeRestore = this.currentMediaTime || 0;
  this.stopProgressTimer();
  // this.resetVolumeSlider();
  this.playerHandler.stop();
  // this.playerState = PLAYER_STATE.IDLE;
  if (cast && cast.framework) {
    if (this.remotePlayer.isConnected) {
      this.setupRemotePlayer();
      this.playerHandler.isCasting = true;
      return;
    }
  }
  this.playerHandler.isCasting = false;
  this.setupLocalPlayer();
};

/**
 * Media seek function
 * @param {Event} event An event object from seek
 */
CastSubsonic.prototype.seekMedia = function(curr) {
  this.playerHandler.seekTo(curr);
};

/**
 * Starts the timer to increment the media progress bar
 */
CastSubsonic.prototype.startProgressTimer = function() {
  this.stopProgressTimer();
  // console.warn('reset');
  this.timeSinceRestart = 0;
  // Start progress timer
  this.timer = setInterval(this.updateStatusHandler, TIMER_STEP);
};

/**
 * Stops the timer to increment the media progress bar
 */
CastSubsonic.prototype.stopProgressTimer = function(reason) {
  if (this.timer) {
    clearInterval(this.timer);
    this.timer = null;
  }
  // fire one last update for playing state
  setTimeout(
    this.updateStatusListeners.bind(this, reason || 'stopProgressTimer'),
    25
  );
};

/**
 * Helper function
 * Increment media current position
 */
CastSubsonic.prototype.updateStatus = function() {
  this.hackCheck();
  this.updateStatusListeners('normal');

  // First sync with the current player's time
  this.currentMediaTime = this.playerHandler.getCurrentMediaTime();
  this.currentMediaDuration = this.playerHandler.getMediaDuration();

  // if (this.playerState === PLAYER_STATE.PLAYING) {
  //   this.firedPauseState = false;
  //   // hackery - stopping by timestamp rather than by reading the state of the audio tag or event from chromecast. need to fix.
  //   if (this.currentMediaTime < this.currentMediaDuration) {
  //     if (this.currentMediaTime === 0 && this.timeSinceRestart > 1000) {
  //       console.warn('had to hack', this.currentMediaTime, this.currentMediaDuration, this.timeSinceRestart);
  //       this.playerHandler.seekTo(this.timeSinceRestart / 1000);
  //       // } else {
  //       //   console.warn('nope all fine this time');
  //     }
  //     this.timeSinceRestart += TIMER_STEP;
  //
  //     this.currentMediaTime += TIMER_STEP / 1000;
  //     this.updateStatusListeners('normal');
  //
  //   } else {
  //     if (this.currentMediaDuration !== 0) {
  //       console.warn('about to end', this.currentMediaTime, this.currentMediaDuration, this.playerState);
  //       this.endPlayback();
  //       this.autoplay = true;
  //       this.updateStatusListeners('normal');
  //     }
  //   }
  // } else {
  //   if (this.currentMediaTime >= this.currentMediaDuration && this.playerState !== PLAYER_STATE.LOADING) {
  //     console.warn('about to hack not playing', this.currentMediaTime, this.currentMediaDuration, this.playerState);
  //     this.endPlayback();
  //     this.updateStatusListeners('normal');
  //     this.firedPauseState = true;
  //     this.autoplay = true;
  //   } else if (!this.firedPauseState) {
  //     this.updateStatusListeners('normal');
  //     this.firedPauseState = true;
  //   }
  // }
};

/**
 * Update progress bar based on timer
 */
CastSubsonic.prototype.updateStatusListeners = function(reason) {
  //if (reason !== 'normal') console.log('reason', reason, this.listeners.length);
  var status = this.playerHandler.getState(reason);
  //if (reason !== 'normal') console.log('status', status);
  this.listeners.forEach(function(l) {
    //if (reason !== 'normal') console.log('dispatching', status, 'to', l);
    l(status);
  });
  status.reason = 'dispatched';
};

CastSubsonic.prototype.addUpdateListener = function(f) {
  this.listeners.push(f);
};

CastSubsonic.prototype.removeUpdateListener = function(f) {
  var array = this.listeners;
  const index = array.indexOf(f);

  if (index !== -1) {
    array.splice(index, 1);
  }
};

/**
 *  End playback. Called when media ends.
 */
CastSubsonic.prototype.endPlayback = function() {
  console.warn('playbackEnded');
  this.currentMediaTime = 0;
  this.playerState = PLAYER_STATE.IDLE;
  this.stopProgressTimer('playbackEnded');
};

/**
 * Makes human-readable message from chrome.cast.Error
 * @param {chrome.cast.Error} error
 * @return {string} error message
 */
CastSubsonic.getErrorMessage = function(error) {
  switch (error.code) {
    case chrome.cast.ErrorCode.API_NOT_INITIALIZED:
      return (
        'The API is not initialized.' +
        (error.description ? ' :' + error.description : '')
      );
    case chrome.cast.ErrorCode.CANCEL:
      return (
        'The operation was canceled by the user' +
        (error.description ? ' :' + error.description : '')
      );
    case chrome.cast.ErrorCode.CHANNEL_ERROR:
      return (
        'A channel to the receiver is not available.' +
        (error.description ? ' :' + error.description : '')
      );
    case chrome.cast.ErrorCode.EXTENSION_MISSING:
      return (
        'The Cast extension is not available.' +
        (error.description ? ' :' + error.description : '')
      );
    case chrome.cast.ErrorCode.INVALID_PARAMETER:
      return (
        'The parameters to the operation were not valid.' +
        (error.description ? ' :' + error.description : '')
      );
    case chrome.cast.ErrorCode.RECEIVER_UNAVAILABLE:
      return (
        'No receiver was compatible with the session request.' +
        (error.description ? ' :' + error.description : '')
      );
    case chrome.cast.ErrorCode.SESSION_ERROR:
      return (
        'A session could not be created, or a session was invalid.' +
        (error.description ? ' :' + error.description : '')
      );
    case chrome.cast.ErrorCode.TIMEOUT:
      return (
        'The operation timed out.' +
        (error.description ? ' :' + error.description : '')
      );
    default:
      return (
        'No idea what happened.' +
        (error.description ? ' :' + error.description : '')
      );
  }
};

export default CastSubsonic;
