import { PLAYER_STATE } from './Constants';
import CastSubsonic from './CastSubsonic';
/**
 * Set the PlayerHandler target to use the remote player
 */
/* globals cast chrome */
CastSubsonic.prototype.setupRemotePlayer = function() {
  console.warn('setupRemotePlayer');
  var castSession = cast.framework.CastContext.getInstance().getCurrentSession();
  this.lastValue = null;
  window.remotePlayer = this.remotePlayer;
  if (!this.initializedRemotePlayer) {
    this.remotePlayerController.addEventListener(
      cast.framework.RemotePlayerEventType.PLAYER_STATE_CHANGED,
      function(v) {
        console.warn(v, this.playerState);
        if (
          this.lastValue === 'PLAYING' &&
          (v.value === null || v.value === 'IDLE') &&
          this.playerState === PLAYER_STATE.PLAYING
        ) {
          this.endPlayback();
        }
        this.lastValue = v.value;
      }.bind(this)
    );

    // Add event listeners for player changes which may occur outside sender app
    this.remotePlayerController.addEventListener(
      cast.framework.RemotePlayerEventType.IS_PAUSED_CHANGED,
      function() {
        if (this.remotePlayer.isPaused) {
          this.playerHandler.pause();
        } else {
          this.playerHandler.play();
        }
      }.bind(this)
    );

    this.remotePlayerController.addEventListener(
      cast.framework.RemotePlayerEventType.IS_MUTED_CHANGED,
      function() {
        if (this.remotePlayer.isMuted) {
          this.playerHandler.mute();
        } else {
          this.playerHandler.unMute();
        }
      }.bind(this)
    );

    this.remotePlayerController.addEventListener(
      cast.framework.RemotePlayerEventType.VOLUME_LEVEL_CHANGED,
      function() {}
    );
  }
  // This object will implement PlayerHandler callbacks with
  // remotePlayerController, and makes necessary UI updates specific
  // to remote playback
  var playerTarget = (this.playerTarget = {});

  this.hackCheck = function() {};

  playerTarget.play = function() {
    if (this.remotePlayer.isPaused) {
      this.remotePlayerController.playOrPause();
    }
  }.bind(this);

  playerTarget.pause = function() {
    if (!this.remotePlayer.isPaused) {
      this.remotePlayerController.playOrPause();
    }
  }.bind(this);

  playerTarget.stop = function() {
    this.remotePlayerController.stop();
  }.bind(this);

  playerTarget.load = function(media) {
    console.warn('Loading...' + media);
    var mediaInfo = new chrome.cast.media.MediaInfo(
      media.src,
      media.contentType || 'audio/mp3'
    );

    mediaInfo.metadata = new chrome.cast.media.MusicTrackMediaMetadata();
    mediaInfo.metadata.metadataType =
      chrome.cast.media.MetadataType.MUSIC_TRACK;
    mediaInfo.metadata.title = media.title;
    mediaInfo.metadata.artist = media.artist;
    mediaInfo.metadata.albumName = media.album;
    mediaInfo.metadata.releaseYear = media.year;
    mediaInfo.metadata.trackNumber = media.track;
    mediaInfo.metadata.discNumber = media.disc;
    mediaInfo.metadata.images = [
      {
        url: media.coverArtURL
      }
    ];

    var request = new chrome.cast.media.LoadRequest(mediaInfo);
    castSession.loadMedia(request).then(
      this.playerHandler.loaded.bind(this.playerHandler),
      function(errorCode) {
        this.playerState = PLAYER_STATE.ERROR;
        console.log(
          'Remote media load error: ' + CastSubsonic.getErrorMessage(errorCode)
        );
      }.bind(this)
    );
  }.bind(this);

  playerTarget.getCurrentMediaTime = function() {
    return this.remotePlayer.currentTime;
  }.bind(this);

  playerTarget.getMediaDuration = function() {
    return this.remotePlayer.duration;
  }.bind(this);

  playerTarget.mute = function() {
    if (!this.remotePlayer.isMuted) {
      this.remotePlayerController.muteOrUnmute();
    }
  }.bind(this);

  playerTarget.unMute = function() {
    if (this.remotePlayer.isMuted) {
      this.remotePlayerController.muteOrUnmute();
    }
  }.bind(this);

  playerTarget.isMuted = function() {
    return this.remotePlayer.isMuted;
  }.bind(this);

  playerTarget.seekTo = function(time) {
    this.remotePlayer.currentTime = time;
    this.remotePlayerController.seek();
  }.bind(this);

  this.playerHandler.setTarget(playerTarget);

  // Setup remote player volume right on setup
  // The remote player may have had a volume set from previous playback
  if (this.remotePlayer.isMuted) {
    this.playerHandler.mute();
  }

  this.autoplay = true;

  this.startProgressTimer();
  this.playerHandler.play();

  this.initializedRemotePlayer = true;
};
