import React from 'react';
import { Link } from 'react-router-dom';

import { withStyles } from '@material-ui/core/styles';
import Card from '@material-ui/core/Card';
import CardActions from '@material-ui/core/CardActions';
import CardContent from '@material-ui/core/CardContent';
import Button from '@material-ui/core/Button';
import Typography from '@material-ui/core/Typography';
import { ChromecastButton } from 'subsonic/components/controls/MediaButtons';
import RemoteSenderMenu from 'subfire/components/remotes/RemoteSenderMenu';

import FlowText from 'subsonic/components/styled/FlowText';

const styles = theme => ({
  card: {
    minWidth: 275
  },
  bullet: {
    display: 'inline-block',
    margin: '0 2px',
    transform: 'scale(0.8)'
  },
  title: {
    marginBottom: 16,
    fontSize: 14,
    color: theme.palette.text.secondary
  },
  pos: {
    marginBottom: 12,
    color: theme.palette.text.secondary
  }
});

const DemoMain = props => {
  const { classes } = props;
  const offline = props.subsonic.offline;
  let offlineContent = '';
  if (offline)
    offlineContent = (
      <span>
        <br />
        THE APP IS CURRENTLY OFFLINE
        <br />
      </span>
    );
  const csProps = props;

  const rsProps = {
    remotes: props.remotes,
    subsonic: props.subsonic,
    SubsonicSource: props.SubsonicSource
  };
  //      <Button size="small" component={Link} to="/tree">Tree</Button>

  return (
    <Card className={classes.card}>
      <CardContent>
        <Typography variant="h2" component="h2" style={{ paddingBottom: 10 }}>
          SubFire Baseline Demo 2.35.0 (including Video)
        </Typography>
        <FlowText component="div" variant="body1">
          This app is meant to be a basic demo of all of the core features of
          the SubFire Subsonic framework. It is expected that subsequent apps
          start by making a clone of this and change App and components and
          styles to suit your use-cases.
          {offlineContent}
          <br />
          It will be used as the baseline for several released or upcoming
          projects:
          <ul>
            <li>
              A new mobile player along a browser extension and mini-player mode
              (done).
            </li>
            <li>
              A new TV application (that will be tablet-compatible) for FireOS
              and potentially Android TV. (Beta released)
            </li>
            <li>
              A rewrite of the "toothbrush timer" (done, but needs to be updated
              to most recent base).
            </li>
            <li>
              A "Dashboard" app for landscape-phones in cars, following the
              Android Auto specification for safety (done).
            </li>
            <li>
              A browser-based desktop app with playlist editor (designing).
            </li>
            <li>
              A kids-room player with limited controls, playing triggered
              remotely by parent/guardian in another room.
            </li>
          </ul>
          All of these are packaged as Progressive Web Apps for easy
          deployment/installation through Chrome on multiple platforms.
        </FlowText>
      </CardContent>
      <CardActions>
        <Button size="small" component={Link} to="/artists">
          Artists (ID3)
        </Button>
        <Button size="small" component={Link} to="/albums">
          Albums
        </Button>
        <Button size="small" component={Link} to="/indexes">
          Folders
        </Button>
        <Button size="small" component={Link} to="/search">
          Search
        </Button>
        <Button size="small" component={Link} to="/prefs">
          Preferences
        </Button>
      </CardActions>
      <CardActions>
        <Button size="small" component={Link} to="/radio">
          SubFire Radio
        </Button>
        <Button size="small" component={Link} to="/playlists/all">
          All Playlists
        </Button>
        <Button size="small" component={Link} to="/playlists/normal">
          Playlists
        </Button>
        <Button size="small" component={Link} to="/podcasts">
          Podcasts
        </Button>
        <Button size="small" component={Link} to="/player">
          Resume
        </Button>
      </CardActions>
      <CardActions>
        <Button size="small" component={Link} to="/bookmarks">
          Bookmarks
        </Button>
        <Button size="small" component={Link} to="/loading/playQueue/0">
          Restore Play Queue
        </Button>
        <Button size="small" component={Link} to="/nowPlaying">
          Now Playing
        </Button>
        <Button size="small" component={Link} to="/videos">
          Videos
        </Button>
        <Button size="small" component={Link} to="/muppet">
          Muppets!
        </Button>
      </CardActions>
      <CardActions>
        <Button size="small" component={Link} to="/remote">
          Receive
        </Button>
        <Button size="small" component={Link} to="/remoteControl">
          Remote Control
        </Button>
        <RemoteSenderMenu variant="listItem" {...rsProps} />
        <ChromecastButton {...csProps} />
      </CardActions>
    </Card>
  );
};

export default withStyles(styles)(DemoMain);
