import React, { Component } from 'react';
import { Link } from 'react-router-dom';
import Card from '@material-ui/core/Card';
import CardActions from '@material-ui/core/CardActions';
import CardContent from '@material-ui/core/CardContent';
import Button from '@material-ui/core/Button';
import IconButton from '@material-ui/core/Button';
import Typography from '@material-ui/core/Typography';

import { emptytrack } from 'subsonic/stores/SubsonicStore';

import {
  PlayPauseButton,
  NextButton,
  PrevButton,
  SkipAlbumButton,
  Replay10Button,
  Forward30Button,
  ShuffleButton,
  BookmarkButton,
  SavePlayQueueButton,
  ChromecastButton
} from 'subsonic/components/controls/MediaButtons';
import RemoteReceiverToggleButton from 'subfire/components/remotes/RemoteReceiverToggleButton';
import Notification from 'subsonic/components/controls/Notification';
import SubsonicActions from 'subsonic/actions/SubsonicActions';
import ColorActions from 'subfire/actions/ColorActions';

import GradientGeneratingImage from 'subfire/components/GradientGeneratingImage';

import FlexibleBackground from 'subfire/components/FlexibleBackground';
// import GradientBackground from "subfire/components/GradientBackground";
// import FuzzyImageBackground from "subfire/components/FuzzyImageBackground";

import NowPlaying from 'subsonic/components/controls/NowPlaying';

import RangeSlider from 'subsonic/components/controls/RangeSlider';

import HollowCard from 'subsonic/components/styled/HollowCard';

import MenuIcon from '@material-ui/icons/FormatListBulleted';

import SongListDrawer from 'subsonic/components/styled/SongListDrawer';

class DemoPlayer extends Component {
  static handleMediaType(currentFile) {
    const m = document.querySelector('#media_element');
    if (currentFile.isVideo) {
      m.style.visibility = 'visible';
      m.style.position = 'absolute';
      m.style.top = '60px';
      m.style.left = '0px';
      m.width = 800;
      m.height = 450;
    } else {
      m.style.visibility = 'hidden';
      m.style.position = 'absolute';
      m.style.left = '1px';
      m.style.top = '1px';
      m.width = 1;
      m.height = 1;
    }
  }

  static getDerivedStateFromProps(props, prevState) {
    const { subsonic = {}, SubsonicSource, csProps } = props;
    const { isConnected, currentPlaying } = subsonic;
    if (!isConnected) {
      return prevState;
    }
    let newState;
    if (!currentPlaying.queue.length) {
      newState = {
        showBackground: false,
        currentFile: emptytrack,
        reason: csProps.reason
      };
    } else {
      const currentFile = currentPlaying.queue[currentPlaying.idx];
      const p = csProps;
      let {
        showBackground,
        isWaitingForNext,
        coverSize,
        coverArt,
        mediaSrc,
        mediaIcon,
        reason
      } = prevState;
      if (currentFile !== prevState.currentFile) {
        console.log('next arrived', isWaitingForNext, reason, p.reason);
        isWaitingForNext = false;
        coverSize = Math.min(
          Math.trunc(window.innerWidth / 2),
          window.innerHeight - 200
        );
        coverArt = SubsonicSource.getCoverArtURL(currentFile.id, coverSize);
        mediaSrc =
          currentFile.localPath ||
          (currentFile.src && currentFile.src.startsWith('http')
            ? currentFile.src
            : SubsonicSource.getStreamingURL(currentFile.id));
        mediaIcon =
          SubsonicSource.getCoverArtURL(currentFile.id, 192) + '&co=false';

        // now load castplayer
        currentFile.coverArtURL = coverArt;
        currentFile.src = mediaSrc;
        if (
          !window.cp.currentMedia ||
          window.cp.currentMedia.src !== mediaSrc
        ) {
          window.cp.playerHandler.load(currentFile, 0);
        }
        DemoPlayer.handleMediaType(currentFile);
      } else {
        if (
          !prevState.isWaitingForNext &&
          reason !== p.reason &&
          p.reason === 'playbackEnded'
        ) {
          console.log('endPlayback next...', reason, p.reason);
          isWaitingForNext = true;
          SubsonicActions.playerNext.defer();
        }
      }
      // hack - no easy way to pass this around
      if (window.bookmarkSeekTo) {
        window.castSubsonic.seekMedia(window.bookmarkSeekTo);
        delete window.bookmarkSeekTo;
      }

      if (prevState.showBackground !== p.isPlaying) {
        showBackground = p.isPlaying;
      }

      // if (p.reason !== 'normal') console.log('p', p);

      const { currentMediaTimeString, durationString } = p;

      newState = {
        currentFile,
        coverArt,
        mediaSrc,
        mediaIcon,
        coverSize,
        showBackground,
        isWaitingForNext,
        currentMediaTimeString,
        durationString,
        reason: csProps.reason
      };
    }
    return Object.assign({}, prevState, newState);
  }

  constructor(props) {
    super(props);
    this.state = {
      // songDrawerOpen: false
    };
    // this.state = DemoPlayer.getDerivedStateFromProps(props);
  }

  // componentWillReceiveProps(nextProps) {
  //   this.setState(DemoPlayer.getDerivedStateFromProps(nextProps, this.state));
  // }

  // fullscreen notes
  // $('video').requestFullscreen().then(function(res,err) { console.log(res); } );
  // document.exitFullscreen().then(function(res,err) { console.log(res); } );

  // not every player will do this - if we keep an inline playing frame a-la Plex, then this needs to not happen here.
  // stop: control the player because of internal state; pause = toggle autoplay
  componentWillUnmount() {
    const { csProps } = this.props;
    if (csProps.isPlaying) {
      window.cp.playerHandler.stop();
    }
    ColorActions.resetColors();

    DemoPlayer.handleMediaType({});
  }

  componentDidMount() {
    if (window.cp.playerState === 'STOPPED') {
      window.cp.playerHandler.play();
    }
  }

  toggleBookmark = currentMediaTime => {
    // console.log('toggleBookmark', currentMediaTime, this.props);
    const { subsonic } = this.props;
    const { currentPlaying } = subsonic;
    const id = currentPlaying.queue[currentPlaying.idx].id;

    if (subsonic.bookmarksById[id]) {
      SubsonicActions.deleteBookmark(id);
    } else {
      SubsonicActions.createBookmark(id, Math.trunc(currentMediaTime * 1000));
    }
  };

  // seekValueChange = (seekValueState) => {
  //   console.log(seekValueState);
  // }

  handleSongDrawer = event => {
    SubsonicActions.toggleMenu();
  };

  handleSongDrawerClose = () => {
    SubsonicActions.toggleMenu();
  };

  handleSongClick = (song, idx) => {
    idx = idx * 1;
    SubsonicActions.playerSkipToIndex(idx);
    SubsonicActions.toggleMenu();
    // this.setState({songDrawerOpen: false});
  };

  render() {
    // console.log('DemoPlayer', this.props, this.state);

    const {
      subsonic,
      SubsonicSource,
      useGradients,
      useSingleGradient,
      useFuzzyImage,
      history,
      csProps
    } = this.props;

    const {
      showBackground,
      currentFile,
      coverArt,
      coverSize,
      currentMediaTimeString,
      durationString
    } = this.state;

    if (!subsonic.isConnected) {
      return (
        <Card>
          <Typography component="div">Still Loading</Typography>
        </Card>
      );
    }

    const { currentPlaying, following, isShowingMenu } = subsonic;
    // console.log('following', following);
    const id =
      currentPlaying &&
      currentPlaying.queue &&
      currentPlaying.queue[currentPlaying.idx]
        ? currentPlaying.queue[currentPlaying.idx].id
        : '-1';

    if (!currentPlaying.queue.length || !currentFile) {
      let message = `It looks like there may be a problem loading the player. If this does not go away, then please go back and try again.`;
      if (following) {
        message = `The person you are following may not be listening anymore.`;
      }
      return (
        <Card>
          <CardContent>
            <Typography component="div">{message}</Typography>
          </CardContent>
          <CardActions>
            <Button>
              <a onClick={history.goBack}>Go Back</a>
            </Button>
            <Button component={Link} to="/">
              Go Home
            </Button>
          </CardActions>
          <NowPlaying {...this.props} />
        </Card>
      );
    }

    const c = currentFile;
    const background = (
      <FlexibleBackground
        showGradients={showBackground}
        single={useSingleGradient}
        selector=".page-bg"
        image={coverArt}
        showBackground={true}
        fadeBackground={!showBackground}
      />
    );

    // const background = useGradients || useSingleGradient ? (
    //   <ColorContainer>
    //     <GradientBackground showGradients={showBackground} single={useSingleGradient} />
    //   </ColorContainer>
    // ) : useFuzzyImage ? (
    //     <FuzzyImageBackground
    //       selector=".page-bg"
    //       image={coverArt}
    //       showBackground={true}
    //       fadeBackground={!showBackground}
    //     />
    // ) : (
    //   <span />
    // );

    const nowPlaying = subsonic.following ? <NowPlaying {...this.props} /> : '';
    /*
<IconButton onClick={this.handleSongDrawer}>
  <MenuIcon/>
</IconButton>*/

    return (
      <React.Fragment>
        <IconButton onClick={this.handleSongDrawer}>
          <MenuIcon />
        </IconButton>
        <PlayPauseButton
          {...csProps}
          fab={true}
          standalone={true}
          color="primary"
        />
        <PrevButton
          {...csProps}
          buttonClassName="button-class"
          iconClassName="icon-class"
        />
        <NextButton {...csProps} />
        <SkipAlbumButton {...csProps} />
        <Replay10Button {...csProps} />
        <Forward30Button {...csProps} />
        <BookmarkButton
          {...csProps}
          isBookmarked={subsonic.bookmarksById[id]}
          action={this.toggleBookmark}
        />
        <SavePlayQueueButton {...csProps} />
        <ShuffleButton />
        <ChromecastButton {...csProps} />
        <RemoteReceiverToggleButton {...csProps} {...this.props} />
        <RangeSlider {...csProps} seekValueChange={this.seekValueChange} />
        <Notification
          {...csProps}
          mediaIcon={this.state.mediaIcon}
          song={this.state.currentFile}
        />
        <SongListDrawer
          {...csProps}
          open={isShowingMenu}
          classes={{}}
          subsonic={subsonic}
          handleClose={this.handleSongDrawerClose}
          handleSongClick={this.handleSongClick}
          anchor="left"
        />
        <GradientGeneratingImage
          src={coverArt}
          size={coverSize}
          imageId="cover-image"
        />

        <HollowCard>
          {c.title}&nbsp;({currentPlaying.idx + 1}/{' '}
          {currentPlaying.queue.length})
          <br /> {c.artist}
          <br /> {c.album}&nbsp;({c.year})
          <br /> {currentMediaTimeString}&nbsp;-&nbsp;{durationString}
        </HollowCard>

        {nowPlaying}
        {background}
      </React.Fragment>
    );
  }
}

export default DemoPlayer;
