import React from 'react';
import connectToStores from 'utils/connectToStores';
import { backgroundTypes } from 'subfire/actions/PreferenceActions';
import PreferenceStore from 'subfire/stores/PreferenceStore';

import ColorContainer from 'subfire/components/containers/ColorContainer';
import GradientBackground from 'subfire/components/GradientBackground';
import FuzzyImageBackground from 'subfire/components/FuzzyImageBackground';

class FlexibleBackground extends React.Component {
  static getStores(props) {
    return [PreferenceStore];
  }
  static getPropsFromStores(props) {
    return PreferenceStore.getState();
  }

  render() {
    const { backgroundType, showGradients } = this.props;

    let background = <span />,
      single = false;
    switch (backgroundType) {
      case backgroundTypes.singleGradient:
        single = true;
        background = (
          <ColorContainer>
            <GradientBackground showGradients={showGradients} single={single} />
          </ColorContainer>
        );
        break;
      case backgroundTypes.gradients:
        background = (
          <ColorContainer>
            <GradientBackground showGradients={showGradients} single={single} />
          </ColorContainer>
        );
        break;
      case backgroundTypes.fuzzyImage:
        const { selector, image, showBackground, fadeBackground } = this.props;
        background = (
          <FuzzyImageBackground
            selector={selector}
            image={image}
            showBackground={showBackground}
            fadeBackground={fadeBackground}
          />
        );
        break;
      default:
        console.warn('default === span for black');
    }
    return background;
  }
}

export default connectToStores(FlexibleBackground);
