import React from 'react';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogContentText from '@material-ui/core/DialogContentText';
import DialogTitle from '@material-ui/core/DialogTitle';
import Button from '@material-ui/core/Button';
import Select from '@material-ui/core/Select';
import InputLabel from '@material-ui/core/InputLabel';
import MenuItem from '@material-ui/core/MenuItem';
import FormControl from '@material-ui/core/FormControl';
import { withStyles } from '@material-ui/core/styles';

import TVResponsiveDialog from 'subsonic/components/hoc/TVResponsiveDialog';

import connectToStores from 'utils/connectToStores';
import PreferenceStore from 'subfire/stores/PreferenceStore';
import PreferenceActions, {
  backgroundTypes
} from 'subfire/actions/PreferenceActions';

class PreferencesStores {
  static getStores(props) {
    return [PreferenceStore];
  }
  static getPropsFromStores(props) {
    return PreferenceStore.getState();
  }
}

class Preferences extends React.Component {
  onBackgroundTypeChange = e => {
    const value = e.target.value;
    PreferenceActions.setBackgroundType(value);
  };

  handleClose = () => {
    if (this.props.handleClose) this.props.handleClose();
    if (this.props.history) this.props.history.goBack();
  };

  render() {
    const { classes, backgroundType, native } = this.props;

    return (
      <TVResponsiveDialog open={true} disableBackdropClick={true}>
        <DialogTitle>{'SubFire Standard Preferences'}</DialogTitle>
        <DialogContent>
          <DialogContentText>
            The following preferences can be modified:
          </DialogContentText>

          <FormControl className={classes.formControl}>
            <InputLabel htmlFor="current-selection">Background Type</InputLabel>
            <Select
              native={native}
              id="current-selection"
              value={backgroundType}
              onChange={this.onBackgroundTypeChange}
            >
              {Object.entries(backgroundTypes).map(o => {
                if (native)
                  return (
                    <option key={o[1]} value={o[1]}>
                      {o[1]}
                    </option>
                  );
                return (
                  <MenuItem key={o[1]} value={o[1]}>
                    {o[1]}
                  </MenuItem>
                );
              })}
            </Select>
          </FormControl>
        </DialogContent>
        <DialogActions>
          <Button onClick={this.handleClose}>Close</Button>
        </DialogActions>
      </TVResponsiveDialog>
    );
  }
}

const styles = theme => ({
  root: {
    display: 'flex',
    flexWrap: 'wrap'
  },
  formControl: {
    margin: 0,
    width: '100%'
  },
  selectEmpty: {
    marginTop: theme.spacing(2)
  }
});

export default withStyles(styles)(
  connectToStores(PreferencesStores, Preferences)
);
