import React, { Component } from 'react';
import { Link } from 'react-router-dom';

import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';

import FormControl from '@material-ui/core/FormControl';
import Input from '@material-ui/core/Input';
import InputLabel from '@material-ui/core/InputLabel';
import Select from '@material-ui/core/Select';
import MenuItem from '@material-ui/core/MenuItem';

import Paper from '@material-ui/core/Paper';
import Typography from '@material-ui/core/Typography';

import Grid from '@material-ui/core/Grid';

import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';
import IconButton from '@material-ui/core/IconButton';

import GridList from '@material-ui/core/GridList';
import GridListTile from '@material-ui/core/GridListTile';
import ListSubheader from '@material-ui/core/ListSubheader';
import ArrowUpward from '@material-ui/icons/ArrowUpward';
import ArrowDownward from '@material-ui/icons/ArrowDownward';
import PlayIcon from '@material-ui/icons/PlayArrow';
import { withStyles } from '@material-ui/core/styles';

import CoverCard from 'subsonic/components/styled/CoverCard';
import CoverGridTile from 'subsonic/components/styled/CoverGridTile';
import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

const styles = theme => ({
  root: {
    // display: "flex",
    // flexWrap: "wrap",
    // justifyContent: "space-around",
    // overflow: "hidden",
    // paddingLeft: 10
  },
  header: {
    backgroundColor: theme.palette.background.paper,
    marginTop: '10px'
  },
  gridList: {
    backgroundColor: '#000000'
  },
  icon: {
    color: 'rgba(255, 255, 255, 0.54)'
  }
});

class RadioBrowser extends Component {
  constructor(props) {
    super(props);
    this.myRef = React.createRef();
  }

  static renderTypes = {
    GRID: 'grid',
    TABLE: 'table',
    LIST: 'list',
    SELECT: 'select',
    CARDS: 'cards'
  };

  updateDimensions = () => {
    this.setState({ width: window.innerWidth, height: window.innerHeight });
  };

  componentDidMount = () => {
    window.addEventListener('resize', this.updateDimensions);
    this.updateDimensions();
  };

  componentWillUnmount = () => {
    window.removeEventListener('resize', this.updateDimensions);
  };

  stationCategory(s) {
    return s.stationData.radiostation.o
      ? s.stationData.radiostation.o.t
      : 'zzzzz no class';
  }

  scrollIntoView = i => {
    var next = 'grid-category-header' + (i * 1 + 1);
    console.warn(next, document.getElementById(next));
    document.getElementById(next).scrollIntoView({
      block: 'start',
      inline: 'nearest',
      behavior: 'smooth'
    });
  };

  render() {
    // console.warn('pl render', this.props);
    const loadPlaylistContents = this.props.loadPlaylistContents;
    const history = this.props.history;
    const playlistRouting = {
      routeToSelectId: id => {
        const link = loadPlaylistContents
          ? '/loading/playlist/' + id + '/list'
          : '/loading/station/' + id;
        history.push(link);
      },
      routeToSelectValue: evt => {
        const id = evt.target.value;
        if (!id) return;
        const link = loadPlaylistContents
          ? '/loading/playlist/' + id + '/list'
          : '/loading/station/' + id;
        history.push(link);
      },
      PlayerRefLink: React.forwardRef((props, ref) => {
        const link = '/loading/station/' + props.id;
        return (
          <Link innerRef={ref} to={link}>
            {props.children}
          </Link>
        );
      })
    };

    const {
      subsonic,
      renderMode,
      selectedId,
      SubsonicSource,
      classes,
      native,
      fullWidth,
      label,
      includeEmpty,
      renderEmptyMessage
    } = this.props;
    const { stations } = subsonic;
    const {
      routeToSelectId,
      routeToSelectValue,
      PlayerRefLink
    } = playlistRouting;
    const rm = renderMode || RadioBrowser.renderTypes.TABLE;

    var that = this;
    var pl = stations.sort(function(s1, s2) {
      var c1 = that.stationCategory(s1);
      var c2 = that.stationCategory(s2);
      var c = c1.localeCompare(c2);
      if (c === 0) return s1.name.localeCompare(s2.name);
      return c;
    });
    let rv;

    if (!pl || pl.length === 0) {
      let message = 'No Stations have been added to this server.';
      if (!subsonic.isConnected) {
        message = 'Not yet connected...';
      } else if (subsonic.isBusy) {
        message = 'Still loading...';
      }
      rv = renderEmptyMessage ? (
        renderEmptyMessage(message)
      ) : (
        <Paper>
          <Typography variant="h4">{message}</Typography>
        </Paper>
      );
    } else {
      const sortedStations = pl.reduce(
        (rv, n, i, ss) => {
          const thisCat = n.stationData.radiostation.o
            ? n.stationData.radiostation.o.t
            : 'Radio Stations';
          if (rv.categories.indexOf(thisCat) === -1) {
            rv.categories.push(thisCat);
            rv.categoryStations[thisCat] = [];
          }
          rv.categoryStations[thisCat].push(n);
          return rv;
        },
        { categories: [], categoryStations: {} }
      );

      if (rm === RadioBrowser.renderTypes.CARDS) {
        const innerWidth = window.innerWidth;

        // const spacing = innerWidth > 560 ? 20 : 6;
        const cellSize = innerWidth > 560 ? 250 : 164;
        // const cols = innerWidth / (cellSize + spacing);
        // const width = cols * (cellSize + spacing);

        rv = (
          <Grid container className={classes.root}>
            <Grid item xs={12} className="card-grid">
              {sortedStations.categories.map(c =>
                sortedStations.categoryStations[c].map((n, i) => (
                  <CoverCard
                    key={n.id}
                    name={n.name}
                    size={cellSize}
                    coverArt={n.coverArt}
                    linkComponent={PlayerRefLink}
                    id={n.id}
                    category={c}
                    link={'/loading/station/' + n.id}
                    SubsonicSource={SubsonicSource}
                  />
                ))
              )}
            </Grid>
          </Grid>
        );
      } else if (rm === RadioBrowser.renderTypes.GRID) {
        let innerWidth = window.innerWidth;
        if (this.myRef && this.myRef.current) {
          innerWidth = this.myRef.current.offsetWidth;
        }
        if (isNaN(innerWidth)) innerWidth = 360;

        const spacing = innerWidth > 360 ? 10 : 6;
        const cellSize = innerWidth > 360 ? 170 : 164;
        const cols = innerWidth / (cellSize + spacing);
        const width = cols * (cellSize + spacing);

        const sortedStations = pl.reduce(
          (rv, n, i, ss) => {
            const thisCat = n.stationData.radiostation.o
              ? n.stationData.radiostation.o.t
              : 'Radio Stations';
            if (rv.categories.indexOf(thisCat) === -1) {
              rv.categories.push(thisCat);
              rv.categoryStations[thisCat] = [];
            }
            rv.categoryStations[thisCat].push(n);
            return rv;
          },
          { categories: [], categoryStations: {} }
        );

        rv = (
          <div className={classes.root} ref={this.myRef}>
            {sortedStations.categories.map((c, i, a) => (
              <GridList
                key={'category-' + c}
                className={classes.gridList}
                cellHeight={cellSize}
                style={{ width: width }}
                cols={cols}
                spacing={spacing}
              >
                <GridListTile
                  className={classes.header}
                  cols={cols}
                  style={{ height: 'auto' }}
                >
                  <ListSubheader
                    id={'grid-category-header' + i}
                    component="div"
                  >
                    <Typography variant="h4">{c}</Typography>
                    {i + 1 < a.length && (
                      <ListItemSecondaryAction
                        style={{ width: 48, height: 48 }}
                      >
                        <IconButton
                          aria-label="Next"
                          onClick={this.scrollIntoView.bind(this, i)}
                        >
                          <ArrowDownward />
                        </IconButton>
                      </ListItemSecondaryAction>
                    )}
                    {i + 1 === a.length && (
                      <ListItemSecondaryAction
                        style={{ width: 48, height: 48 }}
                      >
                        <IconButton
                          aria-label="First"
                          onClick={this.scrollIntoView.bind(this, -1)}
                        >
                          <ArrowUpward />
                        </IconButton>
                      </ListItemSecondaryAction>
                    )}
                  </ListSubheader>
                </GridListTile>
                {sortedStations.categoryStations[c].map(n => (
                  <CoverGridTile
                    onClick={routeToSelectId.bind({}, n.id)}
                    key={n.id}
                    name={n.name}
                    size={cellSize}
                    coverArt={n.coverArt}
                    linkComponent={PlayerRefLink}
                    id={n.id}
                    SubsonicSource={SubsonicSource}
                  />
                ))}
              </GridList>
            ))}
            <ScrollToTopFab {...this.props.fabProps} />
          </div>
        );
      } else if (rm === RadioBrowser.renderTypes.LIST) {
        rv = (
          <div className={classes.root} ref={this.myRef}>
            <List>
              {sortedStations.categories.map((c, i, a) => (
                <React.Fragment key={c}>
                  <ListSubheader
                    id={'grid-category-header' + i}
                    component="div"
                  >
                    <Typography variant="h4">{c}</Typography>
                    {i + 1 < a.length && (
                      <ListItemSecondaryAction
                        style={{ width: 48, height: 48 }}
                      >
                        <IconButton
                          aria-label="Next"
                          onClick={this.scrollIntoView.bind(this, i)}
                        >
                          <ArrowDownward />
                        </IconButton>
                      </ListItemSecondaryAction>
                    )}
                    {i + 1 === a.length && (
                      <ListItemSecondaryAction
                        style={{ width: 48, height: 48 }}
                      >
                        <IconButton
                          aria-label="First"
                          onClick={this.scrollIntoView.bind(this, -1)}
                        >
                          <ArrowUpward />
                        </IconButton>
                      </ListItemSecondaryAction>
                    )}
                  </ListSubheader>
                  {sortedStations.categoryStations[c].map(n => (
                    <ListItem
                      key={'rs' + n.id}
                      button
                      onClick={routeToSelectId.bind({}, n.id)}
                    >
                      <ListItemIcon>
                        <img
                          alt=""
                          width={48}
                          height={48}
                          src={SubsonicSource.getCoverArtURL(n.coverArt, 48)}
                        />
                      </ListItemIcon>
                      <ListItemText
                        primary={n.name}
                        secondary={n.songCount + ' songs'}
                      />
                      <ListItemSecondaryAction
                        style={{ width: 48, height: 48 }}
                      >
                        <IconButton
                          aria-label="Play"
                          component={PlayerRefLink}
                          id={n.id}
                        >
                          <PlayIcon />
                        </IconButton>
                      </ListItemSecondaryAction>
                    </ListItem>
                  ))}
                </React.Fragment>
              ))}
            </List>
          </div>
        );
      } else if (rm === RadioBrowser.renderTypes.TABLE) {
        rv = (
          <Table>
            <TableBody>
              {pl.map(n => {
                const selected = '' + n.id === '' + selectedId;
                return (
                  <TableRow selected={selected} key={'pl' + n.id}>
                    <TableCell>
                      <PlayerRefLink id={n.id}>
                        <img
                          src={this.props.SubsonicSource.getCoverArtURL(
                            n.coverArt,
                            40
                          )}
                          alt={n.name}
                        />
                      </PlayerRefLink>
                    </TableCell>
                    <TableCell>
                      <PlayerRefLink id={n.id}>{n.name}</PlayerRefLink>
                    </TableCell>
                  </TableRow>
                );
              })}
            </TableBody>
          </Table>
        );
      } else if (rm === RadioBrowser.renderTypes.SELECT && native) {
        rv = (
          <FormControl className={classes.formControl} fullWidth={fullWidth}>
            <InputLabel htmlFor="radio-label">{label}</InputLabel>
            <Select
              fullWidth={fullWidth}
              native
              value={'' + selectedId}
              onChange={routeToSelectValue}
              input={<Input id="radio-label" />}
            >
              {includeEmpty && <option value="" key="empty-value" />}
              {sortedStations.categories.map((c, i, a) => (
                <optgroup label={c} key={'cat-' + c}>
                  {sortedStations.categoryStations[c].map(n => (
                    <option value={'' + n.id} key={'pl' + n.id}>
                      {n.name}
                    </option>
                  ))}
                </optgroup>
              ))}
            </Select>
          </FormControl>
        );
      } else if (rm === RadioBrowser.renderTypes.SELECT && !native) {
        var v = [];
        if (includeEmpty) {
          v.push(<MenuItem value="" key="empty-value" />);
        }
        sortedStations.categories.forEach((c, i, a) => {
          v.push(<ListSubheader key={'cat-' + c}>{c}</ListSubheader>);
          sortedStations.categoryStations[c].forEach(n =>
            v.push(
              <MenuItem value={'' + n.id} key={'pl' + n.id}>
                <img
                  crossOrigin="anonymous"
                  src={this.props.SubsonicSource.getCoverArtURL(n.coverArt, 15)}
                  alt={pl.name}
                />
                &nbsp;
                {n.name}
              </MenuItem>
            )
          );
        });

        rv = (
          <FormControl className={classes.formControl} fullWidth={fullWidth}>
            <InputLabel htmlFor="radio-label">{label}</InputLabel>
            <Select
              fullWidth={fullWidth}
              value={'' + selectedId}
              onChange={routeToSelectValue}
              input={<Input id="radio-label" />}
            >
              {v}
            </Select>{' '}
          </FormControl>
        );
      }
    }
    return rv;
  }
}

export default withStyles(styles)(RadioBrowser);
