import React, { Component } from 'react';

// import rangeSliderColor from "subsonic/components/controls/RangeSliderColors";

import { MuiThemeProvider, createMuiTheme } from '@material-ui/core/styles';
import { colorToMaterial } from 'utils/material-color';

import amber from '@material-ui/core/colors/amber';
import blue from '@material-ui/core/colors/blue';
import blueGrey from '@material-ui/core/colors/blueGrey';
import brown from '@material-ui/core/colors/brown';
import cyan from '@material-ui/core/colors/cyan';
import deepOrange from '@material-ui/core/colors/deepOrange';
import deepPurple from '@material-ui/core/colors/deepPurple';
import green from '@material-ui/core/colors/green';
import grey from '@material-ui/core/colors/grey';
import indigo from '@material-ui/core/colors/indigo';
import lightBlue from '@material-ui/core/colors/lightBlue';
import lightGreen from '@material-ui/core/colors/lightGreen';
import lime from '@material-ui/core/colors/lime';
import orange from '@material-ui/core/colors/orange';
import pink from '@material-ui/core/colors/pink';
import purple from '@material-ui/core/colors/purple';
import red from '@material-ui/core/colors/red';
import teal from '@material-ui/core/colors/teal';
import yellow from '@material-ui/core/colors/yellow';

const colors = {
  amber,
  blue,
  blueGrey,
  brown,
  cyan,
  deepOrange,
  deepPurple,
  green,
  grey,
  indigo,
  lightBlue,
  lightGreen,
  lime,
  orange,
  pink,
  purple,
  red,
  teal,
  yellow
};

const indexes = [
  '50',
  '100',
  '200',
  '300',
  '400',
  '500',
  '600',
  '700',
  '800',
  '900'
];

const aIndex = ['A100', 'A200', 'A400', 'A700'];

function FindIndexes(key) {
  let m, l, d;
  if (key.startsWith('A')) {
    if (key === 'A100') key = '200';
    if (key === 'A200') key = '400';
    if (key === 'A400') key = '500';
    if (key === 'A700') key = '700';
  }

  m = key;
  const i = indexes.indexOf(m);
  l = i - 2;
  if (l < 0) l = 0;
  l = indexes[l];

  d = i + 2;
  if (d < 9) d = 9;
  d = indexes[d];

  return { m, l, d };
}

function parseColor(cname) {
  let c = cname.split('-');
  if (c.length === 2) {
    // console.log(c);
  } else {
    console.warn('oops', c);
  }
  return c;
}

function genPaletteFromColor(c, p) {
  const mc = colorToMaterial(...c);
  let colorName = mc.closestMaterialName;
  // console.log(colorName);
  // brown is boring...
  if (colorName.startsWith('brown')) {
    colorName = colorToMaterial(...p[3]).closestMaterialName;
    // console.log('replaced with ', colorName)
  }
  // console.log(mc);
  let colorKeys = parseColor(colorName);
  if (colorKeys.length === 2) {
    if (colorKeys[0] === 'black') colorKeys = ['grey', '800'];
    if (colorKeys[0] === 'white') colorKeys = ['grey', '400'];
    const ac = colors[colorKeys[0]];
    // console.log(ac);
    const idx = FindIndexes(colorKeys[1]);
    // if (!ac || !idx) {
    //   debugger;
    // }
    return {
      light: ac[idx.l],
      main: ac[idx.m],
      dark: ac[idx.d]
    };
  } else {
    return null;
  }
}

class ColorTheme extends Component {
  render() {
    const {
      palette,
      defaultPrimary,
      defaultSecondary,
      typography = {},
      overrides = {}
    } = this.props;

    // hack
    overrides.MuiSnackbarContent = {
      root: {
        color: 'white'
      }
    };

    let primary = defaultPrimary || colors.amber,
      secondary = defaultSecondary || colors.indigo;

    if (palette && palette.length) {
      primary = genPaletteFromColor(palette[0], palette) || primary;
      secondary = genPaletteFromColor(palette[1], palette) || secondary;
    } else {
      // love to find the best place for this
      document.getElementsByTagName('body')[0].style.cssText = '';
    }

    const typographyConfig = Object.assign({}, typography, {});

    const config = {
      palette: {
        primary: primary,
        secondary: secondary,
        type: 'dark'
      },
      typography: typographyConfig,
      overrides: overrides
    };
    const theme = createMuiTheme(config);

    return (
      <MuiThemeProvider theme={theme}>{this.props.children}</MuiThemeProvider>
    );
  }
}

export default ColorTheme;
