import React from 'react';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogContentText from '@material-ui/core/DialogContentText';
import DialogTitle from '@material-ui/core/DialogTitle';
import Button from '@material-ui/core/Button';

import TVResponsiveDialog from 'subsonic/components/hoc/TVResponsiveDialog';

import CredentialsActions from 'subsonic/actions/CredentialsActions';
import AuthExchangeActions from 'subfire/actions/AuthExchangeActions';

import Tea from 'utils/Tea';

import { withStyles } from '@material-ui/core/styles';
const styles = theme => ({
  centerHeader: {
    textAlign: 'center',
    fontSize: '40pt',
    letterSpacing: '.5ch',
    color: 'white'
  }
});

class AuthCodeClient extends React.Component {
  componentDidMount() {
    const { digits, route } = this.props;
    AuthExchangeActions.clientRequest(digits)
      .then(function(response, err) {
        if (response) console.log(response.ok);
        if (err) console.error(err);

        if (response.ok) return response.json();
        return response.json();
      })
      .then(function(j) {
        console.log('back', j);
        if (j.status === 200) {
          // if parse failure, let the exception fire
          let data = Tea.decrypt(j.data, digits);
          //console.log(data);
          data = JSON.parse(data);
          //console.log(data);

          // localStorage.setItem("subsonic.credentials", data.configs);
          const configs = JSON.parse(data.configs);
          //console.log(configs);
          CredentialsActions.importCredentials(configs);
          route.history.goBack();
          // setTimeout(document.location.reload.bind(window.location), 100);
        }
      })
      .catch(function(err) {
        console.error(err);
      });
  }

  buttonWasMounted(ref) {
    if (ref) ref.focus();
  }

  render() {
    const { digits, classes, handleClose } = this.props;

    return (
      <TVResponsiveDialog
        disableBackdropClick
        open={true}
        className={classes.paperFullScreen}
      >
        <DialogTitle>{'Authenticate Via Existing Source'}</DialogTitle>
        <DialogContent>
          <DialogContentText>
            On another device where you are already authenticated, enter the
            following {(digits + '').length} digits.
          </DialogContentText>
          <h1 className={classes.centerHeader}>{digits}</h1>
        </DialogContent>
        <DialogActions>
          <Button buttonRef={this.buttonWasMounted} onClick={handleClose}>
            Cancel
          </Button>
        </DialogActions>
      </TVResponsiveDialog>
    );
  }
}

export default withStyles(styles)(AuthCodeClient);
