import React, { Component } from 'react';
import Grid from '@material-ui/core/Grid';
import { withStyles } from '@material-ui/core/styles';

import { emptytrack } from 'subsonic/stores/SubsonicStore';

import {
  PlayPauseButton,
  NextButton,
  PrevButton,
  SkipAlbumButton,
  Replay10Button,
  Forward30Button,
  ShuffleButton,
  BookmarkButton,
  SavePlayQueueButton
} from 'subsonic/components/controls/MediaButtons';
import Notification from 'subsonic/components/controls/Notification';
import SubsonicActions from 'subsonic/actions/SubsonicActions';
import ColorActions from 'subfire/actions/ColorActions';

import GradientGeneratingImage from 'subfire/components/GradientGeneratingImage';
import GradientBackground from 'subfire/components/GradientBackground';
import FuzzyImageBackground from 'subfire/components/FuzzyImageBackground';

import RangeSlider from 'subsonic/components/controls/RangeSlider';
import CastSubsonic from 'subsonic/components/containers/CastSubsonic';

import ColorContainer from 'subfire/components/containers/ColorContainer';

import HollowCard from 'subsonic/components/styled/HollowCard';

const styles = {
  base: {
    position: 'fixed',
    bottom: 5,
    left: 5,
    right: 5,
    marginLeft: 5,
    marginRight: 5,
    marginBottom: 5
  },
  text: {
    fontSize: 'smaller'
  }
};

class FooterPlayer extends Component {
  static getDerivedStateFromProps(props, prevState) {
    const { subsonic, SubsonicSource } = props;
    const { currentPlaying } = subsonic;

    let newState;

    if (!currentPlaying.queue.length) {
      newState = {
        showBackground: false,
        currentFile: emptytrack,
        coverArt: SubsonicSource.getCoverArtURL(emptytrack.id, 60)
      };
    } else {
      const currentFile = currentPlaying.queue[currentPlaying.idx];
      if (!prevState || currentFile !== prevState.currentFile) {
        const isWaitingForNext = false;
        const coverArt = SubsonicSource.getCoverArtURL(currentFile.id, 60);
        const mediaSrc =
          currentFile.localPath ||
          (currentFile.src && currentFile.src.startsWith('http')
            ? currentFile.src
            : SubsonicSource.getStreamingURL(currentFile.id));
        const mediaIcon =
          SubsonicSource.getCoverArtURL(currentFile.id, 192) + '&co=false';

        // now load castplayer
        currentFile.coverArtURL = coverArt;
        currentFile.src = mediaSrc;
        window.cp.playerHandler.load(currentFile, 0);
        // console.warn(currentFile);
        newState = {
          currentFile,
          coverArt,
          mediaSrc,
          mediaIcon,
          isWaitingForNext
        };
      }
    }
    return Object.assign({}, prevState, newState);
  }

  constructor(props) {
    super(props);
    this.state = {};
    // this.state = FooterPlayer.getDerivedStateFromProps(props);
  }

  audioUpdates = p => {
    // hack - no easy way to pass this around
    if (window.bookmarkSeekTo) {
      window.castSubsonic.seekMedia(window.bookmarkSeekTo);
      window.bookmarkSeekTo = null;
    }

    if (this.state.showBackground !== p.isPlaying) {
      this.setState({ showBackground: p.isPlaying });
    }

    // if (p.reason !== 'normal') console.log('p', p);

    if (!this.state.isWaitingForNext && p.reason === 'playbackEnded') {
      console.log('endPlayback next...');
      this.setState({ isWaitingForNext: true });
      SubsonicActions.playerNext.defer();
    }
  };

  // componentWillReceiveProps(nextProps) {
  //   this.setState(FooterPlayer.getDerivedStateFromProps(nextProps, this.state));
  // }

  // not every player will do this - if we keep an inline playing frame a-la Plex, then this needs to not happen here.
  // stop: control the player because of internal state; pause = toggle autoplay
  componentWillUnmount() {
    window.cp.playerHandler.stop();
    ColorActions.resetColors();
  }

  toggleBookmark = currentMediaTime => {
    // console.log('toggleBookmark', currentMediaTime, this.props);
    const { subsonic, SubsonicSource } = this.props;
    const { currentPlaying } = subsonic;
    const id = currentPlaying.queue[currentPlaying.idx].id;

    if (subsonic.bookmarksById[id]) {
      SubsonicActions.deleteBookmark(id);
    } else {
      SubsonicActions.createBookmark(id, Math.trunc(currentMediaTime * 1000));
    }
  };

  render() {
    const {
      subsonic,
      SubsonicSource,
      useGradients,
      useFuzzyImage,
      classes
    } = this.props;
    const { currentPlaying } = subsonic;
    const id =
      currentPlaying &&
      currentPlaying.queue &&
      currentPlaying.queue[currentPlaying.idx]
        ? currentPlaying.queue[currentPlaying.idx].id
        : '-1';

    const { showBackground, currentFile, coverArt } = this.state;
    const c = currentFile;
    const background = useGradients ? (
      <ColorContainer>
        <GradientBackground showGradients={showBackground} />
      </ColorContainer>
    ) : useFuzzyImage ? (
      <FuzzyImageBackground
        selector=".page-bg"
        image={coverArt}
        showBackground={showBackground}
      />
    ) : (
      <span />
    );

    return (
      <div className={classes.base}>
        {background}
        <HollowCard>
          <Grid container spacing={24}>
            <Grid item xs={1}>
              <GradientGeneratingImage src={coverArt} item xs={6} sm={3} />
            </Grid>
            <Grid item xs={3} className={classes.text}>
              {c.title} ({currentPlaying.idx + 1} /{' '}
              {currentPlaying.queue.length})
              <br />
              {c.artist}
              <br />
              {c.album} ({c.year})
            </Grid>
            <Grid item xs={8}>
              <CastSubsonic updateHandler={this.audioUpdates}>
                <PrevButton />
                <Replay10Button />
                <PlayPauseButton fab />
                <Forward30Button />
                <NextButton />
                <SkipAlbumButton />
                <Notification
                  mediaIcon={this.state.mediaIcon}
                  song={this.state.currentFile}
                />
                <BookmarkButton
                  isBookmarked={subsonic.bookmarksById[id]}
                  action={this.toggleBookmark}
                />
                <SavePlayQueueButton />
                <ShuffleButton />

                <RangeSlider />
              </CastSubsonic>
            </Grid>
          </Grid>
        </HollowCard>
      </div>
    );
  }
}

export default withStyles(styles)(FooterPlayer);
