import React, { Component } from 'react';

import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogContentText from '@material-ui/core/DialogContentText';
import DialogTitle from '@material-ui/core/DialogTitle';

import Button from '@material-ui/core/Button';
import TextField from '@material-ui/core/TextField';
import { withStyles } from '@material-ui/core/styles';
import TVResponsiveDialog from 'subsonic/components/hoc/TVResponsiveDialog';

import RemotesActions, { RemoteStates } from 'subfire/actions/RemotesActions';
import RemoteReceiverToggleButton from './RemoteReceiverToggleButton';
const styles = theme => ({});

class RemoteConfigScreen extends Component {
  static getDerivedStateFromProps(props, prevState) {
    return {
      remoteName: prevState.remoteNameChanged
        ? prevState.remoteName
        : props.remotes.remoteName,
      remoteNameChanged: prevState.remoteNameChanged
    };
  }

  constructor(props) {
    super(props);
    this.state = {
      remoteNameChanged: false,
      remoteName: props.remotes.remoteName
    };
  }

  enableRemoteReceiver = () => {
    var remoteName = this.state.remoteName;
    this.setState({ remoteName: remoteName });
    RemotesActions.enableRemoteReceiver(
      remoteName,
      this.props.SubsonicSource,
      this.props.history
    );
  };

  disableRemoteReceiver = () => {
    RemotesActions.disableRemoteReceiver();
  };

  remoteNameChange = e => {
    this.setState({
      remoteName: e.target.value,
      remoteNameChanged: true
    });
  };

  render() {
    const { classes, history, remotes } = this.props;
    const remoteEnabled =
      remotes.remoteState === RemoteStates.receiver ||
      remotes.remoteState === RemoteStates.receiverPaused;
    // console.log(remotes, remoteEnabled);
    const value = this.state.remoteName || '';
    return (
      <TVResponsiveDialog
        disableBackdropClick
        open={true}
        className={classes.paperFullScreen}
      >
        <DialogTitle>Remote Receiver Configuration</DialogTitle>
        <DialogContent>
          <DialogContentText>Name This Client</DialogContentText>
          <TextField
            label="zz-sbr-"
            value={value}
            className={classes.textField}
            onChange={this.remoteNameChange}
            margin="normal"
          />
        </DialogContent>
        <DialogActions>
          <RemoteReceiverToggleButton {...this.props} />
          <Button
            size="small"
            onClick={
              remoteEnabled
                ? this.disableRemoteReceiver
                : this.enableRemoteReceiver
            }
            disabled={value === ''}
          >
            {remoteEnabled ? 'Disable' : 'Enable'}
          </Button>
          <Button size="small" onClick={history.goBack}>
            Close
          </Button>
        </DialogActions>
      </TVResponsiveDialog>
    );
  }
}

export default withStyles(styles)(RemoteConfigScreen);
