import React from 'react';
import ActionButton from 'subsonic/components/controls/ActionButton';

import { Airplay } from '@material-ui/icons';
import { withStyles } from '@material-ui/core/styles';

import RemotesActions, { RemoteStates } from 'subfire/actions/RemotesActions';

const style = theme => {
  return {
    enabled: {
      color: theme.palette.primary.light
    },
    paused: {
      color: '#999999'
    },
    disabled: {
      display: 'none'
    }
  };
};

const RemoteReceiverToggleButton = props => {
  if (!props.remotes) return null;
  const remoteEnabled = props.remotes.remoteState === RemoteStates.receiver;
  const remotePaused =
    props.remotes.remoteState === RemoteStates.receiverPaused;
  const disabled = !remoteEnabled && !remotePaused;

  const icon = <Airplay className={props.iconClassName} />;
  const label = remoteEnabled ? 'Disable' : 'Enable';
  const action = remoteEnabled
    ? RemotesActions.pauseRemoteReceiver
    : RemotesActions.resumeRemoteReceiver;
  const classes = props.classes;
  const coreProps = {
    action: action,
    label: label,
    disabled: props.isLoading,
    fab: props.fab,
    standalone: props.standalone,
    buttonClassName: disabled
      ? props.classes.disabled
      : remoteEnabled
      ? props.classes.enabled
      : props.classes.paused
  };
  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export default withStyles(style)(RemoteReceiverToggleButton);
