import React, { Component } from 'react';

import IconButton from '@material-ui/core/IconButton';
import Menu from '@material-ui/core/Menu';
import MenuItem from '@material-ui/core/MenuItem';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import { withStyles } from '@material-ui/core/styles';

import { Phonelink } from '@material-ui/icons';

import RemotesActions, { RemoteStates } from 'subfire/actions/RemotesActions';
const ITEM_HEIGHT = 48;

const styles = theme => ({
  listItemReceiver: {
    display: 'none'
  },
  receiver: {
    display: 'none'
  },
  sending: {
    color: theme.palette.primary.light
  },
  normal: {
    color: '#999999'
  }
});

class RemoteSenderMenu extends Component {
  state = {
    anchorEl: null
  };

  handleClick = event => {
    this.setState({ anchorEl: event.currentTarget });
  };

  handleClose = () => {
    this.setState({ anchorEl: null });
  };

  handleMenuClick = playlist => {
    console.log('playlist', playlist, this.props.SubsonicSource);
    RemotesActions.enableSending(playlist, this.props.SubsonicSource);
    this.setState({ anchorEl: null });
  };

  handleDisableClick = () => {
    RemotesActions.disableSending();
  };

  render() {
    // console.warn(this.props);
    const { classes, remotes, variant = 'button' } = this.props;
    const { anchorEl } = this.state;
    const options = remotes.receivers;
    const isReceiver =
      remotes.remoteState === RemoteStates.receiver ||
      remotes.remoteState === RemoteStates.receiverPaused;
    const isSending = remotes.remoteState === RemoteStates.sender;
    // console.warn('props', this.props);
    let className, liClassName;
    if (isReceiver) {
      className = classes.receiver;
      liClassName = classes.listItemReceiver;
    } else if (isSending) {
      className = classes.sending;
    } else if (options.length === 0) {
      className = classes.receiver; // nobody to see
      liClassName = classes.listItemReceiver;
    } else {
      className = classes.normal;
    }

    const TheIcon = isSending ? Phonelink : Phonelink;
    // console.log('className', className, isReceiver, isSending, options.length, classes);

    let rootOn, rootOff;
    if (variant === 'button') {
      rootOn = (
        <IconButton className={className} onClick={this.handleDisableClick}>
          <TheIcon />
        </IconButton>
      );
      rootOff = (
        <IconButton
          aria-label="show list"
          aria-owns={anchorEl ? 'long-menu' : null}
          aria-haspopup="true"
          className={className}
          onClick={this.handleClick}
        >
          <TheIcon />
        </IconButton>
      );
    } else if (variant === 'listItem') {
      rootOn = (
        <ListItem
          className={liClassName}
          button={true}
          onClick={this.handleDisableClick}
        >
          <ListItemIcon className={className}>
            <TheIcon />
          </ListItemIcon>
          <ListItemText primary="Remote Sending On" />
        </ListItem>
      );
      rootOff = (
        <ListItem
          className={liClassName}
          button={true}
          aria-label="show list"
          aria-owns={anchorEl ? 'long-menu' : null}
          aria-haspopup="true"
          onClick={this.handleClick}
        >
          <ListItemIcon className={className}>
            <TheIcon />
          </ListItemIcon>
          <ListItemText primary="Remote Sending Off" />
        </ListItem>
      );
    }

    if (isSending) {
      return rootOn;
    }
    return (
      <div>
        {rootOff}
        <Menu
          id="long-menu"
          anchorEl={anchorEl}
          open={Boolean(anchorEl)}
          onClose={this.handleClose}
          PaperProps={{
            style: {
              maxHeight: ITEM_HEIGHT * 4.5,
              width: 200
            }
          }}
        >
          {options.map(option => (
            <MenuItem
              key={option.id}
              onClick={this.handleMenuClick.bind(this, option)}
            >
              {option.name.substr(7)}
            </MenuItem>
          ))}
        </Menu>
      </div>
    );
  }
}

export default withStyles(styles)(RemoteSenderMenu);
