import React, { Component } from 'react';
import { Redirect } from 'react-router-dom';

import RemotesActions, { RemoteStates } from 'subfire/actions/RemotesActions';
import SubsonicActions from 'subsonic/actions/SubsonicActions';
import SubsonicLoadingCard from './SubsonicLoadingCard';
import ColorActions from 'subfire/actions/ColorActions';
import { arrayShuffle } from '../../utils/utils.js';

class SubsonicLoader extends Component {
  static findArtist(id, subsonic, SubsonicSource) {
    let i, j;
    const ais = subsonic.allArtistIndexes;
    for (i = 0; i < ais.length; ++i) {
      for (j = 0; j < ais[i].artist.length; ++j) {
        if (ais[i].artist[j].id === id) {
          return ais[i].artist[j];
        }
      }
    }
    return {
      name: '' + id,
      id: '' + id
    };
  }

  static findAlbum(id, subsonic, SubsonicSource) {
    let object = subsonic.albums.find(x => {
      return x.id === id;
    });
    if (!object && subsonic.artistAlbums && subsonic.artistAlbums.album) {
      object = subsonic.artistAlbums.album.find(x => {
        return x.id === id;
      });
    }
    // not loaded yet, lets just fake it.
    if (!object)
      object = {
        name: '' + id,
        title: '' + id,
        id: '' + id,
        coverArt: 'al-' + id
      };
    return object;
  }

  static findDirectory(id, subsonic, SubsonicSource) {
    let object;
    return object;
  }

  static findObject(type, id, mode, subsonic, SubsonicSource) {
    let object;

    switch (type) {
      case 'playlist':
      case 'station':
        object = subsonic.allPlaylists.find(x => {
          return x.id === id;
        });
        break;
      case 'albumID3':
      case 'albumAction':
        object = SubsonicLoader.findAlbum(id, subsonic, SubsonicSource);
        break;
      case 'artist':
      case 'artistAction':
        object = SubsonicLoader.findArtist(id, subsonic, SubsonicSource);
        break;
      case 'album':
      case 'directoryAction':
      case 'directory':
        object = SubsonicLoader.findDirectory(id, subsonic, SubsonicSource);
        break;
      case 'folderShuffle':
        id = id || -1;
        object = subsonic.musicFolders.find(m => {
          return m.id + '' === id + '';
        }) || {
          name: 'All Music Folders',
          id: -1
        };
        console.log(object, id);
        object.coverArt = -1;
        break;
      case 'playQueue':
        object = {
          name: 'Play Queue',
          id: -1
        };
        break;
      case 'podcasts':
        object = {
          name: 'Podcasts',
          id: -1
        };
        break;
      case 'song':
        object = {
          name: 'Song or Video',
          id: id
        };
        break;
      default:
        console.error('unsupported type', type, id, mode);
        object = {
          name: 'Unsupported Type',
          id: -1
        };
    }
    return object;
  }

  // need to verify before redirect because sometimes isBusy is being unset by other things like folders, artists, indexes, or ancient history.
  // MUST be something that will resolve. checking empty is not good enough because the result of the query may be empty as well.
  // means store will need to init more things as null instead.
  static checkIfReallyLoaded(type, id, mode, subsonic) {
    if (type === 'podcasts') {
      return subsonic.podcasts !== null;
    }
    // will expand as needed
    return false;
  }

  static getDerivedStateFromProps(props, prevState) {
    const type = props.route.match.params.type;
    const id = props.route.match.params.id;
    const mode = props.route.match.params.mode;

    const position = props.route.match.params.position;
    const bookmarkId = props.route.match.params.bookmarkId;

    const subsonic = props.subsonic;
    const SubsonicSource = props.SubsonicSource;
    const object = SubsonicLoader.findObject(
      type,
      id,
      mode,
      subsonic,
      SubsonicSource
    );
    //console.log('found object', object);

    const isBusy = props.subsonic.isBusy;
    let loadingState =
      prevState && prevState.loadingState ? prevState.loadingState : 'init';
    if (
      !isBusy &&
      ((prevState && prevState.isBusy) ||
        SubsonicLoader.checkIfReallyLoaded(type, id, mode, subsonic))
    ) {
      console.warn('isBusy', isBusy, 'loading state to ready');
      loadingState = 'ready';
    }

    const state = {
      isBusy,
      loadingState,
      type,
      id,
      mode,
      position,
      bookmarkId,
      object
    };
    // console.warn('loader', state, props, prevState);
    return state;
  }

  constructor(props) {
    super(props);
    this.state = {};
    // this.state = SubsonicLoader.getDerivedStateFromProps(props);
    // console.log('constructor  state', this.state);
    // this.state.loadingState = 'init';
  }

  componentDidMount() {
    console.log('mount', this.props, this.state);
    const { route, remotes, subsonic } = this.props;
    const { type, id, mode, position } = this.state;
    if (remotes.remoteState === RemoteStates.sender) {
      this.startLoadRemote(route.location, type, id, mode, position, subsonic);
    } else {
      this.startLoad(type, id, mode, position, subsonic);
    }
    delete window.bookmarkSeekTo;
  }

  componentWillUnmount() {
    // ColorActions.resetColors.defer();
  }

  startLoadRemote(location, type, id, mode, position, subsonic) {
    switch (type) {
      case 'playlist':
      case 'station':
      case 'albumAction':
      case 'playQueue':
      case 'directoryAction':
      case 'folderShuffle':
      case 'artistAction':
      case 'song':
        RemotesActions.sendRoute(location.pathname);
        this.setState({ loadingState: 'ready' });
        break;
      case 'albumID3':
        SubsonicActions.loadAlbum.defer([id, true]);
        break;
      case 'album':
        SubsonicActions.loadAlbum.defer([id, false]);
        break;
      case 'artist':
        // remote - load it locally
        const artist = SubsonicLoader.findArtist(id, this.props.subsonic);
        SubsonicActions.setArtist.defer(artist);
        break;
      case 'directory':
        // remote - load it locally
        SubsonicActions.setMusicDirectory.defer(id);
        break;
      case 'podcasts':
        // not sure yet
        SubsonicActions.fetchPodcasts.defer();
        break;
      default:
        console.error('unsupported remote type', type);
    }
  }

  startLoad(type, id, mode, position, subsonic) {
    console.log('loading', type, id, mode, position);
    const bookmarkSource = 'SubFire';
    let loadingState = this.state.loadingState;
    SubsonicActions.setBookmarkContext.defer({
      type,
      id,
      mode,
      position,
      bookmarkSource
    });
    switch (type) {
      case 'playlist':
        SubsonicActions.selectPlaylist.defer(id, mode);
        break;
      case 'station':
        SubsonicActions.selectStation(id);
        break;
      case 'albumID3':
        SubsonicActions.loadAlbum.defer([id, true]);
        break;
      case 'album':
        SubsonicActions.loadAlbum.defer([id, false]);
        break;
      case 'playQueue':
        SubsonicActions.fetchPlayQueue();
        break;
      case 'directoryAction':
        SubsonicActions.playMusicDirectorySongs(id, mode);
        break;
      case 'directory':
        SubsonicActions.setMusicDirectory.defer(id);
        break;
      case 'song':
        SubsonicActions.playSong(id);
        break;
      case 'podcasts':
        SubsonicActions.fetchPodcasts.defer();
        break;
      case 'folderShuffle':
        id = id === '-1' ? null : id;
        SubsonicActions.shuffleMusicFolder.defer(id);
        break;
      case 'albumAction':
        const albumItem = subsonic.albumsById[id];
        if (albumItem) {
          const songs = albumItem.song;
          if (mode === 'shuffle') {
            arrayShuffle(songs);
          }
          SubsonicActions.startPlaying(songs, 0);
          loadingState = 'ready';
        } else {
          if (loadingState === 'init') {
            SubsonicActions.loadAlbum.defer([id, true]);
            loadingState = 'loading';
          }
        }
        break;
      case 'artistAction':
        const aartist = SubsonicLoader.findArtist(id, this.props.subsonic);
        console.log('artistAction', aartist);
        SubsonicActions.setAndPlayArtistSongs(aartist, mode);
        break;
      case 'artist':
        const artist = SubsonicLoader.findArtist(id, this.props.subsonic);
        console.log('startLoad', artist);
        SubsonicActions.setArtist.defer(artist);
        break;
      default:
        console.error('unsupported type', type);
    }
    this.setState({ loadingState: loadingState });
  }

  // componentWillReceiveProps(nextProps) {
  //   var state = SubsonicLoader.getDerivedStateFromProps(nextProps, this.state);
  //   this.setState(state);
  //   console.log('new state', state);
  // }

  shouldComponentUpdate(nextProps, nextState) {
    return true;
  }

  // todo - route replace to /remoteControl or other location if specified in props

  goBack = history => {
    if (!this.goneBack) {
      if (this.props.remoteRoute) {
        history.replace(this.props.remoteRoute);
      } else {
        history.goBack();
      }
    }
    this.goneBack = true;
  };

  render() {
    // console.log('render', this.props, this.state);
    let { position = 0, bookmarkId = null } = this.state;
    const { route, subsonic } = this.props;
    const { history } = route;
    const { id, type, mode } = route.match.params;
    let { goBack } = this.props;
    const LoadingComponent = this.props.LoadingComponent || SubsonicLoadingCard;
    let successRoute = '/player'; // this includes bookmarks as well as /loading

    if (!route.match.params['0'] && !route.match.params.bookmarkId) {
      successRoute = '/songs';
    }

    if (
      this.state.loadingState === 'loading' ||
      this.state.loadingState === 'init'
    )
      return (
        <LoadingComponent
          SubsonicSource={this.props.SubsonicSource}
          subsonic={this.props.subsonic}
          object={this.state.object}
          type={this.state.type}
          id={this.state.id}
          top={this.props.top}
        />
      );
    if (type === 'playlist' && mode === 'list') {
      return <Redirect to={`/playlist/${id}`} />;
    }
    if (type === 'artist') {
      return <Redirect to={`/artist/${id}`} />;
    }
    if (type === 'album') {
      return <Redirect to={`/directory/${id}`} />;
    }
    if (type === 'albumID3') {
      return <Redirect to={`/album/${id}`} />;
    }
    if (type === 'directory') {
      return <Redirect to={`/directory/${id}`} />;
    }
    if (type === 'podcasts' && !window.bookmarkState) {
      return <Redirect to={`/podcasts`} />;
    } else {
      if (this.props.remotes.remoteState === RemoteStates.sender) {
        this.goBack(history);
        return null;
      }

      if (type === 'playQueue' && subsonic.playQueue) {
        position = subsonic.playQueue.position;
        bookmarkId = subsonic.playQueue.current;
      }

      if (type === 'albumAction') {
        const albumItem = subsonic.albumsById[id];
        if (albumItem) {
          const songs = albumItem.song;
          if (mode === 'shuffle') {
            arrayShuffle(songs);
          }
          SubsonicActions.startPlaying.defer(songs, 0);
        }
      }

      if (goBack) {
        this.goBack(history);
        return null;
      }

      if (successRoute === '/player' && type !== 'albumAction') {
        SubsonicActions.startPlayingCurrentQueue.defer();
      }

      if (position || bookmarkId) {
        const bookmark = {
          position,
          bookmarkId
        };
        SubsonicActions.playerSkipToTrack.defer(bookmark);
      }

      return <Redirect to={successRoute} />;
    }
  }
}

export default SubsonicLoader;
