import React from 'react';

import { withStyles } from '@material-ui/core/styles';
import Grid from '@material-ui/core/Grid';

import CircularProgress from '@material-ui/core/CircularProgress';
import HollowCard from 'subsonic/components/styled/HollowCard';
import GradientGeneratingImage from 'subfire/components/GradientGeneratingImage';
import FuzzyImageBackground from 'subfire/components/FuzzyImageBackground';

import SubsonicActions from 'subsonic/actions/SubsonicActions';

const styles = theme => ({
  card: {
    textAlign: 'center'
  }
});

const SubsonicLoadingCard = props => {
  // console.log(props);
  const { SubsonicSource, object, classes, top = 0 } = props;

  const { innerWidth, innerHeight } = window;
  let cellSize =
    innerWidth <= 600 ? innerWidth - 20 : Math.trunc(innerWidth / 2);
  cellSize = Math.min(cellSize, innerHeight - 100 - top);
  // load the one already loaded in the previous grid
  const artSize = innerWidth > 360 ? 170 : 164;
  // console.warn(object);
  const coverArt =
    object && (object.coverArt || object.id)
      ? SubsonicSource.getCoverArtURL(object.coverArt || object.id, artSize)
      : '';

  const loadingString = object
    ? object.name || object.title
    : "something. no really. we're loading something.";

  SubsonicActions.setLoadingTitle.defer(loadingString);

  return (
    <div style={{ paddingTop: 25 }}>
      <Grid container spacing={0}>
        <Grid item sm={3} xs={false} />
        <Grid item sm={6} xs={12}>
          <HollowCard>
            <div className={classes.card}>Loading...</div>
            <div className={classes.card}>
              <GradientGeneratingImage
                src={coverArt}
                imageId="the-loading-image"
                size={cellSize}
              />
              <CircularProgress className="center-progress" size={50} />
            </div>
            <div className={classes.card}>{loadingString}</div>
            <FuzzyImageBackground
              selector=".page-bg"
              image={coverArt}
              showBackground={true}
            />
          </HollowCard>
        </Grid>
      </Grid>
    </div>
  );
};

export default withStyles(styles)(SubsonicLoadingCard);
