import React, { Component } from 'react';

import MobilePlayer from './MobilePlayer';
import MiniPlayer from './MiniPlayer';
import { emptytrack } from 'subsonic/stores/SubsonicStore';
import SubsonicActions from 'subsonic/actions/SubsonicActions';

import UAParser from 'ua-parser-js';

class ThePlayer extends Component {
  static getDerivedStateFromProps(props, prevState) {
    const { subsonic = {}, SubsonicSource, csProps } = props;
    const { isConnected, currentPlaying } = subsonic;
    if (!isConnected) {
      return prevState;
    }
    let newState;

    if (!currentPlaying.queue.length) {
      newState = {
        showBackground: false,
        currentFile: emptytrack,
        currentPlaying: currentPlaying
      };
    } else {
      const currentFile = currentPlaying.queue[currentPlaying.idx];
      const p = csProps;
      // console.log(p.reason);
      let {
        showBackground,
        isWaitingForNext,
        coverSize,
        coverArt,
        mediaSrc,
        mediaIcon,
        reason
      } = prevState;
      if (currentFile !== prevState.currentFile) {
        console.warn('next track...');
        isWaitingForNext = false;
        const minWidth = Math.trunc(window.innerWidth * (10 / 12));
        /*
        - 64 // appbar
        - 56 // buttons
        - 32 // slider
        - 16 // time strings
        - 80 // info cards
        */
        const maxHeight = Math.trunc(window.innerHeight - 248);

        coverSize = Math.min(minWidth, maxHeight);
        coverSize = coverSize <= 0 ? window.lastCoverSize : coverSize;
        window.lastCoverSize = coverSize;
        coverArt = SubsonicSource.getCoverArtURL(currentFile.id, coverSize);
        mediaSrc =
          currentFile.localPath ||
          (currentFile.src && currentFile.src.startsWith('http')
            ? currentFile.src
            : SubsonicSource.getStreamingURL(currentFile.id));
        mediaIcon =
          SubsonicSource.getCoverArtURL(currentFile.id, 192) + '&co=false';

        // now load castplayer
        currentFile.coverArtURL = coverArt;
        currentFile.src = mediaSrc;
        console.warn('first load', currentFile);
        window.cp.playerHandler.load(currentFile, 0);
        // console.warn(currentFile);
      } else {
        // console.warn('checking', prevState.isWaitingForNext, reason, p.reason);
        if (
          !prevState.isWaitingForNext &&
          reason !== p.reason &&
          p.reason === 'playbackEnded'
        ) {
          console.warn('endPlayback next...');
          isWaitingForNext = true;
          SubsonicActions.playerNext.defer();
        }
      }

      // hack - no easy way to pass this around
      if (window.bookmarkSeekTo) {
        window.castSubsonic.seekMedia(window.bookmarkSeekTo);
        window.bookmarkSeekTo = null;
      }

      if (prevState.showBackground !== p.isPlaying) {
        showBackground = p.isPlaying;
      }

      // if (p.reason !== 'normal') console.log('p', p);
      const { currentMediaTimeString, durationString } = p;

      newState = {
        currentPlaying,
        currentFile,
        coverArt,
        mediaSrc,
        mediaIcon,
        coverSize,
        showBackground,
        isWaitingForNext,
        currentMediaTimeString,
        durationString,
        reason: csProps.reason
      };
    }
    return Object.assign({}, prevState, newState);
  }

  constructor(props) {
    super(props);

    // never gonna change so no need to be state
    this.ua = new UAParser().getResult();
    // console.log(this.ua);

    var os = this.ua.os;
    this.isMobile = os.name === 'Android' || os.name === 'iOS';
    // const portrait = window.innerHeight >= window.innerWidth;

    this.state = {
      Player: window.lastPlayer || MobilePlayer
    };
  }

  savePlayQueue = () => {
    SubsonicActions.savePlayQueue(
      Math.trunc((this.state.currentMediaTime || 0) * 1000)
    );
  };

  toggleBookmark = currentMediaTime => {
    // console.log('toggleBookmark', currentMediaTime, this.props);
    const { subsonic } = this.props;
    const { currentPlaying } = subsonic;
    const id = currentPlaying.queue[currentPlaying.idx].id;

    if (subsonic.bookmarksById[id]) {
      SubsonicActions.deleteBookmark(id);
    } else {
      SubsonicActions.createBookmark(id, Math.trunc(currentMediaTime * 1000));
    }
  };

  seekValueChange = rangeSliderState => {
    this.setState({ rangeSliderState });
  };

  togglePlayer = () => {
    const isMobilePlayer = this.state.Player === MobilePlayer;
    this.setState({
      Player: isMobilePlayer ? MiniPlayer : MobilePlayer
    });
  };

  componentDidMount() {
    try {
      console.warn('didMount - play');
      window.cp.playerHandler.play();
    } catch (e) {
      // console.error(e);
    }
  }

  componentWillUnmount() {
    window.cp.playerHandler.pause();
    window.lastPlayer = this.state.Player;
  }

  render() {
    const { Player } = this.state;
    // const togglePlayer = this.isMobile ? null : this.togglePlayer;
    const actions = {
      seekValueChange: this.seekValueChange,
      toggleBookmark: this.toggleBookmark,
      savePlayQueue: this.savePlayQueue,
      togglePlayer: this.togglePlayer
    };
    return <Player {...this.props} playerState={this.state} {...actions} />;
  }
}

export default ThePlayer;
