import React from 'react';
import { Link } from 'react-router-dom';

import Button from '@material-ui/core/Button';
import GridList from '@material-ui/core/GridList';

import { VideoLibrary, GridOn, ViewList } from '@material-ui/icons';

import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';
import IconButton from '@material-ui/core/IconButton';

import MusicFolderChooser from './MusicFolderChooser';
import AlbumQueryChooser from './AlbumQueryChooser';

import SubsonicActions from 'subsonic/actions/SubsonicActions';

import InfiniteScroll from 'subsonic/components/styled/InfiniteScroll';
import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

import CoverGridTile from 'subsonic/components/styled/CoverGridTile';
import { withStyles } from '@material-ui/core/styles';

const styles = theme => ({});

class AlbumQueryBrowser extends React.Component {
  constructor(props) {
    super(props);
    this.myRef = React.createRef();
  }

  updateDimensions = () => {
    this.setState({ width: window.innerWidth, height: window.innerHeight });
  };

  componentDidMount = () => {
    window.addEventListener('resize', this.updateDimensions);
    this.updateDimensions();

    const state = this.props.subsonic;
    if (state.albums.length === 0) {
      setTimeout(() => {
        SubsonicActions.fetchAlbumPage.defer(state.albumPage, state.albumID3);
      }, 2000);
    }
  };

  componentWillUnmount = () => {
    window.removeEventListener('resize', this.updateDimensions);
  };

  loadMore = page => {
    const state = this.props.subsonic;
    // console.log(page, state.albumPage, state.albumPage + 2);
    // hackery
    const alt = window.alt;
    if (alt && alt.dispatcher._isDispatching) {
      if (page === state.albumPage + 2) {
        SubsonicActions.fetchAlbumPage.defer(
          state.albumPage + 1,
          state.albumID3
        );
      }
      return;
    }
    SubsonicActions.fetchAlbumPage(state.albumPage + 1, state.albumID3);
  };

  loadAlbumFolder = album => {
    const { albumID3 } = this.props.subsonic;
    if (albumID3) {
      this.props.history.push('/loading/albumID3/' + album.id);
    } else {
      this.props.history.push('/loading/directory/' + album.id);
    }
  };

  render() {
    // console.log(this.props);
    const {
      subsonic,
      SubsonicSource,
      linkComponent,
      linkComponentID3,
      native,
      classes
    } = this.props;
    const isGrid = subsonic.albumsAsGrid;
    const { albumID3 } = subsonic;

    const theLinkComponent = albumID3 ? linkComponentID3 : linkComponent;

    const albums = subsonic.albums || [];
    // console.log('aqb', state.albums.length, state.albumPage, state.hasMoreAlbums, 'hasMore?', !!(state.hasMoreAlbums && albums.length));

    var hasMore =
      subsonic.hasMoreAlbums && albums.length !== 0 && albums.length < 400;

    let innerWidth = window.innerWidth;
    if (this.myRef && this.myRef.current) {
      innerWidth = this.myRef.current.offsetWidth;
    }

    if (isNaN(innerWidth)) innerWidth = 360;

    const spacing = innerWidth > 360 ? 10 : 6;
    const cellSize = innerWidth > 360 ? 170 : 164;

    const cols = innerWidth / (cellSize + spacing);
    const width = cols * (cellSize + spacing);

    const height = this.props.height || window.innerHeight;

    const GridListIcon = isGrid ? GridOn : ViewList;
    const gridListToggle = (
      <div className={classes.buttonCell}>
        <Button
          variant="outlined"
          className={classes.button}
          onClick={SubsonicActions.toggleAlbumsAsGrid}
        >
          {'As ' + (isGrid ? 'Grid' : 'List')}
          <GridListIcon className={classes.rightIcon} />
        </Button>
      </div>
    );
    const musicFolderChooser = (
      <MusicFolderChooser native={native} {...subsonic} />
    );

    const content = isGrid ? (
      <div ref={this.myRef}>
        <GridList
          cellHeight={cellSize}
          width={width}
          cols={cols}
          spacing={spacing}
        >
          {albums.map((n, idx) => (
            <CoverGridTile
              key={
                'alb' +
                n.id +
                (subsonic.albumQuery &&
                (subsonic.albumQuery === 'random' ||
                  subsonic.albumQuery.startsWith('pl-'))
                  ? idx
                  : '')
              }
              onClick={this.loadAlbumFolder.bind(this, n)}
              subname={n.artist}
              name={n.title || n.name}
              normal
              size={160}
              coverArt={n.coverArt}
              linkComponent={theLinkComponent}
              id={n.id}
              SubsonicSource={SubsonicSource}
            />
          ))}
        </GridList>
      </div>
    ) : (
      <List>
        {albums.map(item => (
          <ListItem
            key={'alb' + item.id}
            button
            component={Link}
            to={
              albumID3
                ? `/loading/albumID3/${item.id}`
                : `/loading/directory/${item.id}`
            }
          >
            <ListItemIcon>
              <img
                alt=""
                width={48}
                height={48}
                src={SubsonicSource.getCoverArtURL(item.coverArt, 48)}
              />
            </ListItemIcon>
            <ListItemText
              primary={item.title || item.album || item.name}
              secondary={item.artist}
            />
            <ListItemSecondaryAction>
              <IconButton
                aria-label="Play"
                component={Link}
                to={
                  albumID3
                    ? `/loading/albumAction/${item.id}/play`
                    : `/loading/directoryAction/${item.id}/all`
                }
              >
                <VideoLibrary />
              </IconButton>
            </ListItemSecondaryAction>
          </ListItem>
        ))}
      </List>
    );

    return (
      <div
        style={{ height: height, overflow: 'auto' }}
        id="album-query-wrapper"
      >
        <AlbumQueryChooser
          native={native}
          {...subsonic}
          SubsonicSource={SubsonicSource}
          musicFolderChooser={musicFolderChooser}
          gridListToggle={gridListToggle}
        />
        <InfiniteScroll
          useWindow={false}
          initialLoad={false}
          pageStart={subsonic.albumPage}
          loadMore={this.loadMore}
          hasMore={hasMore}
        >
          {content}
        </InfiniteScroll>
        <ScrollToTopFab {...this.props.fabProps} />
      </div>
    );
  }
}

export default withStyles(styles)(AlbumQueryBrowser);
