import React, { Component } from 'react';

import SubsonicActions from 'subsonic/actions/SubsonicActions';
import PlaylistsBrowser from 'subsonic/components/browsers/PlaylistsBrowser';

import Grid from '@material-ui/core/Grid';
import Select from '@material-ui/core/Select';
import MenuItem from '@material-ui/core/MenuItem';
import FormControl from '@material-ui/core/FormControl';
import FormControlLabel from '@material-ui/core/FormControlLabel';
import Input from '@material-ui/core/Input';
import InputLabel from '@material-ui/core/InputLabel';
import Switch from '@material-ui/core/Switch';

class AlbumQueryChooser extends Component {
  constructor(props) {
    super(props);
    let plKey = props.albumQuery.startsWith('pl-') ? props.albumQuery : 'pl-';
    this.state = {
      queries: [
        {
          id: 'random',
          name: 'Random'
        },
        {
          id: 'newest',
          name: 'Recently Added'
        },
        {
          id: 'recent',
          name: 'Recently Played'
        },
        {
          id: 'frequent',
          name: 'Most Played'
        },
        {
          id: 'alphabeticalByName',
          name: 'By Album Name'
        },
        {
          id: 'alphabeticalByArtist',
          name: 'By Artist Name'
        },
        {
          id: 'starred',
          name: 'Starred'
        },
        {
          id: 'byYear',
          name: 'Oldest First'
        },
        {
          id: 'byYear reversed',
          name: 'Newest First'
        },
        {
          id: plKey,
          name: 'From Playlist'
        }
      ]
    };
  }

  updatePlQuery(id) {
    const queries = this.state.queries;
    queries[9].id = 'pl-' + id;
  }

  handleChange = evt => {
    SubsonicActions.setAlbumQuery(evt.target.value, this.props.albumID3);
  };

  handleID3Change = (evt, isInputChecked) => {
    SubsonicActions.setAlbumQuery(this.props.albumQuery, isInputChecked);
  };

  handlePlaylistChange = plID => {
    SubsonicActions.setAlbumQuery('pl-' + plID, this.props.albumID3);
    this.updatePlQuery(plID);
  };

  render() {
    const props = this.props;
    const native = props.native;
    const gridListToggle = props.gridListToggle;
    const musicFolderChooser = props.musicFolderChooser;
    const queryItems = this.state.queries.map(m => {
      if (native)
        return (
          <option key={m.id} value={m.id}>
            {m.name}
          </option>
        );
      return (
        <MenuItem key={m.id} value={m.id}>
          {m.name}
        </MenuItem>
      );
    });

    let value = props.albumQuery;
    const id3 = this.props.albumID3 || false;
    const label = 'ID3?';

    const plMode = value.startsWith('pl-');
    let plSelect = null;
    if (plMode) {
      const currentPlaylistId = value.substring(3);
      plSelect = (
        <Grid item={true} xs={6} md={3}>
          <FormControl fullWidth={true}>
            <InputLabel htmlFor="pl-select">Playlist</InputLabel>
            <PlaylistsBrowser
              native={props.native}
              subsonic={props}
              SubsonicSource={props.SubsonicSource}
              renderMode={PlaylistsBrowser.renderTypes.SELECT}
              selectedId={currentPlaylistId}
              onSelect={this.handlePlaylistChange}
              showThumbs={false}
            />
          </FormControl>
        </Grid>
      );
    }

    const id3Selector = (
      <Grid item={true} xs={2} md={1}>
        <FormControlLabel
          control={
            <Switch
              label={label}
              value="id3"
              checked={id3}
              onChange={this.handleID3Change}
            />
          }
          label="ID3"
        />
      </Grid>
    );

    const albumQuerySelector = (
      <Grid item={true} xs={plMode ? 5 : 12} md={plMode ? 3 : 6}>
        <FormControl fullWidth={true}>
          <InputLabel htmlFor="album-query-select">Sorted By...</InputLabel>
          <Select
            native={props.native}
            value={value}
            onChange={this.handleChange}
            input={<Input name="album-query" id="album-query-select" />}
          >
            {queryItems}
          </Select>
        </FormControl>
      </Grid>
    );

    const gridListToggleSection = (
      <Grid item xs={5} md={2}>
        {gridListToggle}
      </Grid>
    );

    const musicFolderChooserSection = (
      <Grid item xs={5} md={3}>
        {musicFolderChooser}
      </Grid>
    );

    const content =
      window.innerWidth >= 960 ? (
        <Grid container>
          {musicFolderChooserSection}
          {albumQuerySelector}
          {plSelect}
          {id3Selector}
          {gridListToggleSection}
        </Grid>
      ) : (
        <Grid container>
          {musicFolderChooserSection}
          {id3Selector}
          {gridListToggleSection}
          {albumQuerySelector}
          {plSelect}
        </Grid>
      );
    return content;
  }
}

export default AlbumQueryChooser;
