import React, { Component } from 'react';
import { Redirect } from 'react-router-dom';

import Typography from '@material-ui/core/Typography';
import { withStyles } from '@material-ui/core/styles';

import ArtistActionButtons from './ArtistActionButtons';

import SubsonicActions from 'subsonic/actions/SubsonicActions';

import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

import SongList from 'subsonic/components/browsers/SongList';

const styles = theme => ({
  li: {
    listStyle: 'none'
  }
});

class AlbumSongBrowser extends Component {
  // constructor(props) {
  //   super(props);
  // }

  playItem = song => {
    console.warn('playItem', song);
    if (this.props.remotes.remoteState === 'sender') {
      this.props.history.push('/loading/song/' + song.id);
    } else {
      SubsonicActions.startPlaying(song, 0);
      this.props.history.push('/player');
    }
  };

  render() {
    const {
      subsonic,
      SubsonicSource,
      match,
      showBackButton,
      history
    } = this.props;
    const id = match.params.id;
    if (!subsonic || !subsonic.isConnected) {
      return <Typography>awaiting login...</Typography>;
    }

    if (!subsonic.albumsById[id]) {
      if (id === undefined || id === 'undefined') {
        return <Typography>problem 1</Typography>;
      }
      return <Redirect to={'/loading/albumID3/' + id} />;
    }

    const contents = subsonic.albumsById[id];
    if (!contents) {
      return <Typography>oops?</Typography>;
    }

    // render first one's cover art, then list the rest
    // const songSection = contents.song.map(item => (
    //   <ListItem
    //     key={`item-${item.id}`}
    //     classes={{ container: classes.li }}
    //     button
    //     onClick={this.playItem.bind(this, item)}
    //   >
    //     <ListItemIcon>
    //       <img src={SubsonicSource.getCoverArtURL(item.coverArt, 24)} />
    //     </ListItemIcon>
    //     <ListItemText primary={`${item.title}`} />
    //   </ListItem>
    // ));
    // console.log(id, contents.song[0].artistId);

    const songSection = (
      <SongList
        contents={contents.song}
        SubsonicSource={SubsonicSource}
        showMeta={false}
        handleSongClick={this.playItem}
      />
    );
    return (
      <div>
        <ArtistActionButtons
          id={id}
          artistId={contents.song[0].artistId}
          loadbase="artistAction"
          SubsonicActions={SubsonicActions}
          isGrid={subsonic.albumsAsGrid}
          showBackButton={showBackButton}
          history={history}
        />
        {songSection}
        <ScrollToTopFab {...this.props.fabProps} />
      </div>
    );
  }
}

export default withStyles(styles)(AlbumSongBrowser);
