import React from 'react';
import { Link } from 'react-router-dom';

import {
  Radio,
  Album,
  VideoLibrary,
  Shuffle,
  ThumbUp,
  GridOn,
  ViewList,
  Reply
} from '@material-ui/icons';
import Button from '@material-ui/core/Button';

import { withStyles } from '@material-ui/core/styles';
const styles = theme => ({
  buttonRow: {
    display: 'flex',
    width: '100%',
    justifyContent: 'space-around'
  },
  buttonCell: {
    textAlign: 'center'
  },
  button: {
    margin: theme.spacing(1)
  },
  rightIcon: {
    marginLeft: theme.spacing(1)
  }
});

const ArtistActionButtons = props => {
  const {
    classes,
    loadbase,
    id,
    SubsonicActions,
    isGrid,
    showBackButton,
    history
  } = props;
  let { artistId } = props;
  const isAlbum = !!artistId;

  // console.log(id, artistId, isAlbum);

  if (!artistId) artistId = id;
  let top = '';
  let radio = '';
  let backButton = '';
  if (showBackButton && history) {
    backButton = (
      <div className={classes.buttonCell}>
        <Button onClick={history.goBack}>
          <Reply />
        </Button>
      </div>
    );
  }

  if (loadbase !== 'directoryAction') {
    // music directory isn't fully in cache, so no name, so this doesn't work :()
    top = (
      <div className={classes.buttonCell}>
        <Button component={Link} to={`/loading/${loadbase}/${artistId}/top`}>
          <ThumbUp />
        </Button>
      </div>
    );
  }

  radio = (
    <div className={classes.buttonCell}>
      <Button component={Link} to={`/loading/${loadbase}/${artistId}/radio`}>
        <Radio />
      </Button>
    </div>
  );

  const GridListIcon = isGrid ? GridOn : ViewList;
  const gridListToggle = (
    <div className={classes.buttonCell}>
      <Button
        variant="outlined"
        className={classes.button}
        onClick={SubsonicActions.toggleAlbumsAsGrid}
      >
        {'As ' + (isGrid ? 'Grid' : 'List')}
        <GridListIcon className={classes.rightIcon} />
      </Button>
    </div>
  );

  return (
    <div className={classes.buttonRow}>
      {backButton}
      {top}
      {!isAlbum && (
        <React.Fragment>
          <div className={classes.buttonCell}>
            <Button component={Link} to={`/loading/${loadbase}/${id}/all`}>
              <VideoLibrary />
            </Button>
          </div>
          <div className={classes.buttonCell}>
            <Button component={Link} to={`/loading/${loadbase}/${id}/shuffle`}>
              <Shuffle />
            </Button>
          </div>
          <div className={classes.buttonCell}>
            <Button
              component={Link}
              to={`/loading/${loadbase}/${id}/shuffleAlbums`}
            >
              <Album />
            </Button>
          </div>
        </React.Fragment>
      )}
      {isAlbum && (
        <React.Fragment>
          <div className={classes.buttonCell}>
            <Button component={Link} to={`/loading/albumAction/${id}/play`}>
              <VideoLibrary />
            </Button>
          </div>
          <div className={classes.buttonCell}>
            <Button component={Link} to={`/loading/albumAction/${id}/shuffle`}>
              <Shuffle />
            </Button>
          </div>
        </React.Fragment>
      )}
      {radio}
      {gridListToggle}
    </div>
  );
};

export default withStyles(styles)(ArtistActionButtons);
