import React, { Component } from 'react';
import { Redirect, Link } from 'react-router-dom';

import SubsonicActions from 'subsonic/actions/SubsonicActions';

import GridList from '@material-ui/core/GridList';
import IconButton from '@material-ui/core/IconButton';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';
import VideoLibrary from '@material-ui/icons/VideoLibrary';

import ArtistActionButtons from 'subsonic/components/browsers/ArtistActionButtons';

import { withStyles } from '@material-ui/core/styles';

import CoverGridTile from 'subsonic/components/styled/CoverGridTile';
import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

const styles = theme => ({});

class ArtistAlbumsBrowser extends Component {
  constructor(props) {
    super(props);
    this.myRef = React.createRef();
  }

  loadAlbumFolder = album => {
    this.props.history.push('/loading/albumID3/' + album.id);
  };

  updateDimensions = () => {
    this.setState({ width: window.innerWidth, height: window.innerHeight });
  };

  componentDidMount = () => {
    window.addEventListener('resize', this.updateDimensions);
    this.updateDimensions();
  };

  componentWillUnmount = () => {
    window.removeEventListener('resize', this.updateDimensions);
  };

  render() {
    // console.log(this.props);
    const {
      subsonic,
      SubsonicSource,
      linkComponent,
      match,
      showBackButton,
      history
    } = this.props;
    const id = match.params.id;

    if (!subsonic || !subsonic.artistAlbums) {
      return <Redirect to={'/loading/artist/' + id} />;
    }

    const isGrid = subsonic.albumsAsGrid;
    const albums = subsonic.artistAlbums.album || [];
    let innerWidth = window.innerWidth;
    if (this.myRef && this.myRef.current) {
      innerWidth = this.myRef.current.offsetWidth;
    }

    if (isNaN(innerWidth)) innerWidth = 360;
    const spacing = innerWidth > 360 ? 10 : 6;
    const cellSize = innerWidth > 360 ? 170 : 164;

    const cols = innerWidth / (cellSize + spacing);
    const width = cols * (cellSize + spacing);

    const content = isGrid ? (
      <div ref={this.myRef}>
        <GridList
          cellHeight={cellSize}
          width={width}
          cols={cols}
          spacing={spacing}
        >
          {albums.map((n, idx) => (
            <CoverGridTile
              onClick={this.loadAlbumFolder.bind(this, n)}
              key={'alb' + n.id}
              subname={n.artist}
              name={n.title || n.name}
              normal
              size={160}
              coverArt={n.coverArt}
              linkComponent={linkComponent}
              id={n.id}
              SubsonicSource={SubsonicSource}
            />
          ))}
        </GridList>
      </div>
    ) : (
      <List>
        {albums.map(item => (
          <ListItem
            key={'alb' + item.id}
            button
            component={Link}
            to={`/loading/albumID3/${item.id}`}
          >
            <ListItemIcon>
              <img
                alt=""
                width={48}
                height={48}
                src={SubsonicSource.getCoverArtURL(item.coverArt, 48)}
              />
            </ListItemIcon>
            <ListItemText
              primary={item.title || item.album || item.name}
              secondary={item.artist}
            />
            <ListItemSecondaryAction>
              <IconButton
                aria-label="Play"
                component={Link}
                to={`/loading/albumAction/${item.id}/play`}
              >
                <VideoLibrary />
              </IconButton>
            </ListItemSecondaryAction>
          </ListItem>
        ))}
      </List>
    );

    return (
      <div>
        <ArtistActionButtons
          id={id}
          loadbase="artistAction"
          SubsonicActions={SubsonicActions}
          isGrid={subsonic.albumsAsGrid}
          showBackButton={showBackButton}
          history={history}
        />
        {content}
        <ScrollToTopFab {...this.props.fabProps} />
      </div>
    );
  }
}

export default withStyles(styles)(ArtistAlbumsBrowser);
