import React, { Component } from 'react';
import { Link } from 'react-router-dom';

import MusicFolderChooser from './MusicFolderChooser';
import { ShuffleButton } from 'subsonic/components/controls/MediaButtons';

import Avatar from '@material-ui/core/Avatar';
import Grid from '@material-ui/core/Grid';
import List from '@material-ui/core/List';
import ListSubheader from '@material-ui/core/ListSubheader';
import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';

import { withStyles } from '@material-ui/core/styles';
import PlayIcon from '@material-ui/icons/VideoLibrary';
import IconButton from '@material-ui/core/IconButton';

import classNames from 'classnames';

import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

const styles = theme => ({
  listRoot: {
    width: '100%',
    overflow: 'auto',
    display: 'inline-block'
  },
  subheader: {
    backgroundColor: theme.palette.background.paper
  },
  li: {
    listStyle: 'none'
  },
  listSection: {
    backgroundColor: 'inherit'
  },
  ul: {
    backgroundColor: 'inherit',
    padding: 0
  },
  avatar: {
    margin: 5
  },
  bigAvatar: {
    width: 40,
    height: 40
  }
});

class ArtistBrowser extends Component {
  constructor(props) {
    super(props);
    this.state = {
      currentIndex: null,
      showAll: true
    };
  }

  static getDerivedStateFromProps = (nextProps, lastState) => {
    const { subsonic } = nextProps;
    let { artistIndexes, artists, musicFolder } = subsonic;
    artistIndexes = artistIndexes || [];
    artists = artists || [];
    if (!Array.isArray(artistIndexes)) artistIndexes = [artistIndexes];
    if (!Array.isArray(artists)) artists = [artists];
    // console.log('artists', artists.length);

    if (artists.length < 20) {
      artistIndexes = [
        {
          artist: artists,
          name: 'All'
        }
      ];
    }

    const showAll = artists.length < 201;
    const currentIndex = showAll
      ? null
      : musicFolder === lastState.musicFolder && lastState.currentIndex
      ? lastState.currentIndex
      : artistIndexes[0].name;

    return {
      artistIndexes,
      artists,
      showAll,
      currentIndex,
      musicFolder
    };
  };

  componentDidMount() {}

  scrollToTop = () => {
    document.querySelector('.artistList').scrollTop = 0;
  };

  shuffleClick = () => {
    var id = this.props.subsonic.musicFolder || '-1';
    this.props.history.push('/loading/folderShuffle/' + id);
  };

  playArtist = artist => {
    this.props.history.push('/loading/artistAction/' + artist.id + '/all');
  };

  setCurrentIndex = idx => {
    this.setState({
      currentIndex: idx
    });
  };

  render() {
    const { subsonic, SubsonicSource, classes } = this.props;
    const { artistIndexes, showAll, currentIndex } = this.state;

    // TODO: fix that ul/li below
    return (
      <div>
        <Grid container={true}>
          <Grid item={true} xs={10}>
            <MusicFolderChooser {...subsonic} />
          </Grid>
          <Grid item={true} xs={2}>
            <ShuffleButton shuffleClick={this.shuffleClick} />
          </Grid>
        </Grid>
        <List
          className={classNames(classes.listRoot, 'artistList')}
          subheader={<li />}
        >
          {artistIndexes.map(index => (
            <li key={`section-${index.name}`} className={classes.listSection}>
              <ul
                className={classes.ul}
                onClick={
                  !showAll ? this.setCurrentIndex.bind(this, index.name) : null
                }
              >
                <ListSubheader className={classes.subheader}>
                  {`Artists: ${index.name}`}
                </ListSubheader>
                {(showAll || index.name === currentIndex
                  ? index.artist
                  : []
                ).map(item => (
                  <ListItem
                    classes={{ container: classes.li }}
                    key={`item-${index.name}-${item.id}`}
                    button
                    component={Link}
                    to={`/loading/artist/${item.id}`}
                  >
                    <Avatar
                      alt={`${item.name}`}
                      src={SubsonicSource.getCoverArtURL(item.coverArt, 40)}
                      className={classNames(classes.avatar, classes.bigAvatar)}
                    />
                    <ListItemText
                      primary={`${item.name}`}
                      className={classes.listItemText}
                    />
                    <ListItemSecondaryAction>
                      <IconButton
                        aria-label="Play"
                        onClick={this.playArtist.bind(this, item)}
                      >
                        <PlayIcon />
                      </IconButton>
                    </ListItemSecondaryAction>
                  </ListItem>
                ))}
              </ul>
            </li>
          ))}
        </List>
        <ScrollToTopFab
          scrollToTop={this.scrollToTop}
          {...this.props.fabProps}
        />
      </div>
    );
  }
}

export default withStyles(styles)(ArtistBrowser);
