import React, { Component } from 'react';

import { withStyles } from '@material-ui/core/styles';
import Typography from '@material-ui/core/Typography';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';

import Paper from '@material-ui/core/Paper';
import Button from '@material-ui/core/Button';
import IconButton from '@material-ui/core/IconButton';

import Hidden from '@material-ui/core/Hidden';

import DeleteForever from '@material-ui/icons/DeleteForever';

import SubsonicLoader from 'subsonic/components/SubsonicLoader';
import SubsonicActions from 'subsonic/actions/SubsonicActions';

const styles = theme => ({
  root: {
    width: '100%',
    marginTop: theme.spacing(3),
    overflowX: 'auto'
  },
  table: {
    minWidth: 360
  },
  unsetButton: {
    fontWeight: 'inherit',
    fontSize: 'inherit',
    textTransform: 'inherit',
    letterSpacing: 'inherit',
    lineHeight: 'inherit',
    padding: 0
  }
});

class BookmarksBrowser extends Component {
  static parseContext(comment, subsonic, SubsonicSource) {
    // console.warn(comment);
    comment = comment || '';
    try {
      comment = comment.replace(/&quot;/g, '"'); //ampache hack
      // console.warn(comment);
      const data = JSON.parse(comment);
      if (data.bookmarkSource !== 'SubFire') {
        // console.warn(data);
        return {
          contextObject: null,
          contextData: null,
          contextString: 'Not a SubFire Bookmark'
        };
      }
      const object = SubsonicLoader.findObject(
        data.type,
        data.id,
        data.mode,
        subsonic,
        SubsonicSource
      );
      // console.log(data, object);
      return {
        contextObject: object,
        contextData: data,
        contextString: object
          ? (data.type ? data.type.toUpperCase() + ' ' : '') +
            (object.name || object.title || object.album)
          : data.type
          ? data.type.toUpperCase() + ' ' + data.id
          : ''
      };
    } catch (e) {
      return {
        contextObject: null,
        contextData: null,
        contextString:
          comment.indexOf('DSub') >= 0 ? comment : 'Not a SubFire Bookmark'
      };
    }
  }

  static getDerivedStateFromProps(props, prevState) {
    // if (prevState && prevState.bookmarks === props.subsonic.bookmarks)
    //   return prevState;
    const bookmarks = props.subsonic.bookmarks;
    // bookmarks = array of
    //  comment = JSON of type, id, source, potentially mode (albums)
    //  position
    //  entry:
    //    id, album, artist, coverArt, title
    // create the table structure here
    const tableData = bookmarks.map((b, i, a) => {
      // console.log("tableData", b);
      const bookmarkDate = new Date(Date.parse(b.created));
      const context = BookmarksBrowser.parseContext(
        b.comment || '',
        props.subsonic,
        props.SubsonicSource
      );
      const e = Object.assign(
        {},
        b.entry,
        { position: b.position },
        {
          contextData: context.contextData,
          contextObject: context.contextObject,
          contextString: context.contextString,
          bookmarkDate
        }
      );
      return e;
    }, []);
    return {
      bookmarks,
      tableData
    };
  }

  constructor(props) {
    super(props);
    this.state = {};
  }

  handleRowClick(evt, bookmark) {
    // console.log('bookmark clicked', bookmark);
    const d = bookmark.contextData;
    let path;
    if (d) {
      // fix the old album since album now goes to directory browser
      if (d.type === 'album') {
        d.type = 'directoryAction';
        d.mode = 'all';
      }
      path =
        '/bookmark/' +
        bookmark.id +
        '/' +
        bookmark.position +
        '/loading/' +
        d.type +
        '/' +
        d.id;
      if (d.mode) path += '/' + d.mode;
    } else {
      // non-subfire bookmarks
      path =
        '/bookmark/' +
        bookmark.id +
        '/' +
        bookmark.position +
        '/loading/' +
        'song/' +
        bookmark.id;
    }
    // console.log('path', path, this.props.route);
    const { history } = this.props;
    // hack for HashRouter
    // window.bookmarkState = bookmark;
    history.push(path, bookmark);
  }

  handleDeleteClick(evt, bookmark) {
    const d = bookmark;
    if (d) {
      SubsonicActions.deleteBookmark(d.id);
    }
    evt.stopPropagation();
    return false;
  }

  render() {
    // console.log("render bookmarks", this.props, this.state);
    const { classes, SubsonicSource, remotes, ...props } = this.props;

    // yes, letting a little of subfire bleed in. not much i can do about it...
    let warning = null;
    // if (remotes.remoteState === 'sender') warning = (<Typography>Note: currently, bookmark loading does not work correctly over remote casting.</Typography>);
    let nobookmarks = null;
    if (this.state.tableData.length === 0) {
      nobookmarks = (
        <Typography variant="h4">No Bookmarks have been stored.</Typography>
      );
    }

    return (
      <Paper className={classes.root}>
        {warning}
        <Table className={classes.table}>
          <TableHead>
            <TableRow>
              <TableCell className={props.cellClasses} size="small" />
              <TableCell className={props.cellClasses} size="small">
                Title
              </TableCell>
              <TableCell className={props.cellClasses} size="small">
                Context
              </TableCell>
              <Hidden xsDown>
                <TableCell className={props.cellClasses} size="small">
                  Bookmarked
                </TableCell>
              </Hidden>
              <TableCell className={props.cellClasses} size="small">
                Delete
              </TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {this.state.tableData.map(n => {
              return (
                <TableRow
                  key={n.id}
                  onClick={event => this.handleRowClick(event, n)}
                >
                  <TableCell className={props.cellClasses} size="small">
                    <img
                      style={{ width: 40, height: 'auto' }}
                      alt=""
                      src={SubsonicSource.getCoverArtURL(n.coverArt, 40)}
                    />
                  </TableCell>
                  <TableCell className={props.cellClasses} size="small">
                    <Button
                      className={classes.unsetButton}
                      onClick={event => this.handleRowClick(event, n)}
                    >
                      {n.title.substring(0, 100)}
                    </Button>
                  </TableCell>
                  <TableCell className={props.cellClasses} size="small">
                    {n.contextString}
                  </TableCell>
                  <Hidden xsDown>
                    <TableCell className={props.cellClasses} size="small">
                      {n.bookmarkDate.toLocaleString()}
                    </TableCell>
                  </Hidden>
                  <TableCell className={props.cellClasses} size="small">
                    <IconButton
                      onClick={event => this.handleDeleteClick(event, n)}
                    >
                      <DeleteForever />
                    </IconButton>
                  </TableCell>
                </TableRow>
              );
            })}
          </TableBody>
        </Table>
        {nobookmarks}
      </Paper>
    );
  }
}

export default withStyles(styles)(BookmarksBrowser);
