import React, { Component } from 'react';
import { Link } from 'react-router-dom';

import MusicFolderChooser from './MusicFolderChooser';
import { ShuffleButton } from 'subsonic/components/controls/MediaButtons';

import Grid from '@material-ui/core/Grid';
import IconButton from '@material-ui/core/IconButton';
import List from '@material-ui/core/List';
import ListSubheader from '@material-ui/core/ListSubheader';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import { withStyles } from '@material-ui/core/styles';
import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';
import FolderOpen from '@material-ui/icons/FolderOpen';
import PlayIcon from '@material-ui/icons/VideoLibrary';

import classNames from 'classnames';

import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

const styles = theme => ({
  listRoot: {
    width: '100%',
    overflow: 'auto',
    display: 'inline-block'
  },
  subheader: {
    backgroundColor: theme.palette.background.paper
  },
  subhidden: {
    display: 'none'
  },
  listSection: {
    backgroundColor: 'inherit'
  },
  li: {
    listStyle: 'none'
  },
  ul: {
    backgroundColor: 'inherit',
    padding: 0
  },
  avatar: {
    margin: 5
  },
  bigAvatar: {
    width: 40,
    height: 40
  }
});

class IndexBrowser extends Component {
  constructor(props) {
    super(props);
    this.state = {
      currentIndex: null,
      showAll: true,
      musicDirectoryIndexes: []
    };
  }

  static getDerivedStateFromProps = (nextProps, lastState) => {
    const { subsonic } = nextProps;
    let { musicDirectoryIndexes, musicDirectories, musicFolder } = subsonic;
    musicDirectoryIndexes = musicDirectoryIndexes || [];
    musicDirectories = musicDirectories || [];
    if (!Array.isArray(musicDirectoryIndexes)) musicDirectoryIndexes = [musicDirectoryIndexes];
    if (!Array.isArray(musicDirectories)) musicDirectories = [musicDirectories];
    // console.log('musicDirectories', musicDirectories.length);

    if (musicDirectories.length < 20) {
      musicDirectoryIndexes = [
        {
          artist: musicDirectories,
          name: 'All'
        }
      ];
    }

    const showAll = musicDirectories.length < 201;
    const currentIndex = showAll
      ? null
      : musicFolder === lastState.musicFolder && lastState.currentIndex
      ? lastState.currentIndex
      : musicDirectoryIndexes[0].name;

    return {
      musicDirectoryIndexes,
      musicDirectories,
      showAll,
      currentIndex,
      musicFolder
    };
  };

  componentDidMount() {}

  scrollToTop = () => {
    document.querySelector('.indexList').scrollTop = 0;
  };

  shuffleClick = () => {
    var id = this.props.subsonic.musicFolder || '-1';
    this.props.history.push('/loading/folderShuffle/' + id);
  };

  playFolder = album => {
    this.props.history.push('/loading/directoryAction/' + album.id + '/all');
  };

  setCurrentIndex = idx => {
    this.setState({
      currentIndex: idx
    });
  };

  render() {
    const { subsonic, classes } = this.props;
    const { musicDirectoryIndexes, showAll, currentIndex } = this.state;

    if (!musicDirectoryIndexes || musicDirectoryIndexes.length === 0) {
      return <span>loading...</span>;
    }
    const subhidden = {};
    subhidden[classes.subhidden] = musicDirectoryIndexes.length < 10;
    const subheaderClasses = classNames(classes.subheader, subhidden);

    /*
index musicDirectory doesn't have cover art
<Avatar
  alt={`${item.name}`}
  src={SubsonicSource.getCoverArtURL(item.coverArt, 40)}
  className={classNames(
    classes.avatar,
    classes.bigAvatar
  )}
/>
*/
    return (
      <React.Fragment>
        <Grid container={true}>
          <Grid item={true} xs={10}>
            <MusicFolderChooser {...subsonic} />
          </Grid>
          <Grid item={true} xs={2}>
            <ShuffleButton shuffleClick={this.shuffleClick} />
          </Grid>
        </Grid>
        <List
          className={classNames(classes.listRoot, 'indexList')}
          subheader={<li />}
        >
          {musicDirectoryIndexes.map(index => (
            <li key={`section-${index.name}`} className={classes.listSection}>
              <ul
                className={classes.ul}
                onClick={
                  !showAll ? this.setCurrentIndex.bind(this, index.name) : null
                }
              >
                <ListSubheader className={subheaderClasses}>
                  {`Index: ${index.name}`}
                </ListSubheader>
                {(showAll || index.name === currentIndex
                  ? index.artist || []
                  : []
                ).map(item => (
                  <ListItem
                    classes={{ container: classes.li }}
                    key={`item-${index.name}-${item.id}`}
                    button={true}
                    component={Link}
                    to={`/loading/directory/${item.id}`}
                  >
                    <ListItemIcon>
                      <FolderOpen />
                    </ListItemIcon>
                    <ListItemText primary={`${item.name}`} />
                    <ListItemSecondaryAction>
                      <IconButton
                        aria-label="Play"
                        onClick={this.playFolder.bind(this, item)}
                      >
                        <PlayIcon />
                      </IconButton>
                    </ListItemSecondaryAction>
                  </ListItem>
                ))}
              </ul>
            </li>
          ))}
        </List>
        <ScrollToTopFab
          {...this.props.fabProps}
          scrollToTop={this.scrollToTop}
        />
      </React.Fragment>
    );
  }
}

export default withStyles(styles)(IndexBrowser);
