import React, { Component } from 'react';

import { withStyles } from '@material-ui/core/styles';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import Paper from '@material-ui/core/Paper';

import Hidden from '@material-ui/core/Hidden';

import SubsonicActions from 'subsonic/actions/SubsonicActions';

const styles = theme => ({
  root: {
    width: '100%',
    marginTop: theme.spacing(3),
    overflowX: 'auto'
  },
  table: {
    minWidth: 360
  }
});

class NowPlayingBrowser extends Component {
  static getDerivedStateFromProps(props, prevState) {
    if (prevState && prevState.nowPlaying === props.subsonic.nowPlaying)
      return prevState;
    const nowPlaying = props.subsonic.nowPlaying;
    // bookmarks = array of
    //  comment = JSON of type, id, source, potentially mode (albums)
    //  position
    //  entry:
    //    id, album, artist, coverArt, title
    // create the table structure here
    const tableData = nowPlaying.map((b, i, a) => {
      // console.log("tableData", b);
      const e = Object.assign({}, b);
      return e;
    }, []);
    return {
      nowPlaying,
      tableData
    };
  }

  constructor(props) {
    super(props);
    this.state = {};
    // this.state = NowPlayingBrowser.getDerivedStateFromProps(props);
  }

  // componentWillReceiveProps(nextProps) {
  //   this.setState(
  //     NowPlayingBrowser.getDerivedStateFromProps(nextProps, this.state)
  //   );
  // }

  componentDidMount() {
    SubsonicActions.getNowPlaying.defer();
    SubsonicActions.followUser.defer(null);
    this.interval = setInterval(SubsonicActions.getNowPlaying.defer, 10000);
  }

  componentWillUnmount() {
    clearInterval(this.interval);
  }

  handleRowClick(evt, np) {
    console.log('np', np);
    SubsonicActions.followUser(np);

    const { history } = this.props;
    history.push('/player');
    // hack for HashRouter
    // window.bookmarkState = bookmark;
    // history.push(path, bookmark);
  }

  render() {
    // console.log("render bookmarks", this.props, this.state);
    const { classes, SubsonicSource } = this.props;

    return (
      <Paper className={classes.root}>
        <Table className={classes.table}>
          <TableHead>
            <TableRow>
              <TableCell size="small" />
              <TableCell size="small">Title</TableCell>
              <Hidden xsDown>
                <TableCell size="small">Artist</TableCell>
              </Hidden>
              <Hidden xsDown>
                <TableCell size="small">Album</TableCell>
              </Hidden>
              <TableCell size="small">Player</TableCell>
              <TableCell size="small" />
            </TableRow>
          </TableHead>
          <TableBody>
            {this.state.tableData.map(n => {
              return (
                <TableRow
                  key={n.username + n.playerId + n.id}
                  onClick={event => this.handleRowClick(event, n)}
                >
                  <TableCell size="small">
                    <img
                      style={{
                        width: '100%',
                        maxWidth: '40px',
                        height: 'auto'
                      }}
                      alt=""
                      src={SubsonicSource.getCoverArtURL(n.coverArt, 40)}
                    />
                  </TableCell>
                  <TableCell size="small">{n.title}</TableCell>
                  <Hidden xsDown>
                    <TableCell size="small">{n.artist}</TableCell>
                  </Hidden>
                  <Hidden xsDown>
                    <TableCell size="small">{n.album}</TableCell>
                  </Hidden>
                  <TableCell size="small">
                    {n.username + '@' + n.playerName}
                  </TableCell>
                  <Hidden xsDown>
                    <TableCell size="small">
                      {n.minutesAgo + ' minutes ago'}
                    </TableCell>
                  </Hidden>
                  <Hidden mdUp>
                    <TableCell size="small">{n.minutesAgo}</TableCell>
                  </Hidden>
                </TableRow>
              );
            })}
          </TableBody>
        </Table>
      </Paper>
    );
  }
}

export default withStyles(styles)(NowPlayingBrowser);
