import React, { Component } from 'react';
import { Redirect } from 'react-router-dom';

import Typography from '@material-ui/core/Typography';
import Grid from '@material-ui/core/Grid';
import Button from '@material-ui/core/Button';
import { withStyles } from '@material-ui/core/styles';
import PlayIcon from '@material-ui/icons/PlayArrow';
import ShuffleIcon from '@material-ui/icons/Shuffle';

import SubsonicActions from 'subsonic/actions/SubsonicActions';
import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

import SongList from 'subsonic/components/browsers/SongList';
import { arrayShuffle } from 'utils/utils';

const styles = theme => ({
  list: {
    paddingInlineStart: 0
  },
  li: {
    listStyle: 'none'
  }
});

class PlaylistBrowser extends Component {
  static getDerivedStateFromProps = (nextProps, lastState) => {
    const incoming = nextProps.subsonic.playlistEntries || [];
    if (incoming.length !== lastState.contents.length) {
      return {
        contents: nextProps.subsonic.playlistEntries || []
      };
    }
    return lastState;
  };

  playItem = song => {
    if (this.props.remotes.remoteState === 'sender') {
      this.props.history.push('/loading/song/' + song.id);
    } else {
      SubsonicActions.startPlaying(song, 0);
      this.props.history.push('/player');
    }
  };

  playList = shuffle => {
    const { match } = this.props;
    const id = match.params.id;
    if (this.props.remotes.remoteState === 'sender') {
      this.props.history.push(
        '/loading/playlist/' + id + (shuffle ? '/shuffle' : '')
      );
    } else {
      let contents = this.state.contents;
      if (shuffle) {
        contents = arrayShuffle(contents.slice(0));
      }
      SubsonicActions.startPlaying(contents, 0);
      this.props.history.push('/player');
    }
  };

  constructor(props) {
    super(props);
    this.state = {
      contents: props.subsonic.playlistEntries
    };
  }

  render() {
    const { subsonic, SubsonicSource, match, classes } = this.props;
    const id = match.params.id;
    if (!subsonic || !subsonic.isConnected || !subsonic.allPlaylists) {
      return <Typography>awaiting login...</Typography>;
    }

    if (!subsonic.playlistInfo) {
      if (id === undefined || id === 'undefined') {
        return <Typography>problem 1</Typography>;
      }
      return <Redirect to={'/loading/playlist/' + id + '/list'} />;
    }

    if ('' + subsonic.playlistInfo.id !== '' + id) {
      return <Redirect to={'/loading/playlist/' + id + '/list'} />;
    }

    const { contents } = this.state;

    const buttons = (
      <Grid container>
        <Grid item xs={1} />
        <Grid item xs={2}>
          <Button
            variant="outlined"
            color="primary"
            className={classes.button}
            onClick={this.playList.bind(this, false)}
          >
            Play
            <PlayIcon className={classes.rightIcon} />
          </Button>
        </Grid>{' '}
        <Grid item xs={1} />
        <Grid item xs={2}>
          <Button
            variant="outlined"
            color="secondary"
            className={classes.button}
            onClick={this.playList.bind(this, true)}
          >
            Shuffle and Play
            <ShuffleIcon className={classes.rightIcon} />
          </Button>
        </Grid>
      </Grid>
    );

    const songListProps = {
      contents,
      SubsonicSource,
      showCover: true,
      coverSize: 48,
      sortable: true,
      handleSongClick: this.playItem
    };
    const songSection = (
      <SongList
        onChange={ids => {
          const songs = this.state.contents;
          let c = ids.map(id => {
            return songs.find(s => s.id * 1 === id * 1);
          });
          this.setState({ contents: c });
        }}
        {...songListProps}
      />
    );

    return (
      <div>
        {buttons}
        {songSection}
        <ScrollToTopFab {...this.props.fabProps} />
      </div>
    );
  }
}

export default withStyles(styles)(PlaylistBrowser);
