import React, { Component } from 'react';

import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableRow from '@material-ui/core/TableRow';

import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';
import IconButton from '@material-ui/core/IconButton';

import FormControlLabel from '@material-ui/core/FormControlLabel';
import FormControl from '@material-ui/core/FormControl';
import Input from '@material-ui/core/Input';
import InputLabel from '@material-ui/core/InputLabel';
import Select from '@material-ui/core/Select';
import MenuItem from '@material-ui/core/MenuItem';
import Checkbox from '@material-ui/core/Checkbox';

import Paper from '@material-ui/core/Paper';
import Typography from '@material-ui/core/Typography';

import GridList from '@material-ui/core/GridList';
import PlayIcon from '@material-ui/icons/PlayArrow';
import ShuffleIcon from '@material-ui/icons/Shuffle';

import CoverCard from 'subsonic/components/styled/CoverCard';
import CoverGridTile from 'subsonic/components/styled/CoverGridTile';
import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

class PlaylistsBrowser extends Component {
  constructor(props) {
    super(props);
    this.myRef = React.createRef();

    this.state = {
      shuffle: false
    };
  }

  static renderTypes = {
    GRID: 'grid',
    LIST: 'list',
    TABLE: 'table',
    SELECT: 'select',
    CARDS: 'cards'
  };

  updateDimensions = () => {
    this.setState({ width: window.innerWidth, height: window.innerHeight });
  };

  componentDidMount = () => {
    window.addEventListener('resize', this.updateDimensions);
    this.updateDimensions();
  };

  componentWillUnmount = () => {
    window.removeEventListener('resize', this.updateDimensions);
  };

  playlistLoad = id => {
    console.log('props!', id, this.props);
    if (this.props.onSelect)
      this.props.onSelect(
        id,
        this.props.route || {
          history: this.props.history,
          location: this.props.location,
          match: this.props.match
        },
        this.state.shuffle
      );
  };

  playlistSelect = evt => {
    this.playlistLoad(evt.target.value);
  };

  handleShuffleChange = (evt, checked) => {
    this.setState({ shuffle: checked });
  };

  render() {
    // console.warn("pl render", this.props);

    const {
      renderAllPlaylists,
      linkComponent,
      subsonic,
      renderMode,
      selectedId,
      SubsonicSource,
      native,
      includeEmpty,
      label,
      fullWidth,
      classes = {},
      showShuffleOption = false,
      renderEmptyMessage
    } = this.props;
    const { shuffle } = this.state;
    const { playlists, allPlaylists } = subsonic;
    const rm = renderMode || PlaylistsBrowser.renderTypes.TABLE;
    const PlayerRefLink = linkComponent;
    var pl = renderAllPlaylists ? allPlaylists : playlists;
    let rv;

    if (!pl || pl.length === 0) {
      let message = 'No Playlists have been added to this server.';
      if (!subsonic.isConnected) {
        message = 'Not yet connected...';
      } else if (subsonic.isBusy) {
        message = 'Still loading...';
      }
      rv = renderEmptyMessage ? (
        renderEmptyMessage(message)
      ) : (
        <Paper>
          <Typography variant="h4">{message}</Typography>
        </Paper>
      );
    } else if (rm === PlaylistsBrowser.renderTypes.CARDS) {
      const innerWidth = window.innerWidth;

      const cellSize = innerWidth > 560 ? 250 : 164;

      rv = (
        <React.Fragment>
          {pl.map((n, i) => (
            <CoverCard
              key={n.id}
              name={n.name}
              size={cellSize}
              coverArt={n.coverArt}
              link={'/loading/playlist/' + n.id}
              id={n.id}
              SubsonicSource={SubsonicSource}
              idx={i}
            />
          ))}
        </React.Fragment>
      );
    } else if (rm === PlaylistsBrowser.renderTypes.GRID) {
      let innerWidth = window.innerWidth;
      if (this.myRef && this.myRef.current) {
        innerWidth = this.myRef.current.offsetWidth;
      }

      if (isNaN(innerWidth)) innerWidth = 360;
      const spacing = innerWidth > 360 ? 10 : 6;
      const cellSize = innerWidth > 360 ? 170 : 164;

      const cols = innerWidth / (cellSize + spacing);
      const width = cols * (cellSize + spacing);
      const Icon = shuffle ? ShuffleIcon : PlayIcon;
      rv = (
        <div ref={this.myRef}>
          {showShuffleOption && (
            <FormControlLabel
              labelPlacement="start"
              control={
                <Checkbox
                  icon={<PlayIcon />}
                  checkedIcon={<ShuffleIcon />}
                  checked={shuffle}
                  onChange={this.handleShuffleChange}
                  value="shuffle"
                />
              }
              label="Shuffle on Play"
            />
          )}
          <GridList
            className="playlist-gridlist"
            cellHeight={cellSize}
            width={width}
            cols={cols}
            spacing={spacing}
          >
            {pl.map(n => (
              <CoverGridTile
                onClick={this.playlistLoad.bind(this, n.id)}
                key={'pl' + n.id}
                name={n.name}
                size={cellSize}
                coverArt={n.coverArt}
                options={{ shuffle }}
                linkComponent={linkComponent}
                id={n.id}
                SubsonicSource={SubsonicSource}
                icon={Icon}
              />
            ))}
          </GridList>
          <ScrollToTopFab {...this.props.fabProps} />
        </div>
      );
    } else if (rm === PlaylistsBrowser.renderTypes.LIST) {
      rv = (
        <div ref={this.myRef}>
          {showShuffleOption && (
            <FormControlLabel
              labelPlacement="start"
              control={
                <Checkbox
                  icon={<PlayIcon />}
                  checkedIcon={<ShuffleIcon />}
                  checked={shuffle}
                  onChange={this.handleShuffleChange}
                  value="shuffle"
                />
              }
              label="Shuffle on Play"
            />
          )}
          <List>
            {pl.map(n => (
              <ListItem
                key={'pl' + n.id}
                button
                onClick={this.playlistLoad.bind(this, n.id)}
              >
                <ListItemIcon>
                  <img
                    alt=""
                    width={48}
                    height={48}
                    src={SubsonicSource.getCoverArtURL(n.coverArt, 48)}
                  />
                </ListItemIcon>
                <ListItemText
                  primary={n.name}
                  secondary={n.songCount + ' songs'}
                />
                <ListItemSecondaryAction style={{ width: 48, height: 48 }}>
                  <IconButton
                    aria-label={shuffle ? 'Shuffle' : 'Play'}
                    component={PlayerRefLink}
                    id={n.id}
                    options={shuffle ? { shuffle } : {}}
                  >
                    {shuffle ? <ShuffleIcon /> : <PlayIcon />}
                  </IconButton>
                </ListItemSecondaryAction>
              </ListItem>
            ))}
          </List>
        </div>
      );
    } else if (rm === PlaylistsBrowser.renderTypes.TABLE) {
      rv = (
        <div ref={this.myRef}>
          {showShuffleOption && (
            <FormControlLabel
              labelPlacement="start"
              control={
                <Checkbox
                  icon={<PlayIcon />}
                  checkedIcon={<ShuffleIcon />}
                  checked={shuffle}
                  onChange={this.handleShuffleChange}
                  value="shuffle"
                />
              }
              label="Shuffle on Play"
            />
          )}
          <Table>
            <TableBody>
              {pl.map(n => {
                const selected = '' + n.id === '' + selectedId;
                return (
                  <TableRow selected={selected} key={'pl' + n.id}>
                    <TableCell>
                      <PlayerRefLink id={n.id}>
                        <img
                          crossOrigin="anonymous"
                          src={this.props.SubsonicSource.getCoverArtURL(
                            n.coverArt,
                            40
                          )}
                          alt={n.name}
                        />
                      </PlayerRefLink>
                    </TableCell>
                    <TableCell>
                      <PlayerRefLink id={n.id}>{n.name}</PlayerRefLink>
                    </TableCell>
                  </TableRow>
                );
              })}
            </TableBody>
          </Table>
        </div>
      );
    } else if (rm === PlaylistsBrowser.renderTypes.SELECT) {
      // console.log('select', pl, selectedId, typeof selectedId);
      var emptyRef = includeEmpty ? (
        native ? (
          <option value="" />
        ) : (
          <MenuItem value="" />
        )
      ) : null;
      rv = (
        <FormControl className={classes.formControl} fullWidth={fullWidth}>
          <InputLabel htmlFor="pl-label">{label}</InputLabel>
          <Select
            fullWidth={fullWidth}
            native={native}
            value={'' + selectedId}
            onChange={this.playlistSelect}
            id="pl-select"
            input={<Input id="pl-label" />}
          >
            {emptyRef}
            {pl.map((n, idx) => {
              // console.log('option', n.id, '' + selectedId === '' + n.id, n.name);
              if (native)
                return (
                  <option value={'' + n.id} key={'pl' + n.id + '-' + idx}>
                    {n.name}
                  </option>
                );
              return (
                <MenuItem value={'' + n.id} key={'pl' + n.id + '-' + idx}>
                  <img
                    crossOrigin="anonymous"
                    src={this.props.SubsonicSource.getCoverArtURL(
                      n.coverArt,
                      15
                    )}
                    alt={pl.name}
                  />
                  &nbsp;
                  {n.name}
                </MenuItem>
              );
            })}
          </Select>
          {showShuffleOption && (
            <FormControlLabel
              labelPlacement="end"
              control={
                <Checkbox
                  icon={<PlayIcon />}
                  checkedIcon={<ShuffleIcon />}
                  checked={shuffle}
                  onChange={this.handleShuffleChange}
                  value="shuffle"
                />
              }
              label="Shuffle on Play"
            />
          )}
        </FormControl>
      );
    }
    return rv;
  }
}

export default PlaylistsBrowser;
