import React from 'react';
import { Redirect } from 'react-router-dom';

import SubsonicActions from 'subsonic/actions/SubsonicActions';
import Paper from '@material-ui/core/Paper';
import Typography from '@material-ui/core/Typography';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import Collapse from '@material-ui/core/Collapse';
import ExpandLess from '@material-ui/icons/ExpandLess';
import ExpandMore from '@material-ui/icons/ExpandMore';
import { withStyles } from '@material-ui/core/styles';

const styles = theme => ({
  root: {
    width: '100%',
    // maxWidth: 360,
    backgroundColor: theme.palette.background.paper
  },
  nested: {
    paddingLeft: theme.spacing(4)
  }
});

class PodcastsBrowser extends React.Component {
  static getDerivedStateFromProps(props, prevState) {
    if (!prevState) prevState = { open: '-1' };
    return Object.assign({}, prevState);
  }

  constructor(props) {
    super(props);
    this.state = {};
    // this.state = PodcastsBrowser.getDerivedStateFromProps(props);
  }

  // componentWillReceiveProps(nextProps) {
  //   this.setState(PodcastsBrowser.getDerivedStateFromProps(nextProps, this.state));
  // }

  handleItemClick = (episode, evt) => {
    console.warn('playing', episode);
    SubsonicActions.playPodcast(episode);
    this.props.history.push('/player');
  };

  handleExpandClick = (id, evt) => {
    this.setState({ open: id === this.state.open ? '-1' : id });
  };

  renderPodcastItem = p => {
    const {
      classes,
      subsonic,
      SubsonicSource,
      podcastProps,
      podcastItemProps
    } = this.props;
    return (
      <ListItem
        key={p.id}
        button
        className={classes.nested}
        onClick={this.handleItemClick.bind(this, p)}
      >
        <ListItemText
          inset
          primary={p.title}
          primaryTypographyProps={podcastItemProps}
        />
      </ListItem>
    );
  };

  renderPodcast = p => {
    const { SubsonicSource, podcastProps } = this.props;
    return (
      <span key={p.id}>
        <ListItem button onClick={this.handleExpandClick.bind(this, p.id)}>
          <ListItemIcon>
            <img src={SubsonicSource.getCoverArtURL(p.coverArt, 24)} alt="" />
          </ListItemIcon>
          <ListItemText
            inset
            primary={p.title}
            primaryTypographyProps={podcastProps}
          />
          {this.state.open === p.id ? <ExpandLess /> : <ExpandMore />}
        </ListItem>
        <Collapse in={this.state.open === p.id} timeout="auto" unmountOnExit>
          <List component="div" disablePadding>
            {p.episode.slice(0, 12).map(e => {
              return this.renderPodcastItem(e);
            })}
          </List>
        </Collapse>
      </span>
    );
  };

  render() {
    const { classes, subsonic, remotes } = this.props;
    // yes, letting a little of subfire bleed in. not much i can do about it...
    let warning = null;
    if (remotes.remoteState === 'sender')
      warning = (
        <Typography>
          Note: currently, podcast loading does not work correctly over remote
          casting. It will load locally.
        </Typography>
      );

    if (subsonic.podcasts === null) {
      console.log('redirect to loading');
      return <Redirect to="/loading/podcasts/all" />;
      // return <span/>;
    }

    if (subsonic.podcasts.length === 0) {
      return (
        <Paper>
          <Typography variant="h4">
            No Podcasts have been added to this server.
          </Typography>
        </Paper>
      );
    }

    const listItems = subsonic.podcasts.map(p => {
      return this.renderPodcast(p);
    });

    return (
      <div className={classes.root}>
        {warning}
        <List component="nav">{listItems}</List>
      </div>
    );
  }
}

export default withStyles(styles)(PodcastsBrowser);
