import React from 'react';

import Select from '@material-ui/core/Select';
import { withStyles } from '@material-ui/core/styles';
import FormControl from '@material-ui/core/FormControl';
import MenuItem from '@material-ui/core/MenuItem';

const ServerSelect = props => {
  const {
    classes,
    credentials,
    handleServerChange,
    native,
    labelId,
    fullWidth = true
  } = props;
  const creds = credentials ? credentials.creds : [{ name: 'unconfigured...' }];
  const current = credentials ? credentials.current : 'unconfigured...';
  const isConnected = credentials ? credentials.isConnected : false;

  const className = isConnected ? '' : classes.notConnected;

  return (
    <FormControl className={classes.formControl} fullWidth={fullWidth}>
      <Select
        native={native}
        id="current-server"
        labelId={labelId}
        SelectDisplayProps={{ id: 'select-server' }}
        MenuProps={{ id: 'menu-server' }}
        value={current}
        onChange={handleServerChange}
        className={className}
      >
        {Object.entries(creds).map(o => {
          if (native)
            return (
              <option key={o[0]} value={o[0]}>
                {o[1].name}
              </option>
            );
          return (
            <MenuItem key={o[0]} value={o[0]}>
              {o[1].name}
            </MenuItem>
          );
        })}
      </Select>
    </FormControl>
  );
};

const styles = theme => ({
  formControl: {
    margin: 0
  },
  notConnected: {
    fontStyle: 'italic'
  }
});

export default withStyles(styles)(ServerSelect);
