import React, { Component } from 'react';

import Sortable from 'react-sortablejs';

// list items
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemIcon from '@material-ui/core/ListItemIcon';

// table items

import Typography from '@material-ui/core/Typography';
import { withStyles } from '@material-ui/core/styles';

const styles = theme => ({
  list: {
    paddingInlineStart: 0
  },
  li: {
    listStyle: 'none'
  }
});

class SongList extends Component {
  handleSongClick = (song, idx) => {
    if (this.props.handleSongClick) this.props.handleSongClick(song, idx);
  };

  render() {
    const {
      contents,
      idx,
      classes,
      showCover = true,
      coverSize = 24,
      SubsonicSource,
      sortable = false,
      showMeta = true
    } = this.props;

    let { sortedOptions } = this.props;

    const onChange = (items, sortable, evt) => {
      console.log(items, sortable);
      if (this.props.onChange) this.props.onChange(items, sortable, evt);
    };

    let songItems = '';
    songItems = contents.map((s, i, cp) => (
      <ListItem
        data-song-id={s.id}
        component="li"
        classes={{ container: classes.li }}
        id={'songs-li-' + i}
        className={
          idx === i ? classes.selectedSongListItem : classes.songListItem
        }
        key={i}
        button={true}
        onClick={this.handleSongClick.bind(this, s, i)}
      >
        {showCover && (
          <ListItemIcon
            style={{
              width: coverSize,
              height: coverSize
            }}
          >
            <img
              alt={s.title}
              src={SubsonicSource.getCoverArtURL(s.coverArt, coverSize)}
            />
          </ListItemIcon>
        )}
        <ListItemText>
          {i + 1}. {s.title}
          {showMeta && (
            <React.Fragment>
              <br />
              <Typography variant="caption">
                {s.artist}- {s.album}
              </Typography>
            </React.Fragment>
          )}
        </ListItemText>
      </ListItem>
    ));

    if (sortable) {
      sortedOptions = Object.assign({}, sortedOptions, {
        animation: 150,
        dataIdAttr: 'data-song-id',
        ghostClass: 'sortable-ghost', // Class name for the drop placeholder
        chosenClass: 'sortable-chosen', // Class name for the chosen item
        dragClass: 'sortable-drag' // Class name for the dragging item
      });
      return (
        <Sortable
          options={sortedOptions}
          className={classes.list}
          tag="ul"
          onChange={onChange}
        >
          {songItems}
        </Sortable>
      );
    }

    return <List className={classes.list}>{songItems}</List>;
  }
}

export default withStyles(styles)(SongList);
