import React, { Component } from 'react';

import { Link } from 'react-router-dom';

import MusicFolderChooser from './MusicFolderChooser';

import Avatar from '@material-ui/core/Avatar';
import ListSubheader from '@material-ui/core/ListSubheader';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import { withStyles } from '@material-ui/core/styles';

import classNames from 'classnames';

import ScrollToTopFab from 'subsonic/components/styled/ScrollToTopFab';

const styles = theme => ({
  root: {
    width: '100%',
    // maxWidth: 360,
    // maxHeight: 300,
    position: 'absolute',
    top: 48,
    bottom: 0,
    // backgroundColor: theme.palette.background.paper,
    overflow: 'auto'
  },
  subheader: {
    backgroundColor: theme.palette.background.paper
  },
  subhidden: {
    display: 'none'
  },
  listSection: {
    backgroundColor: 'inherit'
  },
  ul: {
    backgroundColor: 'inherit',
    padding: 0
  },
  avatar: {
    margin: 5
  },
  bigAvatar: {
    width: 40,
    height: 40
  }
});

class TreeBrowser extends Component {
  UNSAFE_componentWillReceiveProps(nextProps) {
    //console.log("ib will", nextProps);
  }

  componentDidMount() {}

  scrollToTop = () => {
    document.querySelector('.indexList').scrollTop = 0;
  };

  render() {
    const { subsonic, SubsonicSource, classes } = this.props;
    const { musicDirectoryIndexes } = subsonic;
    if (!musicDirectoryIndexes) return <span>loading...</span>;

    const subhidden = {};
    subhidden[classes.subhidden] = musicDirectoryIndexes.length < 10;
    const subheaderClasses = classNames(classes.subheader, subhidden);

    return (
      <div>
        <MusicFolderChooser {...subsonic} />
        <List
          className={classNames(classes.root, 'indexList')}
          subheader={<li />}
        >
          {musicDirectoryIndexes.map(index => (
            <li key={`section-${index.name}`} className={classes.listSection}>
              <ul className={classes.ul}>
                <ListSubheader className={subheaderClasses}>{`Index: ${
                  index.name
                }`}</ListSubheader>
                {index.artist.map(item => (
                  <ListItem
                    key={`item-${index.name}-${item.id}`}
                    button
                    component={Link}
                    to={`/loading/directory/${item.id}`}
                  >
                    <Avatar
                      alt={`${item.name}`}
                      src={SubsonicSource.getCoverArtURL(item.coverArt, 40)}
                      className={classNames(classes.avatar, classes.bigAvatar)}
                    />
                    <ListItemText primary={`${item.name}`} />
                  </ListItem>
                ))}
              </ul>
            </li>
          ))}
        </List>
        <ScrollToTopFab scrollToTop={this.scrollToTop} />
      </div>
    );
  }
}

export default withStyles(styles)(TreeBrowser);
