import React from 'react';
import ActionButton from './ActionButton';

import {
  PlayArrow,
  Pause,
  SkipNext,
  SkipPrevious,
  Redo,
  Replay10,
  Forward30,
  Shuffle,
  Bookmark,
  BookmarkBorder,
  Update,
  Queue
} from '@material-ui/icons';
// import { withStyles } from '@material-ui/core/styles';

import SubsonicActions from 'subsonic/actions/SubsonicActions';

export const PlayPauseButton = props => {
  let label = 'play',
    icon = <PlayArrow className={props.iconClassName} />;
  if (props.isPlaying) {
    label = 'pause';
    icon = <Pause className={props.iconClassName} />;
  } else if (props.isLoading) {
    label = 'loading';
    icon = <Update className={props.iconClassName} />;
  }
  // console.warn(props);
  const coreProps = {
    action: props.actions ? props.actions.playOrPause : null,
    label: label,
    disabled: props.isLoading,
    fab: props.fab,
    color: props.color,
    standalone: props.standalone,
    buttonClassName: props.buttonClassName
  };
  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export const NextButton = props => {
  const icon = <SkipNext className={props.iconClassName} />;
  const coreProps = {
    action: SubsonicActions.playerNext,
    label: 'next',
    // disabled: props.isLoading,
    fab: props.fab,
    buttonClassName: props.buttonClassName
  };

  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export const PrevButton = props => {
  const icon = <SkipPrevious className={props.iconClassName} />;
  const playerPrev = () => {
    var currentTime = props.currentMediaTime;
    if (props.actions && props.actions.seekTo && currentTime > 10) {
      props.actions.seekTo(0);
    } else {
      SubsonicActions.playerPrev();
    }
  };

  const coreProps = {
    action: playerPrev,
    label: 'previous',
    // disabled: props.isLoading,
    fab: props.fab,
    buttonClassName: props.buttonClassName
  };

  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export const ShuffleButton = props => {
  const icon = <Shuffle className={props.iconClassName} />;
  const coreProps = {
    action: props.shuffleClick || SubsonicActions.shufflePlaying,
    label: 'shuffle',
    // disabled: props.isLoading,
    fab: props.fab,
    buttonClassName: props.buttonClassName
  };

  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export const SkipAlbumButton = props => {
  const icon = <Redo className={props.iconClassName} />;
  const coreProps = {
    action: SubsonicActions.playerSkip,
    label: 'skip album',
    // disabled: props.isLoading,
    fab: props.fab,
    buttonClassName: props.buttonClassName
  };

  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export const Replay10Button = props => {
  const replay10Action = () => {
    var currentTime = props.currentMediaTime;
    if (props.actions && props.actions.seekTo) {
      props.actions.seekTo(Math.max(0, props.currentMediaTime - 10));
    }
  };

  const icon = <Replay10 className={props.iconClassName} />;
  const coreProps = {
    action: replay10Action,
    label: 'replay 10 seconds',
    disabled: props.isLoading,
    fab: props.fab,
    buttonClassName: props.buttonClassName
  };

  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export const Forward30Button = props => {
  const forward30Action = () => {
    if (props.actions && props.actions.seekTo) {
      props.actions.seekTo(
        Math.min(props.duration - 1, props.currentMediaTime + 30)
      );
    }
  };

  const icon = <Forward30 className={props.iconClassName} />;
  const coreProps = {
    action: forward30Action,
    label: 'skip 30 seconds',
    disabled: props.isLoading,
    fab: props.fab,
    buttonClassName: props.buttonClassName
  };

  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export const BookmarkButton = props => {
  const bookmarkToggle = () => {
    console.log('internal bookmarkToggle', props);
    props.action(props.currentMediaTime || 0);
  };

  let label = 'not bookmarked',
    icon = <BookmarkBorder className={props.iconClassName} />;
  if (props.isBookmarked) {
    label = 'bookmarked';
    icon = <Bookmark className={props.iconClassName} />;
  }
  const coreProps = {
    action: bookmarkToggle,
    label: label,
    // disabled: props.isLoading,
    fab: props.fab,
    buttonClassName: props.buttonClassName
  };

  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export const SavePlayQueueButton = props => {
  const savePlayQueue = () => {
    SubsonicActions.savePlayQueue(
      Math.trunc((props.currentMediaTime || 0) * 1000)
    );
  };

  const icon = <Queue className={props.iconClassName} />;
  const coreProps = {
    action: savePlayQueue,
    label: 'save play queue',
    // disabled: props.isLoading,
    fab: props.fab,
    buttonClassName: props.buttonClassName
  };

  return <ActionButton {...coreProps}>{icon}</ActionButton>;
};

export class ChromecastButton extends React.Component {
  render() {
    // return (<google-cast-launcher class="fakeMuiButton1 fakeMuiButton2 fakeMuiButton3"/>);
    const props = this.props;
    const coreProps = {
      // disabled: props.isLoading,
      fab: props.fab,
      buttonClassName: props.buttonClassName
    };
    return (
      <ActionButton {...coreProps}>
        <google-cast-launcher class="googleCastButton" />
      </ActionButton>
    );
  }
}
