import React from 'react';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogTitle from '@material-ui/core/DialogTitle';
import Typography from '@material-ui/core/Typography';
import Button from '@material-ui/core/Button';
import TVResponsiveDialog from 'subsonic/components/hoc/TVResponsiveDialog';
import { withStyles } from '@material-ui/core/styles';

import CredentialsActions from 'subsonic/actions/CredentialsActions';

class ImportCredentials extends React.Component {
  constructor(props) {
    super(props);

    const ls = window.localStorage;

    let configs = ls.getItem('configs');
    let config;
    if (configs) {
      configs = JSON.parse(configs);
      config = ls.getItem('config');
    }

    let credentials = ls.getItem('credentials');
    if (credentials) {
      credentials = JSON.parse(credentials);
    }
    this.state = {
      config,
      configs,
      credentials
    };
  }

  handleSubmitPlayer = () => {
    CredentialsActions.importSubFirePlayerCredentials();
  };

  handleSubmitMobile = () => {
    CredentialsActions.importSubFireM1Credentials();
  };

  handleClose = () => {
    if (this.props.handleClose) {
      this.props.handleClose();
    } else if (this.props.route && this.props.route.history) {
      this.props.route.history.goBack();
    }
  };

  render() {
    const { classes } = this.props;
    const { config, configs, credentials } = this.state;

    const playerCancel = credentials ? null : (
      <Button onClick={this.handleClose}>Cancel</Button>
    );

    const player = configs ? (
      <React.Fragment>
        <DialogContent>
          <Typography variant="h6">SubFire Player</Typography>
          <Typography>{`Found config ${config} with server ${
            configs[config].server
          }, username ${configs[config].username} as 1 of ${
            Object.keys(configs).length
          } config(s) that can be imported.`}</Typography>
        </DialogContent>
        <DialogActions>
          {playerCancel}
          <Button color="primary" onClick={this.handleSubmitPlayer}>
            Import SubFire Player
          </Button>
        </DialogActions>
      </React.Fragment>
    ) : null;

    const mobile = credentials ? (
      <React.Fragment>
        <DialogContent>
          <Typography variant="h6">SubFire Mobile</Typography>
          <Typography>{`Found server ${credentials.server}, username ${
            credentials.username
          } that can be imported.`}</Typography>
        </DialogContent>
        <DialogActions>
          <Button onClick={this.handleClose}>Cancel</Button>,
          <Button color="primary" onClick={this.handleSubmitMobile}>
            Import SubFire Mobile
          </Button>
        </DialogActions>
      </React.Fragment>
    ) : null;

    const nothing = !(player || mobile) ? (
      <React.Fragment>
        <DialogContent>
          <Typography>There are no configurations to import.</Typography>
        </DialogContent>
        <DialogActions>
          <Button onClick={this.handleClose} color="primary">
            Close
          </Button>
          ,
        </DialogActions>
      </React.Fragment>
    ) : null;

    return (
      <TVResponsiveDialog
        disableBackdropClick
        open={true}
        className={classes.paperFullScreen}
      >
        <DialogTitle>{'Import Credentials'}</DialogTitle>
        {player}
        {mobile}
        {nothing}
      </TVResponsiveDialog>
    );
  }
}

// Uncomment properties you need
// LoginComponent.propTypes = {};
// LoginComponent.defaultProps = {};

const styles = theme => ({
  root: {
    display: 'flex',
    flexWrap: 'wrap'
  },
  formControl: {
    margin: 0,
    width: '100%'
  },
  selectEmpty: {
    marginTop: theme.spacing(2)
  }
});

export default withStyles(styles)(ImportCredentials);
