import React from 'react';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogContentText from '@material-ui/core/DialogContentText';
import DialogTitle from '@material-ui/core/DialogTitle';
import Button from '@material-ui/core/Button';
import TextField from '@material-ui/core/TextField';
import Select from '@material-ui/core/Select';
import InputLabel from '@material-ui/core/InputLabel';
import MenuItem from '@material-ui/core/MenuItem';
import FormControl from '@material-ui/core/FormControl';
import TVResponsiveDialog from 'subsonic/components/hoc/TVResponsiveDialog';
import { withStyles } from '@material-ui/core/styles';

class LoginCredentials extends React.Component {
  constructor(props) {
    super(props);
    // console.log(props);
    this.handlePasswordKeyDown = this.handlePasswordKeyDown.bind(this);
    var currentCreds = props.current;
    this.state = {
      current: props.credentials.current,
      credentials: Object.assign({}, currentCreds)
    };
  }

  UNSAFE_componentWillReceiveProps(newProps) {
    // console.warn('lc componentWillReceiveProps', newProps);
    const props = newProps;
    var currentCreds = props.current;
    this.setState({
      current: props.credentials.current,
      credentials: Object.assign({}, currentCreds)
    });
  }

  handleSubmit = () => {
    var creds = this.state.credentials;
    var newCreds = {
      name: document.getElementById('name').value,
      server: document.getElementById('server').value,
      username: document.getElementById('username').value,
      password: document.getElementById('password').value || creds.password,
      bitrate: document.getElementById('bitrate')
        ? document.getElementById('bitrate').value
        : 0
    };
    if (newCreds.server.endsWith('/'))
      newCreds.server = newCreds.server.substring(
        0,
        newCreds.server.length - 1
      );
    if (this.props.handleSubmit)
      this.props.handleSubmit(this.state.current, newCreds);
    if (this.props.route && this.props.route.history)
      this.props.route.history.goBack();
  };

  handleDelete = () => {
    //console.log(this.state.current);
    if (this.props.handleDelete) this.props.handleDelete(this.state.current);
    if (this.props.route && this.props.route.history)
      this.props.route.history.goBack();
  };

  handleClose = () => {
    //console.log('handleClose called');
    if (this.props.handleClose) this.props.handleClose();
    if (this.props.route && this.props.route.history)
      this.props.route.history.goBack();
  };

  handlePasswordKeyDown(evt) {
    if (evt.keyCode + '' === '13') {
      this.handleSubmit();
      return false;
    }
    return true;
  }

  handleImport = () => {
    this.props.route.history.push('/import');
  };

  handleChangeCurrent = e => {
    //console.log(e.target.value);
    var state = this.state;
    state.current = e.target.value;
    if (state.current === '__new__') {
      state.credentials = Object.assign({}, state.credentials);
      state.credentials.name = 'Set Name Here';
    } else {
      state.credentials = this.props.credentials.creds[state.current];
    }
    this.setState(state);
  };

  handleChange = property => e => {
    //console.log(property, e.target.value);
    var state = this.state;
    state.credentials[property] = e.target.value;
    this.setState(state);
  };

  render() {
    const { classes, native } = this.props;
    const state = Object.assign({}, this.state);
    state.loginState = {};

    const isDeleteDisabled =
      this.props.credentials.current === state.current ||
      state.current === '__new__';

    let isFailure =
      state.loginState.isLoggedIn !== null &&
      state.loginState.isLoggedIn !== undefined &&
      !state.loginState.isLoggedIn;

    let p = '';
    if (isFailure === true) {
      p = <DialogContentText>The Login Failed...</DialogContentText>;
    } else {
      p = null;
    }
    const bitrates = [
      '320',
      '256',
      '192',
      '160',
      '128',
      '96',
      '80',
      '64',
      '32'
    ];
    const options = [
      {
        value: '0',
        primaryText: 'Unlimited'
      }
    ];
    bitrates.forEach(function(br) {
      options.push({ value: br, primaryText: br + ' Kbps' });
    });

    var that = this;

    const currentSelectorField = (
      <FormControl className={classes.formControl}>
        <InputLabel htmlFor="current-selection">Current Selection</InputLabel>
        <Select
          native={native}
          id="current-selection"
          value={this.state.current}
          onChange={this.handleChangeCurrent}
        >
          {Object.entries(this.props.credentials.creds).map(o => {
            if (native)
              return (
                <option key={o[0]} value={o[0]}>
                  {o[1].name}
                </option>
              );
            return (
              <MenuItem key={o[0]} value={o[0]}>
                {o[1].name}
              </MenuItem>
            );
          })}
          {native && (
            <option key="__new__" value="__new__">
              {state.current === '__new__'
                ? state.credentials.name
                : 'New Server'}
            </option>
          )}
          {!native && (
            <MenuItem key="__new__" value="__new__">
              {state.current === '__new__'
                ? state.credentials.name
                : 'New Server'}
            </MenuItem>
          )}
        </Select>
      </FormControl>
    );

    // const bitrateField = (
    //   <FormControl fullWidth>
    //     <InputLabel htmlFor="bitrate">Bitrate</InputLabel>
    //     <Select
    //       native={native}
    //       fullWidth
    //       id="bitrate"
    //       value={state.credentials.bitrate || "0"}
    //       onChange={that.handleChange("bitrate")}
    //     >
    //       {options.map(o => {
    //         if (native) {
    //           return (
    //             <option key={o} value={o.value}>
    //               {o.primaryText}
    //             </option>
    //           );
    //         } else {
    //           return (
    //             <MenuItem key={o.primaryText} value={o.value}>
    //               {o.primaryText}
    //             </MenuItem>
    //           );
    //         }
    //       })}
    //     </Select>
    //   </FormControl>
    // );

    const bitrateField = null;
    const showBitRateField = false;
    const bitrateFieldRender = showBitRateField ? bitrateField : '';

    const existing = this.props.credentials.creds[state.current];
    let passwordValues = {};
    if (existing && existing.password === state.credentials.password) {
      passwordValues.label = 'Password: unchanged';
      passwordValues.value = '';
    } else {
      passwordValues.label = 'Password';
      passwordValues.value = state.credentials.password;
    }

    return (
      <TVResponsiveDialog
        disableBackdropClick
        open={true}
        className={classes.paperFullScreen}
      >
        <DialogTitle>{'Server Credentials'}</DialogTitle>
        <DialogContent>
          {p}
          {currentSelectorField}
          <TextField
            id="name"
            label="Name"
            margin="dense"
            fullWidth
            onChange={this.handleChange('name')}
            value={state.credentials.name}
          />
          <TextField
            id="server"
            label="Server"
            margin="dense"
            helperText="http(s)://host:port"
            fullWidth
            onChange={this.handleChange('server')}
            value={state.credentials.server}
          />
          <TextField
            id="username"
            label="Username"
            margin="dense"
            fullWidth
            onChange={this.handleChange('username')}
            value={state.credentials.username}
          />
          <TextField
            id="password"
            type="password"
            margin="dense"
            fullWidth
            onChange={this.handleChange('password')}
            onKeyDown={this.handlePasswordKeyDown}
            {...passwordValues}
          />
          {bitrateFieldRender}
        </DialogContent>
        <DialogActions>
          <Button onClick={this.handleImport}>Import</Button>
          <Button disabled={isDeleteDisabled} onClick={this.handleDelete}>
            Delete
          </Button>
          <Button onClick={this.handleClose}>Cancel</Button>
          <Button color="primary" onClick={this.handleSubmit}>
            Submit
          </Button>
        </DialogActions>
      </TVResponsiveDialog>
    );
  }
}

// Uncomment properties you need
// LoginComponent.propTypes = {};
// LoginComponent.defaultProps = {};

const styles = theme => ({
  root: {
    display: 'flex',
    flexWrap: 'wrap'
  },
  formControl: {
    margin: 0,
    width: '100%'
  },
  selectEmpty: {
    marginTop: theme.spacing(2)
  }
});

export default withStyles(styles)(LoginCredentials);
