import React, { Component } from 'react';

import Grid from '@material-ui/core/Grid';
import FormControl from '@material-ui/core/FormControl';
import FormControlLabel from '@material-ui/core/FormControlLabel';
import Input from '@material-ui/core/Input';
import InputAdornment from '@material-ui/core/InputAdornment';
import InputLabel from '@material-ui/core/InputLabel';
import Switch from '@material-ui/core/Switch';

import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import {
  Album,
  People,
  LibraryMusic,
  Search as SearchIcon
} from '@material-ui/icons';

import MusicFolderChooser from 'subsonic/components/browsers/MusicFolderChooser';
import SubsonicActions from 'subsonic/actions/SubsonicActions';

import ArtistSearchResults from 'subsonic/components/search/ArtistSearchResults';
import AlbumSearchResults from 'subsonic/components/search/AlbumSearchResults';
import SongSearchResults from 'subsonic/components/search/SongSearchResults';

const ARTIST_TAB = 0;
const ALBUM_TAB = 1;
const SONG_TAB = 2;

class Search extends Component {
  constructor(props) {
    super(props);
    this.state = {
      query: '',
      id3: false,
      value: SONG_TAB,
      albumPage: 0,
      artistPage: 0,
      songPage: 0,
      pageSize: 10
    };
  }

  startSearch = () => {
    // due to music folder change, this may need to be deferred
    const params = {
      query: this.state.query + '*',
      id3: this.state.id3,
      searchType:
        this.state.value === 0
          ? 'artist'
          : this.state.value === 1
          ? 'album'
          : 'song',
      pageSize: 300, // this.state.pageSize,
      nextPage:
        this.state.value === 0
          ? this.state.albumPage
          : this.state.value === 1
          ? this.state.artistPage
          : this.state.songPage
    };
    SubsonicActions.doSearch.defer(params);
  };

  handleSearchChange = (evt, value) => {
    this.setState({ query: evt.target.value }, () => {
      this.startSearch();
    });
  };

  handleID3Change = (evt, isInputChecked) => {
    this.setState({ id3: isInputChecked }, () => {
      this.startSearch();
    });
  };

  searchFolderChanged = value => {
    SubsonicActions.setSearchMusicFolder(value);
    this.startSearch();
  };

  changeTab = (event, value) => {
    this.setState({ value }, () => {
      this.startSearch();
    });
  };

  render() {
    const { subsonic } = this.props;
    const { musicFolders, searchMusicFolder } = subsonic;
    const { query, id3, value, songPage, pageSize } = this.state;

    const musicChooserProps = {
      musicFolder: searchMusicFolder,
      musicFolders: musicFolders
    };

    const label = 'ID3?';
    const id3Switch = (
      <Switch
        label={label}
        value="id3"
        checked={id3}
        onChange={this.handleID3Change}
      />
    );
    const id3Selector = (
      <Grid item={true} xs={3}>
        <FormControlLabel control={id3Switch} label="ID3" />
      </Grid>
    );

    return (
      <Grid container={true}>
        <Grid xs={9} item={true}>
          <MusicFolderChooser
            {...musicChooserProps}
            musicFolderChanged={this.searchFolderChanged}
          />
        </Grid>
        {id3Selector}
        <Grid xs={9} item={true}>
          <FormControl fullWidth={true}>
            <InputLabel htmlFor="search-input">
              Search String (use * for wildcards)
            </InputLabel>
            <Input
              defaultValue={query}
              onChange={this.handleSearchChange}
              id="search-input"
              startAdornment={
                <InputAdornment position="start">
                  {' '}
                  <SearchIcon />
                </InputAdornment>
              }
            />
          </FormControl>
        </Grid>
        <Grid xs={12} item={true}>
          <Tabs
            value={value}
            onChange={this.changeTab}
            fullWidth={true}
            indicatorColor="primary"
            textColor="primary"
          >
            <Tab icon={<Album />} />
            <Tab icon={<People />} />
            <Tab icon={<LibraryMusic />} />
          </Tabs>
        </Grid>
        <Grid xs={12} item={true}>
          {value === ARTIST_TAB && <ArtistSearchResults />}
          {value === ALBUM_TAB && <AlbumSearchResults />}
          {value === SONG_TAB && (
            <SongSearchResults
              results={subsonic.searchSongsResults}
              page={songPage}
              pageSize={pageSize}
            />
          )}
        </Grid>
      </Grid>
    );
  }
}

export default Search;
