import alt from "Dispatcher";
import uuidv4 from "uuid/v4";
import SubsonicActions from "../actions/SubsonicActions";
import CredentialsActions from "../actions/CredentialsActions";
import SubsonicSource from "../sources/SubsonicSource";

const persistEvents = [
  'doLogin', 'changeCurrent', 'saveCredentials', 'deleteCredentials', 'totalReset', 'importCredentials'
];

const SubsonicDemo = {
  name: "Subsonic Demo",
  server: "http://demo.subsonic.org",
  username: "guest1",
  password: "guest",
  bitrate: "0"
}

const SubFireDemo = {
  name: "SubFire Demo",
  server: "https://subfiresuite.com/ampache",
  username: "guest",
  password: "guest1",
  bitrate: "0"
}

export class CredentialsStoreClass {
  constructor() {
    var storageState = localStorage.getItem("subsonic.credentials");
    // console.log(storageState);
    if (storageState) {
      storageState = JSON.parse(storageState);
      // TODO:[JS] legacy load from subfire or subfirep credentials...
    } else {
      const demo = uuidv4();
      const creds = {};
      creds[demo] = SubFireDemo;
      storageState = {
        creds: creds,
        current: demo
      };
    }
    // console.log(storageState);
    this.creds = storageState.creds;
    this.current = storageState.current;
    this.isConnected = false;

    this.bindActions(CredentialsActions);
    this.bindAction(SubsonicActions.LOGIN_SUCCESS, this.loginSuccess);
    this.bindAction(SubsonicActions.LOGIN_FAILURE, this.loginFailure);

    this.on('afterEach', (payload, state) => this.afterEach(payload, state));
  }

  loginSuccess() {
    this.isConnected = true;
  }

  loginFailure() {
    this.isConnected = false;
  }

  afterEach({payload, state}) {
    //console.log(payload);
    if (persistEvents.indexOf(payload.details.name) >= 0) {
      //console.log("after", payload, state);
      this.persistToStorage(state);
    }
  }

  persistToStorage(state) {
    //console.log('persisting', JSON.stringify(state));
    localStorage.setItem("subsonic.credentials", JSON.stringify(state));
  }

  doLogin() {
    //console.log('doLogin');
    var credentials = this.creds[this.current];
    if (SubsonicSource.isLoginNeeded(credentials)) {
      // console.log('attemting...');
      SubsonicSource.login(credentials).then(() => {
        // console.log('success');
        SubsonicActions.loginSuccess();
      }).catch(error => {
        console.warn('error', error);
        SubsonicActions.loginFailure();
      });
    }
  }

  importSubFirePlayerCredentials() {
    const configs = JSON.parse(localStorage.getItem('configs'));
    const defaultConfig = localStorage.getItem('config');
    Object.keys(configs).forEach((key, idx) => {
      const id = uuidv4();
      const creds = configs[key];
      if (key === defaultConfig) this.current = id;
      this.creds[id] = {
        name: key + " (SubFirePlayer Import)",
        server: creds.server,
        username: creds.username,
        password: creds.password,
        bitrate: creds.bitrate
      };
    });
  }

  importSubFireM1Credentials() {
    const id = uuidv4();
    const creds = JSON.parse(localStorage.getItem('credentials'));
    this.creds[id] = {
      name: 'SubFire Mobile Import',
      server: creds.server,
      username: creds.username,
      password: creds.pwd,
      bitrate: "0"
    };
    this.current = id;
    CredentialsActions.doLogin.defer();
  }

  importCredentials(newCreds) {
    var creds = this.creds;
    // todo: look for name overlaps
    this.creds = Object.assign({}, creds, newCreds.creds);
    this.current = newCreds.current;
    CredentialsActions.doLogin.defer();
  }

  changeCurrent([current, doLogin]) {
    if (current !== this.current) {
      const currentCreds = this.creds[current];
      if (currentCreds) {
        this.current = current;
      }
    }
    if (doLogin) CredentialsActions.doLogin.defer();
  }

  saveCredentials([id, creds, doLogin]) {
    //console.log('saveCredentials', id, creds, doLogin);
    if (!id || id === '__new__') id = uuidv4();
    this.creds[id] = creds;
    this.current = id;
    if (doLogin) CredentialsActions.doLogin.defer();
  }

  deleteCredentials(id) {
    if (id === this.current) return;
    delete this.creds[id];
  }

  totalReset() {
    const demo = uuidv4();
    const creds = {};
    creds[demo] = SubFireDemo;
    const storageState = {
      creds: creds,
      current: demo
    };
    // console.log(storageState);
    this.setState(storageState);
  }
}

export default alt.createStore(CredentialsStoreClass, "CredentialsStore");
