import alt from 'Dispatcher';
/* eslint no-unused-vars: 0 */

import SubsonicActions from '../actions/SubsonicActions';
import SubsonicSource from '../sources/SubsonicSource';

import { arrayShuffle } from 'utils/utils';

let SubsonicStore;

export const emptytrack = {
  album: '',
  albumId: '-1',
  artist: '',
  year: null,
  coverArt: '-1',
  albumYear: 'restarting the play queue',
  contentType: 'audio/mp3',
  duration: 2,
  title: 'restarting...',
  track: 1,
  localPath: './silence2sec.mp3'
};

export class SubsonicStoreClass {
  constructor() {
    this.exportPublicMethods({
      persistQueueLocally: this.persistQueueLocally
    });

    let initMusicFolder = localStorage.getItem('initMusicFolder') || -1;
    initMusicFolder *= 1;

    let currentPlaying = localStorage.getItem('subsonic.lastPlaying');
    if (currentPlaying) {
      let currentPlayingTime =
        localStorage.getItem('subsonic.lastPlaying.currentMediaTime') || '0';
      currentPlayingTime = 1 * currentPlayingTime;
      currentPlaying = JSON.parse(currentPlaying);
      window.bookmarkSeekTo = currentPlayingTime;
    } else {
      currentPlaying = {
        queue: [],
        idx: 0
      };
    }

    let initAlbumQuery = localStorage.getItem('initAlbumQuery') || 'newest';
    let initAlbumID3 = localStorage.getItem('initAlbumID3') === 'true';

    this.state = {
      isShowingMenu: false, // for applications that use it e.g., tv
      // only for visual state reference but all album queries need to send the id3 parameter now
      albumID3: initAlbumID3,

      chromecastAvailable: false,
      chromecastConnected: false,

      isConnected: false,

      postLoginRoute: 'playlists',

      lastLoadingTitle: null,

      allPlaylists: [],
      playlists: [],
      stations: [],
      browsingPlaylist: null,

      internetStations: [],

      playlist: null,
      playlistInfo: null,
      playlistEntries: [],
      playlistNeedsSave: false,
      sortRule: null,
      shufflePlaylist: false,

      musicFolder: initMusicFolder,
      musicFolders: [],

      musicDirectoryIndexes: [],
      musicDirectoryIndex: null,

      nowPlaying: [],
      // username: name, player: id
      following: null,

      albumQuery: initAlbumQuery,
      albumPage: 0,
      albumsPerPage: 64,
      albums: [],
      hasMoreAlbums: false,

      podcasts: null,

      bookmarkContext: null,
      bookmarks: [],
      bookmarksById: {},

      // not sure we'll use this...
      browsing: {
        type: null, // top-folder, musicDirectory, playlists, playlistSongs, artists, artistAlbums, artistAlbumSongs, getSongsQuery, search?
        contents: []
      },

      allArtistIndexes: [],
      artistIndexes: [],
      artistIndex: {},
      artist: null, // contains the full artist+album stuff
      album: null,
      albumsById: {},

      videos: [],

      searchMusicFolder: -1, // always All Music Folders at first
      searchAlbumResults: [],
      searchArtistResults: [],
      searchSongsResults: [],

      currentPlaying: currentPlaying,
      currentQueue: currentPlaying.queue || [],

      albumsAsGrid: true,

      isBusy: false
    };

    this.bindActions(SubsonicActions);
  }

  loginSuccess() {
    console.log('loginSuccess');
    this.refreshPlaylists();
    this.refreshInternetStations();
    this.fetchMusicFolders();
    this.getBookmarks();
    SubsonicSource.fetchAllArtists()
      .then(res => {
        this.setAllArtists(res || []);
        return SubsonicSource.fetchArtists(this.state.musicFolder || null);
      })
      .then(res => {
        this.setArtists(res.indexes || []);
        return SubsonicSource.fetchIndexes(this.state.musicFolder || null);
      })
      .then(res => {
        this.setMusicDirectoryIndexes(res.indexes || []);
      })
      .catch(err => {
        console.error(err);
      });

    this.mergeState({ isConnected: true, podcasts: null });
    SubsonicActions.snack.defer('Login Successful', {
      variant: 'success',
      autoHideDuration: 3000
    });
  }

  loginFailure() {
    console.log('loginFailure');
    this.mergeState({ isConnected: false });
    SubsonicActions.snack.defer('Login Failure, please try again?', {
      variant: 'error',
      autoHideDuration: 10000
    });
  }

  mergeState(newValues) {
    const s = this.state,
      rv = Object.assign({}, s, newValues);
    this.setState(rv);

    if (newValues.currentPlaying) {
      delete window.bookmarkSeekTo;
      this.persistQueueLocally();
    }
  }

  toggleMenu() {
    this.mergeState({ isShowingMenu: !this.state.isShowingMenu });
  }

  toggleAlbumsAsGrid() {
    this.mergeState({ albumsAsGrid: !this.state.albumsAsGrid });
  }

  setPostLoginRoute(route) {
    this.mergeState({ postLoginRoute: route });
  }

  setChromecastAvailable(chromecastAvailable) {
    this.mergeState({ chromecastAvailable: chromecastAvailable });
  }

  setChromecastConnected(chromecastConnected) {
    this.mergeState({ chromecastConnected: chromecastConnected });
  }

  setBusy(busy) {
    this.mergeState({ isBusy: busy });
  }

  followUser(following) {
    this.mergeState({ following: following });
  }

  // ----------------------- STATION ------------------------
  selectStation(pl) {
    this.mergeState({ isBusy: true });
    window.fetcher = SubsonicSource.fetcher;
    pl = this.state.stations.find(function(p) {
      return p.id + '' === pl + '';
    });
    SubsonicSource.generateStation(pl)
      .then(r => {
        this.populateStation(r);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
  }

  populateStation(data) {
    var entry = data && data.playlist ? data.playlist.entry || [] : [];
    this._storeCurrentQueue(entry);
  }

  refreshInternetStations() {
    SubsonicSource.refreshInternetStations()
      .then(r => {
        this.setInternetStations(r);
      })
      .finally(() => {});
  }

  setInternetStations(data) {
    // console.warn('IS', data);
    this.mergeState({ internetStations: data });
  }

  playInternetStation(data) {
    var station = {
      title: data.name,
      artist: '',
      album: data.homePageUrl,
      id: data.id,
      src: data.streamUrl
    };
    this._storeCurrentQueue([station]);
  }

  // ---------------------- NOW PLAYING -----------------------
  getNowPlaying() {
    SubsonicSource.getNowPlaying()
      .then(r => {
        this.returnNowPlaying(r);
      })
      .finally(() => {});
  }

  returnNowPlaying(np) {
    this.mergeState({
      nowPlaying: np.entry
        ? !Array.isArray(np.entry)
          ? [np.entry]
          : np.entry
        : []
    });
  }

  // ------------------------  VIDEOS -----------------------
  getVideos() {
    this.mergeState({ isBusy: true });
    SubsonicSource.fetchVideos()
      .then(r => {
        this.retrievedVideos(r);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
  }

  retrievedVideos(res) {
    // console.log(res);
    this.mergeState({ videos: res });
  }

  // ---------------------- PLAYLISTS -----------------------

  refreshPlaylists() {
    // console.log(SubsonicSource);
    SubsonicSource.refreshPlaylists()
      .then(r => {
        // this needs to post through the action so that RemotesStore gets it.
        SubsonicActions.retrievedPlaylists(r);
      })
      .finally(() => {});
    this.mergeState({ playlist: null, sortRule: 'unsorted' });
  }

  retrievedPlaylists({ allPlaylists, playlists, stations }) {
    // console.log("retrieved", arguments);
    allPlaylists = allPlaylists || [];
    playlists = playlists || [];
    stations = stations || [];

    this.mergeState({
      allPlaylists: allPlaylists,
      playlists: playlists,
      stations: stations,
      sortRule: 'unsorted'
    });
  }

  // ------------------------ PLAYLIST --------------------
  selectPlaylist([id, mode]) {
    console.log('store selectPlaylist', id, mode);
    const shuffle = mode === 'shuffle';
    const loadOnly = mode === 'list';

    var playlist = this.state.allPlaylists.find(function(p) {
      return p.id + '' === id + '';
    });
    if (playlist) {
      this.mergeState({
        playlist: playlist,
        shufflePlaylist: shuffle,
        isBusy: true
      });
      SubsonicSource.fetchPlaylist(playlist.id)
        .then(r => {
          this.populatePlaylist([r, loadOnly]);
        })
        .finally(() => {
          this.mergeState({ isBusy: false });
        });
    }
  }

  populatePlaylist([{ playlist, entries }, loadOnly]) {
    console.warn(
      'populate',
      playlist,
      entries,
      this.state.shufflePlaylist,
      loadOnly
    );
    let playlistData = entries || [];
    const shufflePlaylist = this.state.shufflePlaylist;

    this.mergeState({
      playlistInfo: playlist,
      playlistEntries: playlistData,
      sortRule: 'unsorted',
      playlistNeedsSave: false
    });
    if (shufflePlaylist) {
      playlistData = arrayShuffle(playlistData);
    }
    if (!loadOnly) this._storeCurrentQueue(entries);
  }

  // these two *may* be used here, but i'm more likely to move
  // them over to the editor store instead

  // properties set by binding
  sortFunction(prop, dir, prop2, prop3, a, b) {
    const av = a[prop] || '';
    const bv = b[prop] || '';
    if (av < bv) return dir === 'asc' ? -1 : 1;
    if (av > bv) return dir === 'asc' ? 1 : -1;
    return prop2 ? this.sortFunction(prop2, dir, prop3, null, a, b) : 1;
  }

  sortPlaylist(rule) {
    let playlistData = this.state.playlistEntries;
    if (rule === 'unsorted' || !playlistData) {
      this.mergeState({ sortRule: rule, playlistNeedsSave: true });
      return;
    }
    const regex = /(.*) (.*)/;
    if (rule === 'shuffle') {
      playlistData = arrayShuffle(playlistData);
    } else {
      const a = regex.exec(rule);
      const prop = a[1];
      const dir = a[2];
      let prop2 = null;
      let prop3 = null;
      if (prop === 'album') {
        prop2 = 'track';
      }
      if (prop === 'artist') {
        prop2 = 'album';
        prop3 = 'track';
      }
      playlistData = playlistData.sort(
        this.sortFunction.bind(this, prop, dir, prop2, prop3)
      );
    }
    this.mergeState({
      playlistEntries: playlistData,
      sortRule: rule === 'shuffle' ? 'unsorted' : rule,
      playlistNeedsSave: true
    });
  }

  // ------------------------ BROWSING PODCASTS --------------------

  fetchPodcasts() {
    this.mergeState({ isBusy: true });
    SubsonicSource.fetchPodcasts()
      .then(r => {
        this.returnPodcasts(r);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
  }

  returnPodcasts(channels) {
    // console.warn('isBusy false coming podcasts', channels);
    this.mergeState({
      podcasts: channels || []
    });
  }

  playPodcast(stream) {
    // massage stream into 'song' format
    var pc = Object.assign({}, stream);
    pc.id = stream.streamId;
    this.startPlaying([pc, 0]);
  }

  // ----------------------- BROWSING ALBUM QUERY ----------------------

  setAlbumQuery([query, id3]) {
    var s = this.state;
    // if (s.albumQuery === query && s.albums.length !== 0) return;
    s.albumQuery = query;
    s.albumPage = 0;
    s.albums = [];
    s.albumID3 = id3; // now just a setting for loader and ui to use, not an internal state
    this.mergeState(s);
    this.fetchAlbumPage([0, id3]);

    localStorage.setItem('initAlbumID3', id3);
    localStorage.setItem('initAlbumQuery', query || 'newest');
  }

  fetchAlbumPage([page, id3]) {
    console.log('fetching page', page);
    var s = this.state;
    s.albumPage = page;

    var offset = s.albumPage * s.albumsPerPage;
    var limit = s.albumsPerPage;

    var folder = s.musicFolder || -1;
    if (s.albumQuery.startsWith('pl-')) {
      var pl = s.allPlaylists.find(function(p) {
        return (p.id = s.albumQuery.substr(3));
      });
      if (!pl) return;
      this.mergeState({ isBusy: true });
      SubsonicSource.fetchPlaylistAsAlbums(pl, offset, limit, folder, id3)
        .then(r => {
          this.returnAlbumPage(r);
        })
        .finally(() => {
          this.mergeState({ isBusy: false });
        });
    } else {
      this.mergeState({ isBusy: true });
      SubsonicSource.fetchAlbums(s.albumQuery, offset, limit, folder, id3)
        .then(r => {
          this.returnAlbumPage(r);
        })
        .finally(() => {
          this.mergeState({ isBusy: false });
        });
    }

    // pause fetching
    s.hasMoreAlbums = false;
    this.setState(s);
  }

  returnAlbumPage(albums) {
    // console.warn(albums);
    if (Array.isArray(albums)) {
      albums = {
        albumList: albums
      };
    }
    var s = this.state;
    s.albums = s.albums.concat(albums.albumList);
    s.hasMoreAlbums = albums.albumList.length > 0 && s.albums.length < 400;
    this.setState(s);
  }

  setMusicDirectory(id) {
    // console.log('setMusicDirectory', id);
    if (id) {
      this.mergeState({ isBusy: true });
      SubsonicSource.fetchMusicDirectory(id)
        .then(r => {
          this.retrievedMusicDirectory(r);
        })
        .finally(() => {
          this.mergeState({ isBusy: false });
        });
    }
    this.mergeState({
      musicDirectoryId: id,
      musicDirectoryContents: null,
      isBusy: true
    });
  }

  playMusicDirectorySongs([musicDirectory, type = 'all']) {
    console.log('playMusicDirectorySongs', musicDirectory, type);
    if (typeof musicDirectory !== 'object') {
      musicDirectory = {
        id: musicDirectory,
        name: musicDirectory
      };
    }
    this.mergeState({ isBusy: true });
    SubsonicSource.playMusicDirectorySongs(musicDirectory, type)
      .then(r => {
        this._storeCurrentQueue(r);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
  }

  retrievedMusicDirectory({ directory, children }) {
    // console.log('retrievedMusicDirectory', dir, items, play);
    // TODO: music directory browsing when play is false
    const play = false;
    if (play) {
      children = children.filter(i => {
        return !i.isDir;
      });
      this._storeCurrentQueue(children);
    } else {
      this.mergeState({
        musicDirectory: directory,
        musicDirectoryContents: children,
        isBusy: false
      });
    }
  }

  playSong(id) {
    this.setState({ isBusy: true });
    SubsonicSource.fetchSong(id)
      .then(r => {
        this.retrievedSong(r);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
  }

  retrievedSong(song) {
    this._storeCurrentQueue([song]);
  }

  // ------------------------ BROWSING ARTISTS/ALBUMS --------------------
  setAllArtists(artists) {
    console.log('setAllArtists', artists);
    this.mergeState({ allArtistIndexes: artists });
  }

  setArtists(artistIndexes) {
    console.log('setArtists', artistIndexes);
    artistIndexes = artistIndexes || [];
    if (!Array.isArray(artistIndexes)) artistIndexes = [artistIndexes];
    var artists = [];
    for (var i = 0; i < artistIndexes.length; ++i) {
      artists = artists.concat(artistIndexes[i].artist);
    }
    this.mergeState({
      artistIndexes: artistIndexes,
      artists: artists,
      artistIndex: null,
      artist: null,
      artistAlbums: null
    });
  }

  setArtistIndex(index) {
    console.log('new index', index);
    this.mergeState({ artistIndex: index, artist: null, artistAlbums: null });
  }

  setArtist(artist) {
    // console.warn('setArtist', artist);
    this.mergeState({ artist: artist, artistAlbums: null, isBusy: true });
    if (artist) {
      this.mergeState({ isBusy: true });
      SubsonicSource.fetchArtist(artist.id)
        .then(r => {
          this.setArtistAlbums(r);
        })
        .finally(() => {
          this.mergeState({ isBusy: false });
        });
    }
  }

  setArtistAlbums(artistAlbums) {
    // console.warn('back, setting busy false');
    this.mergeState({
      artist: artistAlbums,
      artistAlbums: artistAlbums,
      isBusy: false
    });
  }

  setAndPlayArtistSongs([artist, type = 'all']) {
    this.setState({ isBusy: true });
    SubsonicSource.playArtistSongs(artist, type)
      .then(r => {
        this._storeCurrentQueue(r);
      })
      .finally(() => {
        this.setState({ isBusy: false });
      });
  }

  playArtistSongs(type = 'all') {
    this.setAndPlayArtistSongs([this.state.artist, type]);
  }

  loadAlbum([id, id3]) {
    var s = this.state;
    if (id3) {
      this.setAlbum(id);
    } else {
      this.setMusicDirectory(id);
    }
  }

  setAlbum(id) {
    if (id) {
      this.mergeState({ isBusy: true });
      SubsonicSource.fetchAlbum(id)
        .then(r => {
          this.retrievedAlbum(r);
        })
        .finally(() => {
          this.setState({ isBusy: false });
        });
    }
  }

  retrievedAlbum(album) {
    console.log('retrievedAlbum', album);
    var albumsById = this.state.albumsById;
    albumsById[album.id + ''] = album;
    this.mergeState({
      albumsById: albumsById
    });
  }

  // ------------------------ BROWSING MUSIC FOLDERS --------------------

  shuffleMusicFolder(folder) {
    console.log('event shuffleMusicFolder', folder, this.state.musicFolder);
    folder = folder || this.state.musicFolder || -1;
    this.mergeState({ isBusy: true });
    SubsonicSource.fetchRandomSongs(folder)
      .then(r => {
        this._storeCurrentQueue(r);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
  }

  fetchMusicFolders() {
    this.mergeState({ isBusy: true });
    SubsonicSource.fetchMusicFolders()
      .then(r => {
        this.setMusicFolders(r);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
  }

  setMusicFolders(musicFolders) {
    console.log('musicFolders', musicFolders);
    // TODO:[JS] add all music folders to the top
    const defaultFolder = {
      id: -1,
      name: 'All Music Folders'
    };
    musicFolders.unshift(defaultFolder);
    this.mergeState({
      musicFolders: musicFolders,
      musicFolder: this.state.musicFolder || defaultFolder.id
    });
  }

  setMusicFolder(musicFolder) {
    console.log('musicFolder', musicFolder);

    this.mergeState({
      musicFolder: musicFolder,
      musicDirectoryId: null,
      musicDirectoryContents: null,
      isBusy: true
    });
    // refresh the current state to the new folder selection
    SubsonicSource.fetchArtists(this.state.musicFolder || null)
      .then(res => {
        this.setArtists(res.indexes || []);
        return SubsonicSource.fetchIndexes(this.state.musicFolder || null);
      })
      .then(res => {
        this.setMusicDirectoryIndexes(res.indexes || []);
        SubsonicActions.setAlbumQuery([
          this.state.albumQuery,
          this.state.albumID3
        ]);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
    localStorage.setItem('initMusicFolder', musicFolder || -1);
  }

  setSearchMusicFolder(musicFolder) {
    console.log('searchMusicFolder', musicFolder);
    this.mergeState({ searchMusicFolder: musicFolder });
  }

  setMusicDirectoryIndexes(musicDirectoryIndexes) {
    let musicDirectories = [];
    if (!Array.isArray(musicDirectoryIndexes)) {
      musicDirectoryIndexes = [musicDirectoryIndexes];
    }
    for (let mdi of musicDirectoryIndexes) {
      musicDirectories = musicDirectories.concat(mdi.artist);
    }
    this.mergeState({
      musicDirectoryIndexes: musicDirectoryIndexes,
      musicDirectoryIndex: null,
      musicDirectories: musicDirectories
    });
  }

  setMusicDirectoryIndex(index) {
    this.mergeState({ musicDirectoryIndex: index });
  }

  appendToPlayQueue(entry) {
    const currentPlaying = Object.assign({}, this.state.currentPlaying);
    if (!Array.isArray(entry)) entry = [entry];
    if (
      currentPlaying.queue.length === 2 &&
      currentPlaying.queue[1] === emptytrack
    ) {
      currentPlaying.queue.pop(0);
    }
    currentPlaying.queue = currentPlaying.queue.concat(entry);
    if (currentPlaying.queue.length === 1)
      currentPlaying.queue = currentPlaying.queue.concat([emptytrack]);
    this.mergeState({ currentPlaying: currentPlaying });
  }

  // ------------- server play queue --------------
  fetchPlayQueue() {
    this.mergeState({ isBusy: true });
    console.log('getPlayQueue');
    SubsonicSource.fetchPlayQueue()
      .then(r => {
        console.warn('fpq', r);
        window.bookmarkSeekTo = r.position / 1000;
        this._storeCurrentQueue(r.entry);
        this.mergeState({ playQueue: r });
      })
      .finally(() => {
        console.warn('fpq finally');
        this.mergeState({ isBusy: false });
      });
  }

  savePlayQueue(currentMediaTime) {
    const { queue, idx } = this.state.currentPlaying;
    SubsonicSource.savePlayQueue(queue, idx, currentMediaTime).then(r => {
      SubsonicActions.snack.defer('Queue Saved', {
        variant: 'success',
        autoHideDuration: 3000
      });
    });
  }

  // ------------- play queue ------------
  startPlayingCurrentQueue() {
    this.startPlaying([this.state.currentQueue || [emptytrack], 0]);
  }

  _storeCurrentQueue(entry) {
    console.warn('_storeCurrentQueue', entry);
    this.mergeState({ currentQueue: entry });
  }

  startPlaying([queue, current]) {
    console.warn('startPlaying', queue, current);
    const currentPlaying = this.state.currentPlaying;
    if (!Array.isArray(queue)) queue = [queue];
    if (queue.length === 1) queue.push(emptytrack);
    currentPlaying.queue = queue;

    let idx =
      typeof current === 'object'
        ? current.id
          ? current.id
          : 0
        : parseInt(current, 10);
    for (let i = 0; i < queue.length; ++i) {
      // console.log(i, pl[i].id, c, pl[i].id === c+"");
      if (queue[i].id + '' === idx + '') {
        idx = i;
        break;
      }
    }

    currentPlaying.idx = idx || 0;
    console.log('setting via start sp');
    this.mergeState({ currentPlaying: currentPlaying });
  }

  playerNext() {
    const currentPlaying = this.state.currentPlaying;
    currentPlaying.idx++;

    console.log(
      'playerNext',
      currentPlaying.idx,
      currentPlaying.queue.length,
      currentPlaying
    );

    if (currentPlaying.idx >= currentPlaying.queue.length)
      currentPlaying.idx = 0;
    this.mergeState({ currentPlaying: currentPlaying });
  }

  playerSkip() {
    const currentPlaying = this.state.currentPlaying;
    const a = currentPlaying.queue[currentPlaying.idx].albumId;
    const q = currentPlaying.queue;
    do {
      currentPlaying.idx++;
    } while (
      q[currentPlaying.idx] &&
      q[currentPlaying.idx].albumId === a &&
      currentPlaying.idx < q.length
    );
    if (!q[currentPlaying.idx] || currentPlaying.idx > q.length)
      currentPlaying.idx = 0;
    this.mergeState({ currentPlaying: currentPlaying });
  }

  playerSkipToTrack(entry) {
    const { bookmarkId, position } = entry;
    const id = bookmarkId || entry.id;
    const currentPlaying = this.state.currentPlaying;
    const q = currentPlaying.queue;
    let match = false;
    for (let i = 0; i < q.length; ++i) {
      if (id * 1 === q[i].id * 1) {
        currentPlaying.idx = i;
        match = true;
      }
    }
    if (
      !match &&
      this.state.bookmarksById &&
      this.state.bookmarksById[bookmarkId]
    ) {
      entry = this.state.bookmarksById[bookmarkId].entry;
      q.unshift(entry);
      currentPlaying.idx = 0;
    }
    this.mergeState({ currentPlaying: currentPlaying });
    // hack - need to pass this through, once it is loaded
    window.bookmarkSeekTo = parseInt(position || '0', 10) / 1000;
  }

  playerSkipToIndex(idx) {
    const currentPlaying = this.state.currentPlaying;
    currentPlaying.idx = idx;
    this.mergeState({ currentPlaying: currentPlaying });
  }

  playerPrev() {
    const currentPlaying = this.state.currentPlaying;
    currentPlaying.idx--;
    if (currentPlaying.idx === -1)
      currentPlaying.idx = currentPlaying.queue.length - 1;
    this.mergeState({ currentPlaying: currentPlaying });
  }

  stopPlaying() {
    // const currentPlaying = this.state.currentPlaying;
    // currentPlaying.queue = [];
    // this.mergeState({currentPlaying: currentPlaying});
  }

  shufflePlaying() {
    const currentPlaying = this.state.currentPlaying;
    currentPlaying.queue = arrayShuffle(currentPlaying.queue);
    currentPlaying.idx = 0;
    this.mergeState({ currentPlaying: currentPlaying });
  }

  setBookmarkContext(context) {
    // console.log('bookmark context', context);
    this.mergeState({ bookmarkContext: context });
  }

  createBookmark([id, position]) {
    // assemble context as
    const context = Object.assign({}, this.state.bookmarkContext);
    const comment = JSON.stringify(context);
    SubsonicSource.createBookmark(id, position, comment).then(() => {
      SubsonicActions.snack.defer('Bookmark Saved', {
        variant: 'success',
        autoHideDuration: 3000
      });
      this.getBookmarks();
    });
  }

  deleteBookmark(id) {
    SubsonicSource.deleteBookmark(id).then(() => {
      this.getBookmarks();
    });
  }

  getBookmarks() {
    this.mergeState({ isBusy: true });
    SubsonicSource.getBookmarks()
      .then(res => {
        this.storeBookmarks(res);
      })
      .finally(() => {
        this.mergeState({ isBusy: false });
      });
  }

  storeBookmarks(bookmarks) {
    console.log('bookmarks', bookmarks, typeof bookmarks);
    bookmarks = bookmarks || [];
    if (!Array.isArray(bookmarks)) bookmarks = [bookmarks];
    const bookmarksById = bookmarks.reduce((bmi, b, i, a) => {
      //console.log('reduce', bmi, 'b', b, 'i', i, 'a', a);
      bmi[b.entry.id] = b;
      return bmi;
    }, {});
    //console.log(bookmarks, bookmarksById);
    this.mergeState({
      bookmarks: bookmarks,
      bookmarksById: bookmarksById || {}
    });
  }

  setOffline(offline) {
    this.mergeState({ offline: offline });
  }

  setLoadingTitle(title) {
    this.mergeState({ lastLoadingTitle: title });
  }

  doSearch({ searchType, query, nextPage, pageSize, id3 }) {
    if (!nextPage) {
      this.mergeState({
        searchArtistResults: [],
        searchAlbumResults: [],
        searchSongsResults: []
      });
    }
    const params = {
      artistCount: searchType === 'artist' ? pageSize : 0,
      artistOffset: searchType === 'artist' ? pageSize * nextPage : 0,
      albumCount: searchType === 'album' ? pageSize : 0,
      albumOffset: searchType === 'album' ? pageSize * nextPage : 0,
      songCount: searchType === 'song' ? pageSize : 0,
      songOffset: searchType === 'song' ? pageSize * nextPage : 0
    };
    const musicFolderId = this.state.searchMusicFolder;
    if (musicFolderId && musicFolderId + '' !== '-1')
      params.musicFolderId = musicFolderId;
    SubsonicSource.search(query, id3, params)
      .then(r => {
        this.searchResults(r);
      })
      .finally(() => {});
  }

  searchResults(res) {
    console.warn(res);
    this.mergeState({
      searchArtistResults: this.state.searchArtistResults.concat(res.artist),
      searchAlbumResults: this.state.searchAlbumResults.concat(res.album),
      searchSongsResults: this.state.searchSongsResults.concat(res.song)
    });
  }

  persistQueueLocally() {
    window.localStorage.setItem(
      'subsonic.lastPlaying',
      JSON.stringify(this.state.currentPlaying)
    );
    window.localStorage.setItem(
      'subsonic.lastPlaying.currentMediaTime',
      '' + this.state.currentMediaTime
    );
  }

  setSnackerFunction(f) {
    // console.warn('snacker-f', f);
    this.mergeState(f);
  }

  snack([message, options]) {
    // console.log('snack', message, options, this.state.snack);
    // varient = varient || {};
    // varient.autoHideDuration = 50000;

    if (this.state.snack) {
      this.state.snack(message, options);
    }
  }
}

SubsonicStore = alt.createStore(SubsonicStoreClass, 'SubsonicStore');
export default SubsonicStore;
